@@ -4,7 +4,6 @@
 #include "screem-page.h"
 #include "screem-site.h"
 
-#include "screem-application.h"
 #include "screem-window.h"
 #include "screem-editor.h"
 
@@ -17,13 +16,13 @@ enum {
 gboolean screem_page_open_with_filename( ScreemSite *site, 
 					 ScreemWindow *window,
 					 const gchar *filename );
-gint screem_page_save_as( ScreemPage *page, ScreemWindow *window );
-gint screem_page_save_confirmation( ScreemPage *page, gboolean closing );
+gint screem_page_save_as( ScreemPage *page, ScreemWindow *window,
+		const gchar *dir, ScreemApplication *app );
+gint screem_page_save_confirmation( ScreemPage *page, 
+		ScreemWindow *window, gboolean closing );
 void screem_page_create_blank_page( ScreemSite *site );
 gboolean screem_page_close_page( ScreemPage *page, ScreemWindow *window,
 				gboolean confirm );
-void screem_page_druid_begin( ScreemApplication *application,
-			      ScreemWindow *window );
 void build_recent_pages_menu( void );
 
 gboolean screem_page_save_check( ScreemPage *page );
@@ -35,4 +34,10 @@ void screem_page_edit_bookmarks( ScreemWindow *window, ScreemEditor *editor,
 				 ScreemPage *page );
 
 
+gint screem_page_save_page_confirmation( ScreemPage *page,
+		ScreemWindow *window, ScreemApplication *app );
+gint screem_page_save_pages_confirmation( GSList *pages,
+		ScreemWindow *window, ScreemApplication *app );
+
+
 #endif
