@@ -18,6 +18,10 @@
 
 #include "screem-dtd-db.h"
 
+#include "screem-plugin-manager.h"
+#include "screem-macro-manager.h"
+#include "screem-helper-manager.h"
+#include "screem-icon-cache.h"
 #include "screem-session.h"
 
 #define SCREEM_TYPE_APPLICATION            (screem_application_get_type ())
@@ -30,11 +34,6 @@
 typedef struct ScreemWindow ScreemWindow;
 #endif
 
-#ifndef SCREEM_PLUGIN
-#define SCREEM_PLUGIN
-typedef struct ScreemPlugin ScreemPlugin;
-#endif
-
 typedef struct ScreemApplication ScreemApplication;
 typedef struct ScreemApplicationPrivate ScreemApplicationPrivate;
 
@@ -43,12 +42,6 @@ struct ScreemApplication {
 	
 	ScreemApplicationPrivate *priv;
 	
-	GList *plugins;
-
-	GList *helpers;
-
-	GList *macros;
-
 	gboolean offline;
 	gint offline_notify;
 
@@ -68,6 +61,8 @@ ScreemApplication *screem_application_new( void );
 void screem_application_startup( ScreemApplication *app,
 				 const gchar *session,
 				 const gchar **files );
+void screem_application_add_files( ScreemApplication *app,
+		GSList *files );
 void screem_application_close( ScreemApplication *app );
 
 GList *screem_application_get_window_list( ScreemApplication *app );
@@ -86,24 +81,9 @@ void screem_application_close_site( ScreemApplication *app, ScreemSite *site );
 void screem_application_close_all_sites( ScreemApplication *app );
 gboolean screem_application_save_sites( ScreemApplication *app );
 
-void screem_dtd_fill_combo( ScreemApplication *application, GtkWidget *combo );
-void screem_dtd_fill_list( ScreemApplication *application, GtkListStore *store );
-
-void screem_application_register_plugin( ScreemApplication *application, 
-					 ScreemPlugin *plugin );
-
-GtkSourceLanguagesManager* screem_application_load_syntax_tables();
-
-void screem_application_add_helper( ScreemApplication *application,
-				    ScreemHelper *helper );
-void screem_application_remove_helper( ScreemApplication *application,
-				       ScreemHelper *helper );
-
-
-void screem_application_add_macro( ScreemApplication *application,
-				   ScreemEditorMacro *macro );
-void screem_application_remove_macro( ScreemApplication *application,
-				      ScreemEditorMacro *macro );
+void screem_application_init_tag_styles( GtkSourceLanguage *lang );
+GtkSourceLanguagesManager* screem_application_load_syntax_tables( void );
+GHashTable *screem_application_get_features( ScreemApplication *application, const gchar *type );
 
 ScreemDTDDB *screem_application_get_dtd_db( ScreemApplication *application );
 
@@ -118,4 +98,11 @@ GtkTreeModel *screem_application_get_encodings( ScreemApplication *app );
 
 void screem_application_exec_helpers( ScreemApplication *app,
 		ScreemPage *page );
+
+ScreemPluginManager *screem_application_get_plugin_manager( ScreemApplication *app );
+ScreemMacroManager *screem_application_get_macro_manager( ScreemApplication *app );
+ScreemHelperManager *screem_application_get_helper_manager( ScreemApplication *app );
+
+ScreemIconCache *screem_application_get_icon_cache( ScreemApplication *app );
+
 #endif
