@@ -15,6 +15,14 @@
 
 typedef struct ScreemDTDDBPrivate ScreemDTDDBPrivate;
 
+typedef enum {
+	SCREEM_DTD_DB_PUBLIC_COL,
+	SCREEM_DTD_DB_SYSTEM_COL,
+	SCREEM_DTD_DB_ROOT_COL,
+	SCREEM_DTD_DB_ACTIVE_COL,
+	SCREEM_DTD_DB_MAX_COL
+} ScreemDTDDBStoreColumn;
+
 typedef struct  {
 	GObject parent;
 
@@ -31,7 +39,8 @@ GType screem_dtd_db_get_type( void );
 
 GtkListStore *screem_dtd_db_get_store( ScreemDTDDB *db );
 
-ScreemDTD *screem_dtd_db_get_default_dtd( ScreemDTDDB *db );
+ScreemDTD *screem_dtd_db_get_default_dtd( ScreemDTDDB *db,
+		const gchar *mime, const gchar *root );
 
 void screem_dtd_db_fill_list( ScreemDTDDB *db, GtkListStore *store );
 
@@ -49,12 +58,19 @@ gchar *screem_dtd_db_get_system_id( ScreemDTDDB *db,
 				const gchar *publicid,
 				gboolean *override );
 
+gchar *screem_dtd_db_get_mime_type( ScreemDTDDB *db,
+		const gchar *publicid );
+gboolean screem_dtd_db_get_match_root( ScreemDTDDB *db,
+		const gchar *publicid );
+void screem_dtd_db_set_mime_type( ScreemDTDDB *db,
+		const gchar *publicid, const gchar *mime );
+void screem_dtd_db_set_match_root( ScreemDTDDB *db,
+		const gchar *publicid, gboolean match );
 
-void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
-				 const gchar *doctype,
-				 gchar **publicid,
-				 gchar **systemid,
-				 gchar **root );
+
+const gchar *screem_dtd_db_parse_doctype( ScreemDTDDB *db,
+		const gchar *doctype,
+		gchar **publicid, gchar **systemid, gchar **root );
 
 ScreemDTD *screem_dtd_db_get_dtd_from_doctype( ScreemDTDDB *db,
 						const gchar *doctype );
@@ -66,5 +82,10 @@ gchar *screem_dtd_db_resolve_entity( void *db,
 				  const gchar *publicid,
 				  const gchar *systemid );
 
+GSList *screem_dtd_db_get_default_conflicts( ScreemDTDDB *db,
+	const gchar *publicid, const gchar *mime, gboolean rootmatch );
+
+gchar *screem_dtd_db_lookup_mime_type( ScreemDTDDB *db,
+		const gchar *mime, const gchar *root );
 
 #endif
