@@ -9,10 +9,10 @@
 #define SCREEM_DTD(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), SCREEM_TYPE_DTD, ScreemDTD))
 #define SCREEM_IS_DTD(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SCREEM_TYPE_DTD))
 				   
-typedef enum ScreemDTDClose {
-	SCREEM_DTD_SHOULD,
-	SCREEM_DTD_MUST,
-	SCREEM_DTD_MUST_NOT
+typedef enum {
+	SCREEM_DTD_SHOULD = 1,
+	SCREEM_DTD_MUST = 2,
+	SCREEM_DTD_MUST_NOT = 3
 } ScreemDTDTagExistance;
 
 typedef struct ScreemDTD ScreemDTD;
@@ -51,8 +51,8 @@ const gchar *screem_dtd_get_public_id( const ScreemDTD *dtd );
 const gchar *screem_dtd_get_system_id( const ScreemDTD *dtd );
 const gchar *screem_dtd_get_root_name( const ScreemDTD *dtd );
 
-void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
-			const gchar *data );
+gboolean screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
+		const gchar *data );
 
 gchar *screem_dtd_dump( const ScreemDTD *dtd );
 
@@ -98,8 +98,8 @@ const gchar *screem_dtd_element_get_description( const ScreemDTDElement *element
 
 const gchar *screem_dtd_attribute_get_name( const ScreemDTDAttribute *attr );
 const gchar *screem_dtd_attribute_get_description( const ScreemDTDAttribute *attr );
-const gboolean screem_dtd_attribute_get_required( const ScreemDTDAttribute *attr );
-const gboolean screem_dtd_attribute_is_uri( const ScreemDTDAttribute *attr );
+gboolean screem_dtd_attribute_get_required( const ScreemDTDAttribute *attr );
+gboolean screem_dtd_attribute_is_uri( const ScreemDTDAttribute *attr );
 const GSList *screem_dtd_attribute_get_values( const ScreemDTDAttribute *attr );
 const gchar *screem_dtd_attribute_get_default( const ScreemDTDAttribute *attr );
 
