@@ -8,6 +8,8 @@
 
 #include <libgnomevfs/gnome-vfs-monitor.h>
 
+#include "screem-icon-cache.h"
+
 #define SCREEM_TYPE_FILE_BROWSER  (screem_file_browser_get_type())
 #define SCREEM_FILE_BROWSER(obj)  (G_TYPE_CHECK_INSTANCE_CAST( ( obj ), \
                               SCREEM_TYPE_FILE_BROWSER, \
@@ -24,12 +26,12 @@ enum {
 	FILE_BROWSER_URI_COL,
 	FILE_BROWSER_MIME_COL,  /* pointer to const string, as
 				   there is no G_TYPE_CONST_STRING */
+	FILE_BROWSER_SHOW_COL,
 	FILE_BROWSER_MAX_COL
 };
 
 typedef enum {
 	FILE_BROWSE_NORMAL = 0,
-	FILE_BROWSE_FLAT = 1 << 1,
 	FILE_BROWSE_RECURSE = 1 << 2,
 	FILE_BROWSE_ROOT = 1 << 3
 } ScreemFileBrowserMode;
@@ -60,15 +62,10 @@ typedef struct {
 				const gchar *uri,
 				const gchar *mime_type,
 				GtkTreeIter *it );
-
-	/* fired when the theme changes for places where
-	 * a ScreemFileBrowser is being used just to get icons */
-	void (*theme_change)( ScreemFileBrowser *browser );
-	
 } ScreemFileBrowserClass;
 
 GType  screem_file_browser_get_type( void );
-ScreemFileBrowser *screem_file_browser_new();
+ScreemFileBrowser *screem_file_browser_new( ScreemIconCache *cache );
 
 /* set the default mode, passing -1 as the mode to other funcs
    causes this to be used */
@@ -81,9 +78,8 @@ GtkTreeModel *screem_file_browser_get_model( ScreemFileBrowser *browser );
 void screem_file_browser_scan_directory( ScreemFileBrowser *browser,
 					 const gchar *uri,
 					 int mode );
-const gchar* screem_file_browser_scan_iter( ScreemFileBrowser *browser,
-					    GtkTreeIter *iter,
-					    int mode );
+void screem_file_browser_scan_iter( ScreemFileBrowser *browser,
+		GtkTreeIter *iter, int mode );
 
 /* returns a list of filenames from the given list of iterators */
 GList *screem_file_browser_get_pathnames( ScreemFileBrowser *browser,
@@ -94,17 +90,20 @@ void screem_file_browser_set_sort_func( ScreemFileBrowser *browser,
 					GtkTreeIterCompareFunc func,
 					gpointer data );
 
-GdkPixbuf *screem_file_browser_get_icon( ScreemFileBrowser *browser,
-					 const gchar *uri,
-					 gint width, gint height,
-					 gboolean ignore_cache,
-					 gchar **mtype );
-
 void screem_file_browser_file_mod( ScreemFileBrowser *browser,
 				const gchar *directory,
 				const gchar *file,
 				GnomeVFSMonitorEventType type );
 
+
+void screem_file_browser_set_show_hidden( ScreemFileBrowser *browser,
+		gboolean show );
+void screem_file_browser_set_show_backups( ScreemFileBrowser *browser,
+		gboolean show );
+/* override default backup file suffix of ~ */
+void screem_file_browser_set_backup_suffix( ScreemFileBrowser *browser,
+		const gchar *suffix );
+
 /* only call on app exit */
 void screem_file_browser_cleanup( void );
 				
