@@ -33,7 +33,7 @@ typedef enum {
 typedef struct {
 	GObject parent;
 
-	ScreemHelperPrivate *private;
+	ScreemHelperPrivate *priv;
 } ScreemHelper;
 
 typedef struct {
@@ -64,4 +64,6 @@ void screem_helper_add_to_window( ScreemHelper *helper,
 void screem_helper_remove_from_window( ScreemHelper *helper,
 				       ScreemWindow *window );
 
+gchar *screem_helper_get_uiname( ScreemHelper *helper );
+
 #endif
