@@ -1,8 +1,6 @@
 #ifndef SCREEM_HINT_H
 #define SCREEM_HINT_H
 
-#include <glade/glade.h>
-
 #include <gtk/gtkwindow.h>
 
 #define SCREEM_TYPE_HINT  (screem_hint_get_type())
@@ -16,7 +14,6 @@
 typedef struct {
 	GtkWindow parent;
 
-	GladeXML *xml;
 	GtkWidget *label;
 
 	gboolean show_hint;
