@@ -25,7 +25,7 @@ typedef struct ScreemLinkViewPrivate ScreemLinkViewPrivate;
 typedef struct {
 	ScreemView parent;
 
-	ScreemLinkViewPrivate *private;
+	ScreemLinkViewPrivate *priv;
 } ScreemLinkView;
 
 typedef struct {
