@@ -16,7 +16,7 @@ typedef struct ScreemPageModelPrivate ScreemPageModelPrivate;
 typedef struct  {
 	GtkTreeStore parent;
 
-	ScreemPageModelPrivate *private;
+	ScreemPageModelPrivate *priv;
 } ScreemPageModel;
 
 typedef struct {
@@ -25,7 +25,8 @@ typedef struct {
 } ScreemPageModelClass;
 
 ScreemPageModel *screem_page_model_new( ScreemPage *page, GObject *app );
-void screem_page_model_force_check( ScreemPageModel *model );
+void screem_page_model_force_check( ScreemPageModel *model,
+		gboolean in_progress );
 
 void screem_page_model_build_model( ScreemPageModel *model );
 void screem_page_model_ensure_built( ScreemPageModel *model );
