@@ -26,7 +26,7 @@ typedef struct ScreemPreviewPrivate ScreemPreviewPrivate;
 typedef struct {
 	ScreemView parent;
 
-	ScreemPreviewPrivate *private;
+	ScreemPreviewPrivate *priv;
 } ScreemPreview;
 
 typedef struct {
@@ -45,6 +45,4 @@ void screem_preview_display( ScreemPreview *preview, const gchar *text,
 void screem_preview_uri( ScreemPreview *preview, const gchar *uri,
 			 gboolean online );
 
-void screem_preview_print( ScreemView *view, gboolean preview );
-
 #endif
