@@ -14,7 +14,9 @@ void screem_site_close_site( ScreemSite *site );
 
 void screem_site_create_template( ScreemWindow *window );
 
-gboolean screem_site_save_confirmation( ScreemSite *site );
+gboolean screem_site_save_confirmation( ScreemSite *site,
+		ScreemWindow *window );
 
+void screem_site_stats_dialog( ScreemSite *site, ScreemWindow *window );
 
 #endif
