@@ -30,8 +30,7 @@ typedef enum  {
         LOCAL = 0,
         FTP = 1,
 	DAV = 2,
-        RSH = 3,
-	SSH = 4,
+	SSH = 3,
 	UPLOAD_OTHER = 5
 } UploadMethods;
 
@@ -47,12 +46,13 @@ typedef struct ScreemSitePrivate ScreemSitePrivate;
 typedef struct {
         GObject parent;
      
-	ScreemSitePrivate *private;
+	ScreemSitePrivate *priv;
 } ScreemSite;
 
 typedef struct {
         GObjectClass parent_class;
 
+		void (*ctags_running)( ScreemSite *site );
 } ScreemSiteClass;
 
 GType       screem_site_get_type(void);
@@ -108,9 +108,6 @@ const gchar* screem_site_get_http_url( ScreemSite *site );
 
 gboolean screem_site_get_fake_flag( ScreemSite *site );
 
-void screem_site_set_passive_ftp( ScreemSite *site, gboolean flag );
-gboolean screem_site_get_passive_ftp( ScreemSite *site );
-
 void screem_site_set_no_delete( ScreemSite *site, gboolean flag );
 gboolean screem_site_get_no_delete( ScreemSite *site );
 
@@ -129,22 +126,14 @@ ScreemSiteSymlinks screem_site_get_symlinks( ScreemSite *site );
 void screem_site_set_is_import( ScreemSite *site, gboolean val );
 gboolean screem_site_get_is_import( ScreemSite *site );
 
-GList *screem_site_get_auto_open( ScreemSite *site );
-
 gboolean screem_site_is_excluded( ScreemSite *site, const gchar *path );
 void screem_site_add_exclude( ScreemSite *site, const gchar *pattern );
 void screem_site_remove_exclude( ScreemSite *site, const gchar *pattern );
 
-gboolean screem_site_is_ascii( ScreemSite *site, const gchar *path );
-void screem_site_add_ascii( ScreemSite *site, const gchar *pattern );
-void screem_site_remove_ascii( ScreemSite *site, const gchar *pattern );
-
 gboolean screem_site_is_ignored( ScreemSite *site, const gchar *path );
 void screem_site_add_ignore( ScreemSite *site, const gchar *patterm );
 void screem_site_remove_ignore( ScreemSite *site, const gchar *pattern );
 
-void screem_site_add_auto_open( ScreemSite *site, const gchar *file );
-
 void screem_site_save( ScreemSite *site );
 gboolean screem_site_create( ScreemSite *site );
 gboolean screem_site_load( ScreemSite *site );
@@ -156,12 +145,17 @@ ScreemPage* screem_site_locate_page( ScreemSite *site, const gchar *path );
 gboolean screem_site_add_page( ScreemSite *site, ScreemPage *page );
 void screem_site_remove_page_with_path( ScreemSite *site, const gchar *path );
 void screem_site_remove_page( ScreemSite *site, ScreemPage *page );
+
 /* use screem_site_foreach_page() in preference to
  * screem_site_get_pages() as the later builds a new list while
- * the foreach method just goes through each item in a hash table */
-GList *screem_site_get_pages( ScreemSite *site );
+ * the foreach method just goes through each item in a hash table,
+ * and takes notice of excludes / ignores if required*/
+
+GList *screem_site_get_pages( ScreemSite *site, gboolean excludes,
+		gboolean ignores );
 void screem_site_foreach_page( ScreemSite *site,
-		GFunc func, gpointer user_data );
+		GFunc func, gboolean excludes, gboolean ignores,
+		gpointer user_data );
 
 GObject *screem_site_get_todo( ScreemSite *site );
 
@@ -174,7 +168,6 @@ GList* screem_site_get_documents( ScreemSite *site, const gchar *name );
 
 GList* screem_site_get_excludes( ScreemSite *site );
 GList* screem_site_get_ignores( ScreemSite *site );
-GList* screem_site_get_asciis( ScreemSite *site );
 
 /* function pointer type for getting the file upload status' from
    the upload wizard */
