@@ -15,7 +15,7 @@ typedef struct ScreemSpellPrivate ScreemSpellPrivate;
 typedef struct {
 	GObject parent;
 
-	ScreemSpellPrivate *private;
+	ScreemSpellPrivate *priv;
 } ScreemSpell;
 
 typedef struct {
