@@ -21,6 +21,13 @@ typedef enum {
 } ScreemTagFileFormat;
 #endif
 
+enum {
+	SCREEM_TAG_FILE_ICON,
+	SCREEM_TAG_FILE_NAME,
+	SCREEM_TAG_FILE_DATA,
+	SCREEM_TAG_FILE_COLS
+};
+
 enum {
 	TAG_FILE_NAME_COL = 0,
 	TAG_FILE_URI_COL,
@@ -75,9 +82,9 @@ typedef struct {
 GType screem_tag_file_get_type( void );
 ScreemTagFile* screem_tag_file_new( void );
 
-gboolean screem_tag_file_load( ScreemTagFile *tagfile,
+void screem_tag_file_load( ScreemTagFile *tagfile,
 		ScreemTagFileFormat format );
-gboolean screem_tag_file_load_with_uri( ScreemTagFile *tagfile,
+void screem_tag_file_load_with_uri( ScreemTagFile *tagfile,
 					const gchar *uri,
 					ScreemTagFileFormat format );
 
