@@ -19,7 +19,9 @@ typedef struct ScreemTagTreePrivate ScreemTagTreePrivate;
 #ifndef _SCREEM_TAG_FILE_FORMAT
 #define _SCREEM_TAG_FILE_FORMAT
 typedef enum {
-	SCREEM_TAG_TREE_FORMAT
+	SCREEM_TAG_UNKNOWN_FORMAT,
+	SCREEM_TAG_TREE_FORMAT,
+	SCREEM_TAG_WEML_FORMAT
 } ScreemTagFileFormat;
 #endif
 
@@ -46,6 +48,6 @@ gboolean screem_tag_tree_add_uri( const gchar *uri,
 void screem_tag_tree_remove_uri( const gchar *uri );
 
 GSList *screem_tag_tree_autocomplete( const gchar *mime_type,
-					const gchar *prefix );
+		const gchar *prefix, gboolean fortip );
 
 #endif
