@@ -18,7 +18,7 @@ typedef struct ScreemTreeViewPrivate ScreemTreeViewPrivate;
 struct ScreemTreeView {
 	ScreemView parent;
 
-	ScreemTreeViewPrivate *private;
+	ScreemTreeViewPrivate *priv;
 };
 
 typedef struct {
@@ -32,7 +32,7 @@ typedef struct {
 GType screem_tree_view_get_type( void );
 ScreemTreeView *screem_tree_view_new( ScreemWindow *window );
 
-gchar *screem_tree_view_query_element_context( ScreemTreeView *view,
+gchar *screem_tree_view_query_elemeint_context( ScreemTreeView *view,
 					       guint pos,
 					       gboolean query_text,
 					       gboolean build,
