@@ -49,6 +49,9 @@ struct ScreemWindowDetails
 	/* current view */
 	GtkWidget *current;
 
+	/* site file view */
+	GtkWidget *site_view;
+	
 	/* Screem stuff */
 	ScreemSite *current_site;
 	ScreemSpell *spell;
@@ -73,6 +76,9 @@ struct ScreemWindowDetails
 	EggRecentModel *recent_page_model;
 	EggRecentViewUIManager *recent_page_view;
 	
+	EggRecentViewUIManager *recent_page_dropdown_view;
+	EggRecentViewUIManager *recent_site_dropdown_view;
+	
 	gboolean offline;
 
 	guint status_timeout;
@@ -86,7 +92,8 @@ struct ScreemWindowDetails
 void screem_window_recent_page_cb( GtkAction *action, gpointer data );
 void screem_window_recent_site_cb( GtkAction *action, gpointer data );
 
-void screem_window_initialise_menus( ScreemWindow *window );
+void screem_window_init_actions( ScreemWindow *window );
+void screem_window_init_uis( ScreemWindow *window );
 
 void screem_window_change_view( ScreemWindow *window, GtkWidget *view );
 
