@@ -7,6 +7,8 @@
 
 #include "screem-search.h"
 
+#include "screem-view.h"
+
 #define SCREEM_TYPE_WINDOW  (screem_window_get_type())
 #define SCREEM_WINDOW(obj)  (G_TYPE_CHECK_INSTANCE_CAST( ( obj ), SCREEM_TYPE_WINDOW, \
                                                 ScreemWindow ) )
@@ -109,4 +111,8 @@ void screem_window_add_dock_item( ScreemWindow *window, const gchar *text,
 				  const gchar *stock_id, const gchar *name,
 				  GtkWidget *item, GtkPositionType pos );
 
+void screem_window_set_title( ScreemWindow *window, ScreemPage *page );
+
+ScreemView *screem_window_get_view( ScreemWindow *window );
+
 #endif
