@@ -26,8 +26,8 @@ gchar *screem_support_charset_convert_to( const gchar *data,
 					const gchar *charset,
 					const gchar *from );
 
-gchar *screem_get_mime_type( const gchar *filename, gboolean fast );
-gchar *screem_get_mime_type_override( const gchar *filename,
+const gchar *screem_get_mime_type( const gchar *filename, gboolean fast );
+const gchar *screem_get_mime_type_override( const gchar *filename,
 					const gchar *type ); 
 
 const gchar *screem_utf8_skip_space( const gchar *txt );
