@@ -17,7 +17,7 @@
  *
  * Authors:
  *   James Willcox <jwillcox@cs.indiana.edu>
- *   Paolo Bacchilega  <paobac@cvs.gnome.org>
+ *   Paolo Bacchilega <paobac@cvs.gnome.org>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -29,9 +29,6 @@
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs.h>
-#ifndef USE_STABLE_LIBGNOMEUI
-#include <libgnomeui/gnome-icon-theme.h>
-#endif
 #include <gconf/gconf-client.h>
 #include "egg-recent-model.h"
 #include "egg-recent-view.h"
@@ -43,6 +40,8 @@
 #define EGG_RECENT_ACTION "EggRecentFile"
 #define EGG_RECENT_SEPARATOR (NULL)
 
+#define DEFAULT_LABEL_WIDTH_CHARS 30
+
 struct _EggRecentViewUIManager {
 	GObject         parent_instance;
 
@@ -61,9 +60,8 @@ struct _EggRecentViewUIManager {
 
 	gboolean        show_icons;
 	gboolean        show_numbers;
-#ifndef USE_STABLE_LIBGNOMEUI
-	GnomeIconTheme *theme;
-#endif
+
+	GtkIconTheme *theme;
 
 	EggUIManagerTooltipFunc tooltip_func;
 	gpointer        tooltip_func_data;
@@ -71,6 +69,8 @@ struct _EggRecentViewUIManager {
 	EggRecentModel *model;
 	GConfClient    *client;
 	GtkIconSize     icon_size;
+
+	gint            label_width;
 };
 
 
@@ -92,11 +92,59 @@ enum {
 	PROP_UIMANAGER,
 	PROP_PATH,
 	PROP_SHOW_ICONS,
-	PROP_SHOW_NUMBERS
+	PROP_SHOW_NUMBERS,
+	PROP_LABEL_WIDTH
 };
 
 static guint view_signals[LAST_SIGNAL] = { 0 };
 
+static void
+connect_proxy_cb (GtkActionGroup         *action_group,
+		  GtkAction              *action,
+		  GtkWidget              *proxy,
+		  EggRecentViewUIManager *view)
+{
+	if (GTK_IS_MENU_ITEM (proxy))
+	{
+		GtkWidget *label;
+		GtkSettings *settings;
+		guint width;
+		guint height;
+		EggRecentItem *item;
+		const gchar *uri;
+		gchar *mime;
+		GdkPixbuf *pixbuf;
+		GtkWidget *img;
+
+		settings = gtk_widget_get_settings (proxy);
+		gtk_icon_size_lookup_for_settings (settings,
+				GTK_ICON_SIZE_MENU, &width, &height);
+	
+		item = g_object_get_data (G_OBJECT (action), 
+				"egg_recent_uri" );
+		if (item && view->show_icons) {
+			uri = egg_recent_item_peek_uri (item);
+			mime = egg_recent_item_get_mime_type (item);
+
+			pixbuf = egg_recent_util_get_icon (view->theme,
+					uri, mime, width);
+			if (pixbuf) {
+				img = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (proxy));
+				gtk_image_set_from_pixbuf (GTK_IMAGE (img),
+							pixbuf);
+		
+				g_object_unref( G_OBJECT( pixbuf ) );
+			} 
+			g_free( mime );
+		}
+
+		label = GTK_BIN (proxy)->child;
+
+		gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+		gtk_label_set_max_width_chars (GTK_LABEL (label), view->label_width);
+	}
+}
+
 static void
 egg_recent_view_uimanager_clear (EggRecentViewUIManager *view)
 {
@@ -110,6 +158,8 @@ egg_recent_view_uimanager_clear (EggRecentViewUIManager *view)
 		g_object_unref (view->action_group);
 		view->action_group = NULL;
 	}
+
+	gtk_ui_manager_ensure_update (view->uimanager);
 }
 
 static void
@@ -117,7 +167,6 @@ egg_recent_view_uimanager_set_list (EggRecentViewUIManager *view, GList *list)
 {
 	GList  *scan;
 	guint   index = 1;
-	gchar *group;
 
 	g_return_if_fail (view);
 
@@ -127,24 +176,27 @@ egg_recent_view_uimanager_set_list (EggRecentViewUIManager *view, GList *list)
 		view->merge_id = gtk_ui_manager_new_merge_id (view->uimanager);
 
 	if (view->action_group == NULL) {
-		group = g_strdup_printf ("EggRecentActions%u", 
+		gchar *group = g_strdup_printf ("EggRecentActions%u",
 				view->merge_id);
 		view->action_group = gtk_action_group_new (group);
+		g_signal_connect (view->action_group, "connect-proxy",
+				G_CALLBACK (connect_proxy_cb), view);
+		gtk_ui_manager_insert_action_group (view->uimanager,
+				view->action_group, -1);
 		g_free (group);
-		gtk_ui_manager_insert_action_group (view->uimanager, view->action_group, 0);
 	}
-	
+
 	if (view->leading_sep) {
-		group = g_strdup_printf ("EggRecentLeadingSeparator%u",
+		gchar *action = g_strdup_printf ("EggRecentLeadingSeparator%u",
 				view->merge_id);
 		gtk_ui_manager_add_ui (view->uimanager, 
 				       view->merge_id, 
 				       view->path,
-				       group,
+				       action,
 				       EGG_RECENT_SEPARATOR,
 				       GTK_UI_MANAGER_AUTO, 
 				       FALSE);
-		g_free (group);
+		g_free (action);
 	}
 
 	for (scan = list; scan; scan = scan->next, index++) {
@@ -162,7 +214,8 @@ egg_recent_view_uimanager_set_list (EggRecentViewUIManager *view, GList *list)
 			continue;
 
 		name = g_strdup_printf (EGG_RECENT_NAME_PREFIX"%u-%u", 
-				view->merge_id, index);
+				view->merge_id, 
+				index);
 
 		if (view->tooltip_func != NULL)
 			tooltip = (*view->tooltip_func) (item, view->tooltip_func_data);
@@ -174,9 +227,13 @@ egg_recent_view_uimanager_set_list (EggRecentViewUIManager *view, GList *list)
 
 		if (view->show_numbers) {
 			if (index >= 10)
-				label = g_strdup_printf ("%d.  %s", index, escaped);
+				label = g_strdup_printf ("%d.  %s", 
+						index, 
+						escaped);
 			else
-				label = g_strdup_printf ("_%d.  %s", index, escaped);
+				label = g_strdup_printf ("_%d.  %s", 
+						index, 
+						escaped);
 			g_free (escaped);
 		} else 
 			label = escaped;
@@ -184,14 +241,19 @@ egg_recent_view_uimanager_set_list (EggRecentViewUIManager *view, GList *list)
 		action = g_object_new (GTK_TYPE_ACTION,
 				       "name", name,
 				       "label", label,
-				       "stock_id", GTK_STOCK_OPEN,
+				       NULL,
+/*				       (view->show_icons)? "stock_id": NULL,
+				       GTK_STOCK_OPEN,*/
 				       NULL);
 		if (tooltip != NULL) {
 			g_object_set (action, "tooltip", tooltip, NULL);
 			g_free (tooltip);
 		}
 		egg_recent_item_ref (item);
-		g_object_set_data_full (G_OBJECT (action), "egg_recent_uri", item, (GFreeFunc)egg_recent_item_unref);
+		g_object_set_data_full (G_OBJECT (action), 
+				"egg_recent_uri", 
+				item, 
+				(GFreeFunc)egg_recent_item_unref);
 
 		if (view->action_callback != NULL) {
 			GClosure *closure;
@@ -215,16 +277,16 @@ egg_recent_view_uimanager_set_list (EggRecentViewUIManager *view, GList *list)
 	}
 
 	if (view->trailing_sep) {
-		group = g_strdup_printf ("EggRecentTrailingSeparator%u",
+		gchar *action = g_strdup_printf ("EggRecentTrailingSeparator%u",
 				view->merge_id);
 		gtk_ui_manager_add_ui (view->uimanager, 
 				       view->merge_id, 
 				       view->path,
-				       group,
+				       action,
 				       EGG_RECENT_SEPARATOR,
 				       GTK_UI_MANAGER_AUTO, 
 				       FALSE);
-		g_free (group);
+		g_free (action);
 	}
 }
 
@@ -237,6 +299,7 @@ model_changed_cb (EggRecentModel         *model,
 		egg_recent_view_uimanager_set_list (view, list);
 	else
 		egg_recent_view_uimanager_clear (view);
+	gtk_ui_manager_ensure_update (view->uimanager);
 }
 
 static EggRecentModel *
@@ -317,6 +380,9 @@ egg_recent_view_uimanager_set_property (GObject      *object,
 	case PROP_SHOW_NUMBERS:
 		egg_recent_view_uimanager_show_numbers (view, g_value_get_boolean (value));
 		break;
+	case PROP_LABEL_WIDTH:
+		egg_recent_view_uimanager_set_label_width (view, g_value_get_int (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -344,6 +410,9 @@ egg_recent_view_uimanager_get_property (GObject    *object,
 	case PROP_SHOW_NUMBERS:
 		g_value_set_boolean (value, view->show_numbers);
 		break;
+	case PROP_LABEL_WIDTH:
+		g_value_set_int (value, view->label_width);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -380,12 +449,10 @@ egg_recent_view_uimanager_finalize (GObject *object)
 		view->model = NULL;
 	}
 
-#ifndef USE_STABLE_LIBGNOMEUI
 	if (view->theme != NULL) {
 		g_object_unref (view->theme);
 		view->theme = NULL;
 	}
-#endif
 
 	if (view->client != NULL) {
 		g_object_unref (view->client);
@@ -443,6 +510,16 @@ egg_recent_view_uimanager_class_init (EggRecentViewUIManagerClass * klass)
 							       TRUE,
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_LABEL_WIDTH,
+					 g_param_spec_int ("label-width",
+						 	   "Label Width",
+							   "The desired width of the menu label, in characters",
+							   -1,
+							   G_MAXINT,
+							   DEFAULT_LABEL_WIDTH_CHARS,
+							   G_PARAM_READWRITE));
+
 	klass->activate = NULL;
 }
 
@@ -467,15 +544,13 @@ show_menus_changed_cb (GConfClient            *client,
 	egg_recent_view_uimanager_show_icons (view, gconf_value_get_bool (value));
 }
 
-#ifndef USE_STABLE_LIBGNOMEUI
 static void
-theme_changed_cb (GnomeIconTheme         *theme, 
+theme_changed_cb (GtkIconTheme         *theme, 
 		  EggRecentViewUIManager *view)
 {
 	if (view->model != NULL)
 		egg_recent_model_changed (view->model);
 }
-#endif
 
 static void
 egg_recent_view_uimanager_init (EggRecentViewUIManager * view)
@@ -506,17 +581,15 @@ egg_recent_view_uimanager_init (EggRecentViewUIManager * view)
 
 	view->path = NULL;
 
-#ifndef USE_STABLE_LIBGNOMEUI
-	view->theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (view->theme, TRUE);
+	view->theme = gtk_icon_theme_get_default ();
 	g_signal_connect_object (view->theme, "changed",
 				 G_CALLBACK (theme_changed_cb), view, 0);
-#endif
 
 	view->tooltip_func = NULL;
 	view->tooltip_func_data = NULL;
 
 	view->icon_size = GTK_ICON_SIZE_MENU;
+	view->label_width = DEFAULT_LABEL_WIDTH_CHARS;
 }
 
 void
@@ -631,6 +704,21 @@ egg_recent_view_uimanager_get_path (EggRecentViewUIManager *view)
 	return view->path;
 }
 
+void
+egg_recent_view_uimanager_set_label_width (EggRecentViewUIManager *view,
+					   gint                    chars)
+{
+	g_return_if_fail (EGG_IS_RECENT_VIEW_UIMANAGER (view));
+	view->label_width = chars;
+}
+
+gint
+egg_recent_view_uimanager_get_label_width (EggRecentViewUIManager *view)
+{
+	g_return_val_if_fail (EGG_IS_RECENT_VIEW_UIMANAGER (view), DEFAULT_LABEL_WIDTH_CHARS);
+	return view->label_width;
+}
+
 void
 egg_recent_view_uimanager_set_action_func (EggRecentViewUIManager   *view,
 					   GCallback                 callback,
