@@ -2,6 +2,6 @@
 #define __EGG_INTL_H__
 
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #endif /* __EGG_INTL_H__ */
