@@ -1,8 +1,6 @@
-/*  Screem:  css-wizard.c
+/*  Screem:  skel-plugin.c
  *
- *  A Wizard for building up CSS selector patterns
- * 
- *  (C) Copyright 2004 David A Knight
+ *  Copyright (C) 2004 David A Knight
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,40 +14,37 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
-
 #include <config.h>
 
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-entry.h>
-#include <libgnomeui/gnome-file-entry.h>
-
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include <glade/glade.h>
-
 #include <string.h>
 
-#include "screem-plugin.h"
+#include "screem-skel-plugin.h"
 
-#include "screem-window.h"
-#include "screem-editor.h"
+/* setup your plugin here */
 
 #include "support.h"
 
-/* per wizard struct */
-typedef struct {
-	ScreemWindow *window;
-	ScreemEditor *editor;
+/* plugin name should only include a-zA-Z */
+static const gchar *plugin_name = "ScreemCSSWizard";
 
-} CSSWizard;
-
-
-/* keep track of how many wizards we have */
-static GList *wizards = NULL;
+static const gchar *authors[] = {
+	"David A Knight <david@screem.org>",
+	NULL
+};
+static const gchar *displayed_name = N_( "Screem CSS Wizard" );
+static const gchar *description = N_( "A wizard for inserting CSS selectors and properties" );
+static const gchar *version = "2.0.0";
 
+/* add any per instance data items here */
+struct ScreemSkelPluginPrivate {
+	gpointer dummy;
+};
 
 enum {
 	ANYWHERE = 0,
@@ -58,131 +53,70 @@ enum {
 	AFTER
 };
 
-/* Required Plugin parts */
+static void css_selector_wizard_display( GtkAction *action, 
+		gpointer user_data );
+static void create_tag_menu( ScreemPlugin *plugin, GladeXML *xml );
+static void create_action_menu( GladeXML *xml );
+static gchar *create_styles( GladeXML *xml );
+static gchar *create_selector( GtkWidget *widget );
+static void add_page( GtkWidget *widget, const gchar *label_txt );
 
-/* we don't want to try and load old plugins
-   so this is a version symbol for screem to spot */
-int screem_plugin_version = 4;
-
-#define NAME "CSS Selector Wizard"
-#define AUTHOR "David A Knight (david@screem.org)"
-#define PLUGIN_VERSION "2.0"
-#define TAG NULL
-
-
-G_MODULE_EXPORT const gchar* g_module_check_init( GModule *module );
-G_MODULE_EXPORT void g_module_unload( GModule *module );
-G_MODULE_EXPORT void init( ScreemPlugin *plugin );
-G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
-			     GtkWidget *preview, GtkWidget *link_view );
-G_MODULE_EXPORT void remove_ui( GtkWidget *window, GtkWidget *editor,
-				GtkWidget *preview, GtkWidget *link_view );
-
-void css_selector_wizard_display( GtkAction *action, gpointer user_data );
+void css_selector_next_change( GtkComboBox *omenu );
 void css_selector_tag_change( GtkWidget *widget );
+void css_wizard_color_set( GtkWidget *widget, GtkColorButton *cp );
 
-
-G_MODULE_EXPORT const gchar* g_module_check_init( GModule *module )
+/**
+ * setup:
+ * 
+ * this function will be called once for each window,
+ * you should add any actions / ui here, eg.
+ * 
+ * screem_plugin_add_action( plugin, name, label, tip, stock_id,
+ * 				callback, error );
+ * screem_plugin_add_menu( plugin, path, action, error );
+ * screem_plugin_add_toolbar( plugin, path, action, error );
+ *
+ *
+ * to insert text into the current page being edited your callbacks
+ * should make use of
+ * screem_plugin_get_cursor_position( plugin )
+ * screem_plugin_set_cursor_position( plugin, pos )
+ * screem_plugin_insert( plugin, pos, text, length, indent )
+ * 
+ **/
+static gboolean setup( ScreemPlugin *plugin )
 {
-	return NULL;
-}
-
-G_MODULE_EXPORT void g_module_unload( GModule *module )
-{
-}
-
-G_MODULE_EXPORT void init( ScreemPlugin *plugin )
-{
-	plugin->name = NAME;
-	plugin->author = AUTHOR;
-	plugin->version = PLUGIN_VERSION;
-
-	/* these can all be left out if not required */
-	plugin->tag = TAG;
-
-	g_module_symbol( plugin->module, "popup", 
-			 (gpointer*)&plugin->popup );
-	g_module_symbol( plugin->module, "add_ui", 
-			 (gpointer*)&plugin->add_ui );
-	g_module_symbol( plugin->module, "remove_ui", 
-			 (gpointer*)&plugin->remove_ui );
-}
-
-/* not needed if nothing is being added to the UI (unusual) */
-G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
-			     GtkWidget *preview, GtkWidget *link_view )
-{
-	CSSWizard *wizard;
-
-	const gchar *ui = "\
-<ui>\
-<menubar>\
-<menu action=\"Insert\">\
-<menu action=\"Wizards\">\
-<menuitem action=\"CSSSelectorWizard\"/>\
-</menu>\
-</menu>\
-</menubar>\
-<toolbar name=\"Wizards Toolbar\">\
-<toolitem action=\"CSSSelectorWizard\" />\
-</toolbar>\
-</ui>";
-	GtkAction *action;
-	
-	gchar *label;
-	gchar *tip;
 	GError *error;
-	
-	wizard = g_new0( CSSWizard, 1 );
-	wizard->window = SCREEM_WINDOW( window );
-	wizard->editor = SCREEM_EDITOR( editor );
-	
-	label = g_strdup( _( "CSS Selector" ) );
-	tip = g_strdup( _( "A Wizard to help you construct selectors for applying css properties to " ) );
-	action = gtk_action_new( "CSSSelectorWizard", label, tip,
-			GTK_STOCK_EXECUTE );
-	g_signal_connect( G_OBJECT( action ), "activate",
-			G_CALLBACK( css_selector_wizard_display ), wizard );
-	gtk_action_group_add_action( GTK_ACTION_GROUP( wizard->window->action_group ), action );
-	g_free( label );
-	g_free( tip );
-	
+	gboolean ret;
+
 	error = NULL;
-	if( ! gtk_ui_manager_add_ui_from_string( GTK_UI_MANAGER( wizard->window->merge ),
-						ui, strlen( ui ), &error ) ) {
-		g_message( "%s ui error = %s", "css wizard",
-				error->message );
+	
+	ret = screem_plugin_add_interface( plugin, "CSSSelectorWizard",
+				_( "CSS Selector" ),
+				_( "A wizard to help you construct selectors for applying css properties to" ),
+				GTK_STOCK_EXECUTE,
+				G_CALLBACK( css_selector_wizard_display ),
+				&error  );
+	if( ! ret ) {
+		g_print( "Add interface error: %s\n", error->message );
 		g_error_free( error );
-	}
-
-	wizards = g_list_append( wizards, wizard );
+	}	
+	return ret;
 }
 
-/* required it add_ui is present */
-G_MODULE_EXPORT void remove_ui( GtkWidget *window, GtkWidget *editor,
-				GtkWidget *preview, GtkWidget *link_view )
+/**
+ * cleanup:
+ *
+ * this function will be called once for each window when
+ * it is closed, you should cleanup any data items you
+ * have in ScreemSkelPluginPrivate here
+ **/
+static void cleanup( ScreemSkelPluginPrivate *priv )
 {
-	GList *list;
-	CSSWizard *wizard;
 
-	for( list = wizards; list; list = list->next ) {
-		wizard = (CSSWizard*)list->data;
-		if( wizard->window == SCREEM_WINDOW( window ) ) {
-			/* got it */
-			break;
-		}
-	}
-       
-	g_return_if_fail( list != NULL );
-
-	/* wizard is the one to erase */
-	wizards = g_list_remove( wizards, wizard );
-	g_free( wizard );
 }
 
-/* End of required section */
-
-static void create_tag_menu( CSSWizard *wizard, GladeXML *xml )
+static void create_tag_menu( ScreemPlugin *plugin, GladeXML *xml )
 {
 	GtkWidget *widget;
 	GtkWidget *entry;
@@ -200,7 +134,7 @@ static void create_tag_menu( CSSWizard *wizard, GladeXML *xml )
 	widget = glade_xml_get_widget( xml, "tag_menu" );
 	entry = GTK_BIN( widget )->child;
 	
-	page = screem_window_get_document( wizard->window );
+	page = screem_plugin_get_current_document( plugin );
 	if( page ) {
 		comp = gtk_entry_completion_new();
 		store = gtk_list_store_new( 1, G_TYPE_STRING );
@@ -240,7 +174,7 @@ static void create_tag_menu( CSSWizard *wizard, GladeXML *xml )
 	}
 }
 
-static void create_action_menu( CSSWizard *wizard, GladeXML *xml )
+static void create_action_menu( GladeXML *xml )
 {
 	GtkWidget *widget;
 	GtkListStore *store;
@@ -275,31 +209,35 @@ static void add_page( GtkWidget *widget, const gchar *label_txt )
 	GladeXML *xml;
 	GtkWidget *notebook;
 	GtkWidget *label;
-	CSSWizard *wizard;
+	ScreemPlugin *plugin;
 	
 	xml = glade_get_widget_tree( widget );
 	widget = glade_xml_get_widget( xml, "match_box" );
 	notebook = g_object_get_data( G_OBJECT( widget ), "notebook" );
-	wizard = g_object_get_data( G_OBJECT( widget ), "wizard" );
+	plugin = g_object_get_data( G_OBJECT( widget ), "plugin" );
 	
 	xml = glade_xml_new( GLADE_PATH"/css-wizard.glade",
 			"match_box", NULL );
 	widget = glade_xml_get_widget( xml, "match_box" );
 	g_object_set_data( G_OBJECT( widget ),
 			"notebook", notebook );
-	g_object_set_data( G_OBJECT( widget ), "wizard",
-			wizard );
+	g_object_set_data( G_OBJECT( widget ), "plugin", plugin );
 
 	label = gtk_label_new( label_txt );
 	
-	create_tag_menu( wizard, xml );
-	create_action_menu( wizard, xml );
+	create_tag_menu( plugin, xml );
+	create_action_menu( xml );
 	
 	css_selector_tag_change( widget );
 	widget = glade_xml_get_widget( xml, "hyperlink_menu" );
 	gtk_combo_box_set_active( GTK_COMBO_BOX( widget ), 0 );
 	widget = glade_xml_get_widget( xml, "location_menu" );
 	gtk_combo_box_set_active( GTK_COMBO_BOX( widget ), 0 );
+
+	widget = glade_xml_get_widget( xml, "class_entry" );
+	screem_gtk_add_history( widget );
+	widget = glade_xml_get_widget( xml, "id_entry" );
+	screem_gtk_add_history( widget );
 	
 	widget = glade_xml_get_widget( xml, "match_box" );
 	gtk_widget_show( widget );
@@ -314,7 +252,6 @@ static gchar *create_selector( GtkWidget *widget )
 {
 	GladeXML *xml;
 	GtkWidget *notebook;
-	CSSWizard *wizard;
 	gint pages;
 	gint i;
 	gint item;
@@ -327,7 +264,6 @@ static gchar *create_selector( GtkWidget *widget )
 	
 	pattern = g_string_new( "\n" );
 	notebook = g_object_get_data( G_OBJECT( widget ), "notebook" );
-	wizard = g_object_get_data( G_OBJECT( widget ), "wizard" );
 
 	pages = gtk_notebook_get_n_pages( GTK_NOTEBOOK( notebook ) );
 	for( i = 0; i < pages; ) {
@@ -359,7 +295,7 @@ static gchar *create_selector( GtkWidget *widget )
 		widget = glade_xml_get_widget( xml, "id_entry" );
 		if( GTK_WIDGET_IS_SENSITIVE( widget ) ) {
 			/* match id */
-			widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+			widget = GTK_BIN( widget )->child;
 			g_string_prepend( pattern,
 					gtk_entry_get_text( GTK_ENTRY( widget ) ) );
 			g_string_prepend_c( pattern, '#' );
@@ -367,7 +303,7 @@ static gchar *create_selector( GtkWidget *widget )
 		widget = glade_xml_get_widget( xml, "class_entry" );
 		if( GTK_WIDGET_IS_SENSITIVE( widget ) ) {
 			/* match class */
-			widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+			widget = GTK_BIN( widget )->child;
 			g_string_prepend( pattern,
 					gtk_entry_get_text( GTK_ENTRY( widget ) ) );
 			g_string_prepend_c( pattern, '.' );
@@ -482,27 +418,32 @@ static gchar *create_styles( GladeXML *xml )
 	guint i;
 	const gchar *field;
 	const gchar *value;
+	gchar *uri;
 	
 	str = g_string_new( "" );
 	for( i = 0; i < n_fields; ++ i ) {
 		field = fields[ i ];
 		widget = glade_xml_get_widget( xml, field );
-		if( GNOME_IS_ENTRY( widget ) ) {
-			widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
-		} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
-			widget = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
-		} else if( GTK_IS_COMBO_BOX_ENTRY( widget ) ) {
-			widget = GTK_BIN( widget )->child;
-		} else if( GTK_IS_COMBO( widget ) ) {
-			g_warning( "COMBO %s needs replacing\n",
-					field );
-			widget = GTK_COMBO( widget )->entry;
+		uri = NULL;
+		if( GTK_IS_FILE_CHOOSER_BUTTON( widget ) ) {
+			uri = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( widget ) );
+			value = uri;
+			if( value && *value ) {
+				uri = g_strconcat( "url( ", value, ")", NULL );
+				g_free( (gchar*)value);
+				value = uri;
+			}
+		} else {
+			if( GTK_IS_COMBO_BOX_ENTRY( widget ) ) {
+				widget = GTK_BIN( widget )->child;
+			}
+			value = gtk_entry_get_text( GTK_ENTRY( widget ) );
 		}
-		value = gtk_entry_get_text( GTK_ENTRY( widget ) );
 		if( value && *value ) {
 			g_string_append_printf( str,
 					"\t%s: %s;\n", field, value );
 		}
+		g_free( uri );
 	}
 
 	if( str->len > 0 ) {
@@ -516,9 +457,10 @@ static gchar *create_styles( GladeXML *xml )
 	return ret;
 }
 
-void css_selector_wizard_display( GtkAction *action, gpointer user_data )
+static void css_selector_wizard_display( GtkAction *action, 
+		gpointer user_data )
 {
-	CSSWizard *wizard;
+	ScreemPlugin *plugin;
 	ScreemPage *page;
 	GtkWidget *widget;
 	GtkWidget *notebook;
@@ -526,15 +468,11 @@ void css_selector_wizard_display( GtkAction *action, gpointer user_data )
 	gint button;
 	gchar *pattern;
 	gchar *styles;
-	ScreemApplication *app;
-	ScreemSession *session;
 	
-	wizard = (CSSWizard*)user_data;
-	page = screem_window_get_document( wizard->window );
+	plugin = SCREEM_PLUGIN( user_data );
+	
+	page = screem_plugin_get_current_document( plugin );
 
-	app = SCREEM_APPLICATION( wizard->window->application );
-	session = screem_application_get_session( app );
-	
 	if( page ) {
 		xml = glade_xml_new( GLADE_PATH"/css-wizard.glade",
 				"csspattern", NULL );
@@ -542,10 +480,10 @@ void css_selector_wizard_display( GtkAction *action, gpointer user_data )
 		notebook = glade_xml_get_widget( xml, "notebook" );
 		g_object_set_data( G_OBJECT( widget ),
 				"notebook", notebook );
-		g_object_set_data( G_OBJECT( widget ), "wizard",
-				wizard );
-		create_tag_menu( wizard, xml );
-		create_action_menu( wizard, xml );
+		g_object_set_data( G_OBJECT( widget ), "plugin",
+				plugin );
+		create_tag_menu( plugin, xml );
+		create_action_menu( xml );
 	
 		widget = glade_xml_get_widget( xml, "hyperlink_menu" );
 		gtk_combo_box_set_active( GTK_COMBO_BOX( widget ), 0 );
@@ -558,24 +496,23 @@ void css_selector_wizard_display( GtkAction *action, gpointer user_data )
 
 		css_selector_tag_change( widget );
 		
-		screem_session_restore_dialog( session, widget );
+		screem_plugin_restore_from_session( plugin, widget );
 		do {	
 			button = gtk_dialog_run( GTK_DIALOG( widget ) );		} while( button == 0 );
 
-		screem_session_store_dialog( session, widget );
+		screem_plugin_store_in_session( plugin, widget );
 
 		if( button == GTK_RESPONSE_APPLY ) {
-			widget = glade_xml_get_widget( xml, 
-					"match_box" );
+			widget = glade_xml_get_widget( xml, "match_box" );
 			pattern = create_selector( widget );
-			
-			screem_editor_insert( wizard->editor, -1,
-					pattern );
+		
+			screem_plugin_insert( plugin, -1, pattern,
+					strlen( pattern ), FALSE );
 			g_free( pattern );
 
 			styles = create_styles( xml );
-			screem_editor_insert( wizard->editor, -1,
-					styles );
+			screem_plugin_insert( plugin, -1, styles,
+					strlen( styles ), FALSE );
 			g_free( styles );
 		}
 		
@@ -640,7 +577,7 @@ void css_selector_next_change( GtkComboBox *omenu )
 void css_selector_tag_change( GtkWidget *widget )
 {
 	GladeXML *xml;
-	CSSWizard *wizard;
+	ScreemPlugin *plugin;
 	const gchar *name;
 	ScreemPage *page;
 	ScreemDTD *dtd;
@@ -652,15 +589,15 @@ void css_selector_tag_change( GtkWidget *widget )
 
 	xml = glade_get_widget_tree( widget );
 	widget = glade_xml_get_widget( xml, "match_box" );
-	wizard = g_object_get_data( G_OBJECT( widget ), "wizard" );
-
+	plugin = SCREEM_PLUGIN( g_object_get_data( G_OBJECT( widget ),
+				"plugin" ) );
 	widget = glade_xml_get_widget( xml, "tag_menu" );
 	show = ( ! GTK_WIDGET_IS_SENSITIVE( widget ) );
 	entry = GTK_BIN( widget )->child;
 
 	name = gtk_entry_get_text( GTK_ENTRY( entry ) );
 
-	page = screem_window_get_document( wizard->window );
+	page = screem_plugin_get_current_document( plugin );
 	if( page && ! show ) {
 		dtd = screem_page_get_dtd( page );
 		elem = screem_dtd_valid_element( dtd, name );
@@ -692,20 +629,113 @@ void css_wizard_color_set( GtkWidget *widget, GtkColorButton *cp )
 	gchar *text;
 	
 	gtk_color_button_get_color( cp, &c );
-	text = screem_gdk_color_to_string( &c );
+	text = screem_gdk_color_to_string( &c, TRUE );
 
-	if( GNOME_IS_ENTRY( widget ) ) {
-		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
-	} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
-		widget = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
-	} else if( GTK_IS_COMBO_BOX_ENTRY( widget ) ) {
+	if( GTK_IS_COMBO_BOX_ENTRY( widget ) ) {
 		widget = GTK_BIN( widget )->child;
-	} else if( GTK_IS_COMBO( widget ) ) {
-		widget = GTK_COMBO( widget )->entry;
-	}
+	} 
+	g_return_if_fail( GTK_IS_ENTRY( widget ) );
 
 	gtk_entry_set_text( GTK_ENTRY( widget ), text );
 	
 	g_free( text );
 }
 
+
+/* There should be no need to change any code below here */
+enum {
+	ARG_0
+};
+
+static void screem_skel_plugin_class_init( ScreemSkelPluginClass *klass );
+static void screem_skel_plugin_init( ScreemSkelPlugin *skel_plugin );
+static void screem_skel_plugin_finalize( GObject *object );
+
+/* G Object stuff */
+#define PARENT_TYPE SCREEM_TYPE_PLUGIN
+
+static gpointer parent_class;
+
+static void screem_skel_plugin_class_init( ScreemSkelPluginClass *klass )
+{
+	GObjectClass *object_class;
+
+	object_class = G_OBJECT_CLASS( klass );
+
+	object_class->finalize = screem_skel_plugin_finalize;
+	parent_class = g_type_class_peek_parent( klass );
+}
+
+static void screem_skel_plugin_init( ScreemSkelPlugin *skel_plugin )
+{
+	skel_plugin->priv = g_new0( ScreemSkelPluginPrivate, 1 );
+	SCREEM_PLUGIN( skel_plugin )->setup = setup;
+}
+
+static void screem_skel_plugin_finalize( GObject *object )
+{
+	ScreemSkelPlugin *skel_plugin;
+	ScreemSkelPluginPrivate *priv;
+	
+	skel_plugin = SCREEM_SKEL_PLUGIN( object );
+	priv = skel_plugin->priv;
+
+	cleanup( priv );
+	
+	g_free( priv );
+	
+	G_OBJECT_CLASS( parent_class )->finalize( object );
+}
+
+static GType screem_skel_plugin_get_type()
+{
+	static GType type = 0;
+	
+	if( ! type ) {
+		static const GTypeInfo info = {
+			sizeof( ScreemSkelPluginClass ),
+			NULL, /* base init */
+			NULL, /* base finalise */
+			(GClassInitFunc)screem_skel_plugin_class_init,
+			NULL, /* class finalise */
+			NULL, /* class data */
+			sizeof( ScreemSkelPlugin ),
+			0, /* n_preallocs */
+			(GInstanceInitFunc)screem_skel_plugin_init
+		};
+
+		type = g_type_register_static( PARENT_TYPE,
+					       plugin_name,
+					       &info, 0 );
+	}
+
+	return type;
+}
+
+static ScreemSkelPlugin *screem_skel_plugin_new( void )
+{
+	ScreemSkelPlugin *skel_plugin;
+
+	skel_plugin = SCREEM_SKEL_PLUGIN( g_object_new( SCREEM_TYPE_SKEL_PLUGIN, 
+				"name", plugin_name,
+				NULL ) );
+
+	return skel_plugin;
+}
+
+G_MODULE_EXPORT void get_details( ScreemPluginDetails **ret )
+{
+	ScreemPluginDetails *details;
+
+	details = g_new0( ScreemPluginDetails, 1 );
+	details->name = plugin_name;
+	details->displayed_name = displayed_name;
+	details->authors = authors;
+	details->description = description;
+	details->version = version;
+	details->create = screem_skel_plugin_new;
+	details->api_version = SCREEM_PLUGIN_REQUIRED_VERSION;
+
+	*ret = details;
+}
+
