@@ -23,21 +23,13 @@
 
 #include <config.h>
 
-#include <sys/types.h>
-
-#include <stdio.h>
-
-#include <ne_utils.h>
-
 /* our own DEBUG_ channels */
 #define DEBUG_FILES (1<<10)
 #define DEBUG_RCFILE (1<<11)
 #define DEBUG_FTP (1<<12)
 #define DEBUG_GNOME (1<<13)
 #define DEBUG_FILESEXTRA (1<<13)
-
-/* A signal hander */
-typedef void (*sig_handler)(int);
+#define DEBUG_RSH (1<<14)
 
 #ifdef __EMX__
 /* siebert: strcasecmp is stricmp */
@@ -56,30 +48,10 @@ typedef void (*sig_handler)(int);
 #define OPEN_BINARY_FLAGS 0
 #endif
 
-#if !HAVE_STRERROR && !defined(strerror)
-char *strerror (int errnum);
-#endif
+#define ASC2HEX(x) (((x) <= '9') ? ((x) - '0') : (tolower((x)) + 10 -         'a'))
+#define HEX2ASC(x) ((char) ((x) > 9 ? ((x) - 10 + 'a') : ((x) + '0'))        )
 
-extern const char *default_charset;
-void init_charset(void);
-
-/* Map debug options onto a debug mask. Returns non-zero on error, 
- * zero on success. err must be at least 20 bytes.
- * On non-zero return, err will contain the token which was not
- * recognized.
- * example use:
- 
-   char errbuf[20];
-   int mask;
-
-   if (map_debug_options(argv[n], &mask, errbuf)) {
-       printf("did not understand channel %s\n", errbuf);
-       exit(-1);
-   }
- 
-*/ 
-
-int map_debug_options(const char *opts, int *mask, char *err);
-
+#define NE_DEBUG(...)
+							   
 #endif
 
