@@ -27,10 +27,11 @@ struct site_file * FS_NAME (enum file_type type, struct file_state *state,
 {
     struct site_file *file, *direct = NULL, *moved = NULL, *frename = NULL;
     enum file_diff dir_diff;
-    char *bname = NULL; /* init to shut up gcc */
+    char *bname= NULL; /* init to shut up gcc */
+    char *tmp = NULL;
     NE_DEBUG(DEBUG_FILES, FS_UV " on %s: \n", state->filename);
     if (site->checkmoved && (type==file_file)) 
-	bname = base_name(state->filename);
+	bname = g_path_get_basename(state->filename);
     for (file=site->files; file!=NULL; file=file->next) {
 	/* TODO: profile and reorder these checks for best case. */
 	if (file->FS_BETA.exists && 
@@ -50,8 +51,9 @@ struct site_file * FS_NAME (enum file_type type, struct file_state *state,
 	     * 'rename move'. This shouldn't matter, since we prefer
 	     * the move to the rename in the single candidate case,
 	     * and in the multiple candidate case,  */
-	    if ((moved == NULL) && 
-	       strcmp(bname, base_name(file->FS_BETA.filename)) == 0) {
+	    tmp = g_path_get_basename(file->FS_BETA.filename);
+	    if ((moved == NULL) && bname && 
+	       strcmp(bname, tmp) == 0) {
 		NE_DEBUG(DEBUG_FILES, "Move candidate: %s\n", 
 		      file->FS_BETA.filename);
 		moved = file;
@@ -60,10 +62,12 @@ struct site_file * FS_NAME (enum file_type type, struct file_state *state,
 		      file->FS_BETA.filename);
 		frename = file;
 	    }
+	    g_free(tmp);
 	}
 	/* Stop searching if we've found all we need */
 	if (direct!=NULL && moved!=NULL && frename!=NULL) break;
     }
+    g_free(bname);
     NE_DEBUG(DEBUG_FILES, "Found: %s-%s-%s\n", 
 	  direct?"direct":"", moved?"moved":"", frename?"rename":"");
     /* We prefer a direct move to a rename */
