@@ -1,6 +1,6 @@
 /* 
    sitecopy, manage remote web sites.
-   Copyright (C) 1998-2000, Joe Orton <joe@orton.demon.co.uk>.
+   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>.
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,11 +41,15 @@
 #ifndef FRONTEND_H
 #define FRONTEND_H
 
-#include <nsocket.h>
-
 #include "common.h"
 #include "sites.h"
 
+typedef enum {
+    fe_namelookup,
+    fe_connecting,
+    fe_connected
+} fe_status;
+
 /* Connection Status API.
  *
  * fe_connection() is called to indicate what state the connection is
@@ -56,7 +60,7 @@
  * info will indicate the hostname being looked up for fe_namelookup,
  * and will be NULL otherwise.
  */
-void fe_connection(sock_status status, const char *info);
+void fe_connection(fe_status status, const char *info);
 
 /* The user is required to authenticate themselves for given context,
  * in the given realm on the given hostname.
@@ -76,8 +80,14 @@ typedef enum {
     fe_login_proxy
 } fe_login_context;
 
+#define FE_LBUFSIZ NE_ABUFSIZ
+
+/* Enter username/password; username and password are fixed-size
+ * buffers of size FE_LBUFSIZ. The username field may be pre-filled
+ * by the username from the rcfile; otherwise username[0] == '\0'.
+ * Contents of password are undefined. */
 int fe_login(fe_login_context ctx, const char *realm, const char *hostname,
-	     char **username, char **password);
+	     char *username, char *password);
 
 /* Modifications to the files list within
  *  site_update, site_fetch, and site_synch
@@ -131,15 +141,11 @@ void fe_fetch_found(const struct site_file *file);
 void fe_warning(const char *description, const char *subject,
 		       const char *error);
 
-/* About to verify given file: */
-
-void fe_verifying(struct site_file *file);
-
 /* Verified - whether the remote file matches or not... 
  match will be one of file_new, file_deleted, file_changed, file_unchanged.
  (NOT file_moved yet)
 */
 
-void fe_verified(struct site_file *file, enum file_diff match);
+void fe_verified(const char *fname, enum file_diff match);
 
 #endif /* FRONTEND_H */
