@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites. File handling.
-   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1998-2004, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,11 +37,7 @@
 
 #include <ctype.h>
 
-#include "basename.h"
-
-#include <ne_md5.h>
-#include <ne_string.h>
-#include <ne_alloc.h>
+#include <glib.h>
 
 /* We pick up FNM_LEADING_DIR fnmatch() extension, since we define
  * _GNU_SOURCE in config.h. */
@@ -85,7 +81,7 @@ void file_delete(struct site *site, struct site_file *item)
 /* Prepends  an item to the fnlist. Returns the item. */
 struct fnlist *fnlist_prepend(struct fnlist **list)
 {
-    struct fnlist *item = ne_malloc(sizeof(struct fnlist));
+    struct fnlist *item = g_malloc(sizeof(struct fnlist));
     item->next = *list;
     item->prev = NULL;
     if (*list != NULL) {
@@ -101,8 +97,8 @@ struct fnlist *fnlist_deep_copy(const struct fnlist *src)
     const struct fnlist *iter;
     struct fnlist *dest = NULL, *prev = NULL, *item = NULL;
     for (iter = src; iter != NULL; iter = iter->next) {
-	item = ne_malloc(sizeof(struct fnlist));
-	item->pattern = ne_strdup(iter->pattern);
+	item = g_malloc(sizeof(struct fnlist));
+	item->pattern = g_strdup(iter->pattern);
 	item->haspath = iter->haspath;
 	if (prev != NULL) {
 	    prev->next = item;
@@ -122,7 +118,7 @@ struct fnlist *fnlist_deep_copy(const struct fnlist *src)
 inline int fnlist_match(const char *filename, const struct fnlist *list)
 {
     const struct fnlist *item;
-    char *bname = base_name(filename);    
+    char *bname = g_path_get_basename(filename);
 
     for (item=list; item != NULL; item=item->next) {
 	NE_DEBUG(DEBUG_FILES, "%s ", item->pattern);
@@ -135,7 +131,7 @@ inline int fnlist_match(const char *filename, const struct fnlist *list)
 		break;
 	}
     }
-	
+    g_free(bname);	
 #ifdef DEBUGGING
     if (item) {
 	NE_DEBUG(DEBUG_FILES, "- matched.\n");
@@ -198,18 +194,17 @@ void file_set_diff(struct site_file *file, struct site *site)
     site_leave(site);
 }
 
-inline void file_state_copy(struct file_state *dest, 
-			    const struct file_state *src,
-			    struct site *site)
+void file_state_copy(struct file_state *dest, const struct file_state *src,
+                     struct site *site)
 {
     site_enter(site);
     file_state_destroy(dest);
     memcpy(dest, src, sizeof(struct file_state));
     if (src->linktarget != NULL) {
-	dest->linktarget = ne_strdup(src->linktarget);
+	dest->linktarget = g_strdup(src->linktarget);
     }
     if (src->filename != NULL) {
-	dest->filename = ne_strdup(src->filename);
+	dest->filename = g_strdup(src->filename);
     }
     site_leave(site);
 }
@@ -236,28 +231,13 @@ void file_state_destroy(struct file_state *state)
  */
 int file_checksum(const char *fname, struct file_state *state, struct site *s)
 {
-    int ret;
-    FILE *f;
-    f = fopen(fname, "r" FOPEN_BINARY_FLAGS);
-    if (f == NULL) {
 	return -1;
-    }
-    ret = ne_md5_stream(f, state->checksum);
-    fclose(f); /* worth checking return value? */
-#ifdef DEBUGGING
-    { 
-	char tmp[33] = {0};
-	ne_md5_to_ascii(state->checksum, tmp);
-	NE_DEBUG(DEBUG_FILES, "Checksum: %s = [%32s]\n", fname, tmp);
-    }
-#endif /* DEBUGGING */
-    return ret;
 }    
 
 char *file_full_remote(struct file_state *state, struct site *site)
 {
     char *ret;
-    ret = ne_malloc(strlen(site->remote_root) + strlen(state->filename) + 1);
+    ret = g_malloc(strlen(site->remote_root) + strlen(state->filename) + 1);
     strcpy(ret, site->remote_root);
     if (site->lowercase) {
 	int n, off, len;
@@ -275,7 +255,7 @@ char *file_full_remote(struct file_state *state, struct site *site)
 
 char *file_full_local(struct file_state *state, struct site *site)
 {
-    return ne_concat(site->local_root, state->filename, NULL);
+    return g_strconcat(site->local_root, state->filename, NULL);
 }
 
 char *file_name(const struct site_file *file)
@@ -342,7 +322,7 @@ void file_uploaded(struct site_file *file, struct site *site)
     }
     /* Now the filename */
     if (file->stored.filename) free(file->stored.filename);
-    file->stored.filename = ne_strdup(file->local.filename);
+    file->stored.filename = g_strdup(file->local.filename);
     file->stored.ascii = file->local.ascii;
     file->stored.exists = file->local.exists;
     file->stored.mode = file->local.mode;
@@ -362,7 +342,7 @@ void file_downloaded(struct site_file *file, struct site *site)
     }
     /* Now the filename */
     if (file->local.filename) free(file->local.filename);
-    file->local.filename = ne_strdup(file->stored.filename);
+    file->local.filename = g_strdup(file->stored.filename);
     file->local.ascii = file->stored.ascii;
     file->local.exists = file->stored.exists;
     file->local.mode = file->stored.mode;
