@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1998-2004, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,19 +46,9 @@
 #include <time.h>
 #include <utime.h>
 
-/* neon */
-#include <ne_string.h>
-#include <ne_alloc.h>
-#include <ne_md5.h>
-#include <ne_socket.h>
+#include <glib.h>
+#include <glib/gi18n.h>
 
-#ifdef HAVE_SNPRINTF_H
-#include "snprintf.h"
-#endif /* !HAVE_SNPRINTF_H */
-
-#include "basename.h"
-
-#include "i18n.h"
 #include "common.h"
 #include "frontend.h"
 #include "protocol.h"
@@ -312,26 +302,51 @@ file_retrieve_server(struct site_file *file, struct site *site, void *session)
     free(full_remote);
 }
 
+/* Create new directories and change permissions on existing directories. */
 static int update_create_directories(struct site *site, void *session)
 {
     struct site_file *current;
     int ret = 0;
 
     for_each_file(current, site) {
-	if ((current->type == file_dir) && (current->diff == file_new) && 
-	    (!site->only_marked || !current->marked)) {
-	    /* New dir! */
+	if ((current->type == file_dir) 
+            && (current->diff == file_new || current->diff == file_changed)) {
+	    /* New or changed directory! */
 	    char *full_remote;
+            int oret;
+
 	    if (!fe_can_update(current)) continue;
+
 	    full_remote = file_full_remote(&current->local, site);
-	    fe_updating(current);
-	    if (CALL(dir_create)(session, full_remote) != SITE_OK) {
-		fe_updated(current, false, DRIVER_ERR);
-		ret = 1;
-	    } else {
-		fe_updated(current, true, NULL);
-		file_uploaded(current, site);
-	    }
+
+            if (current->diff == file_new) {
+                fe_updating(current);
+                oret = CALL(dir_create)(session, full_remote);
+                if (oret != SITE_OK) {
+                    fe_updated(current, false, DRIVER_ERR);
+                } else {
+                    fe_updated(current, true, NULL);
+                }
+            } else {
+                oret = SITE_OK;
+            }
+
+            if (site->dirperms && oret == SITE_OK) {
+                fe_setting_perms(current);
+                oret = CALL(file_chmod)(session, full_remote,
+                                        current->local.mode);
+                if (oret == SITE_OK) {
+                    fe_set_perms(current, true, NULL);
+                } else {
+                    fe_set_perms(current, false, DRIVER_ERR);
+                }
+            }
+
+            if (oret != SITE_OK) {
+                ret = 1;
+            } else {
+                file_uploaded(current, site);
+            }
 	    free(full_remote);
 	}
     }
@@ -339,7 +354,7 @@ static int update_create_directories(struct site *site, void *session)
     return ret;
 }
 
-/* Returns the filename to use for tempupload mode, ne_malloc-allocated.
+/* Returns the filename to use for tempupload mode, g_malloc-allocated.
  * (pass the site since we may have different tempupload modes in the
  * future.)
  * FIXME: implement it efficiently */
@@ -348,7 +363,7 @@ static char *temp_upload_filename(const char *filename, struct site *site)
     char *pnt, *ret;
     /* Insert a '.in.' prefix into the filename, AFTER
      * any directories */
-    ret = ne_malloc(strlen(filename) + 4 + 1);
+    ret = g_malloc(strlen(filename) + 4 + 1);
     strcpy(ret, filename);
     pnt = strrchr(ret, '/');
     if (pnt == NULL) {
@@ -372,9 +387,8 @@ static int update_delete_files(struct site *site, void *session)
 	next = current->next;
 	/* Skip directories and links, and only do deleted files on
 	 * this pass */
-	if ((current->diff == file_deleted) &&
-	    (current->type == file_file) && 
-	    (!site->only_marked || current->marked)) {
+	if (current->diff == file_deleted &&
+	    current->type == file_file) {
 	    char *full_remote;
 	    if (!fe_can_update(current)) continue;
 	    full_remote = file_full_remote(&current->stored, site);
@@ -400,7 +414,7 @@ static int update_move_files(struct site *site, void *session)
     struct site_file *current;
     char *old_full_remote, *full_remote;
     for_each_file(current, site) {
-	if (current->diff != file_moved || (site->only_marked && !current->marked)) 
+	if (current->diff != file_moved) 
 	    continue;
 	full_remote = file_full_remote(&current->local, site);
 	/* The file has been moved */
@@ -435,11 +449,10 @@ static int update_files(struct site *site, void *session)
 	/* This loop only handles changed and new files, so
 	 * skip everything else. */
 
-	if ((current->type != file_file) || 
-	    (current->diff == file_deleted) ||
-	    (current->diff == file_moved) ||
-	    (current->diff == file_unchanged) || 
-	    (site->only_marked && !current->marked)) continue;
+	if (current->type != file_file
+	    || current->diff == file_deleted
+	    || current->diff == file_moved
+	    || current->diff == file_unchanged) continue;
 
 	full_local = file_full_local(&current->local, site);
 	full_remote = file_full_remote(&current->local, site);
@@ -566,7 +579,6 @@ static int update_delete_directories(struct site *site, void *session)
      * directories are deleted bottom up */
     for (current=site->files_tail; current!=NULL; current=prev) {
 	prev = current->prev;
-	if (site->only_marked && !current->marked) continue;
 	if ((current->type==file_dir) && (current->diff == file_deleted)) {
 	    char *full_remote;
 	    if (!fe_can_update(current)) continue;
@@ -594,9 +606,8 @@ static int update_links(struct site *site, void *session)
     for (current=site->files; current!=NULL; current=next) {
 	char *full_remote;
 	next = current->next;
-	if ((current->type != file_link) || 
-	    (site->only_marked && !current->marked))
-	    continue;
+	if (current->type != file_link) continue;
+
 	full_remote = file_full_remote(&current->local, site);
 	switch (current->diff) {
 	case file_new:
@@ -646,7 +657,7 @@ static void proto_finish(struct site *site, void *session)
 
 static void proto_seterror(struct site *site, void *session)
 {
-    site->last_error = ne_strdup(DRIVER_ERR);
+    site->last_error = g_strdup(DRIVER_ERR);
 }
 
 const char *site_get_protoname(struct site *site) 
@@ -714,7 +725,7 @@ int site_update(struct site *site)
 	    }
 	}
     }
-    
+
     if (ret == 0) {
 	/* Site updated successfully. */
 	ret = SITE_OK;
@@ -755,9 +766,9 @@ void site_read_local_state(struct site *site)
     int dirtop = 0, /* points to item above top stack item */
 	dirmax = DIRSTACKSIZE; /* size of stack */
 
-    dirstack = ne_malloc(sizeof(char *) * DIRSTACKSIZE);
+    dirstack = g_malloc(sizeof(char *) * DIRSTACKSIZE);
     /* Push the root directory on to the stack */
-    dirstack[dirtop++] = ne_strdup(site->local_root);
+    dirstack[dirtop++] = g_strdup(site->local_root);
     
     /* Now, for all items in the stack, process all the files, and
      * add the dirs to the stack. Everything we put on the stack is
@@ -795,7 +806,7 @@ void site_read_local_state(struct site *site)
 	    
 	    if (full != NULL) free(full);
 
-	    full = ne_concat(this, ent->d_name, NULL);
+	    full = g_strconcat(this, ent->d_name, NULL);
 
 #ifdef __EMX__
 /* There are no symlinks under OS/2, use stat() instead */
@@ -866,7 +877,7 @@ void site_read_local_state(struct site *site)
 		    fe_warning(_("The target of the symlink could not be read."), full, strerror(errno));
 		    continue;
 		}
-		local.linktarget = ne_strdup(tmp);
+		local.linktarget = g_strdup(tmp);
 	    }
 #endif /* __EMX__ */
 	    else if (S_ISDIR(item.st_mode)) {
@@ -877,7 +888,7 @@ void site_read_local_state(struct site *site)
 		    dirstack = realloc(dirstack, sizeof(char *) * dirmax);
 		}
 		/* Add it to the search stack */
-		dirstack[dirtop] = ne_concat(full, "/", NULL);
+		dirstack[dirtop] = g_strconcat(full, "/", NULL);
 		dirtop++;
 	    } else {
 		NE_DEBUG(DEBUG_FILES, "something else.\n");
@@ -887,7 +898,7 @@ void site_read_local_state(struct site *site)
 	    /* Set up rest of the local state */
 	    local.mode = item.st_mode & 0777;
 	    local.exists = true;
-	    local.filename = ne_strdup(fname);
+	    local.filename = g_strdup(fname);
 
 	    current = file_set_local(type, &local, site);
 	    DEBUG_DUMP_FILE_PROPS(DEBUG_FILES, current, site);
@@ -1037,37 +1048,13 @@ site_fetch_walk(struct site *site, struct proto_file *files)
 static void
 site_fetch_csum_read(void *userdata, const char *s, size_t len)
 {
-    struct ne_md5_ctx *md5 = userdata;
-    ne_md5_process_bytes(s, len, md5);
 }
 
 /* Retrieve the remote checksum for all files */
 static int 
 site_fetch_checksum(struct proto_file *files, struct site *site, void *session)
 {
-    struct ne_md5_ctx md5;
-    struct proto_file *file;
-    int ret = 0;
-    
-    for (file = files; file!=NULL; file=file->next) { 
-	if (file->type == proto_file && 
-	    !file_isexcluded(file->filename, site)) {
-	    char *full_remote;
-	    full_remote = ne_concat(site->remote_root, file->filename, NULL);
-	    ne_md5_init_ctx(&md5);
-	    fe_checksumming(file->filename);
-	    if (CALL(file_read)(session, full_remote, 
-				site_fetch_csum_read, &md5) != SITE_OK) {
-		ret = 1;
-		fe_checksummed(full_remote, false, DRIVER_ERR);
-	    } else {
-		ne_md5_finish_ctx(&md5, file->checksum);
-		fe_checksummed(full_remote, true, NULL);
-	    }
-	    free(full_remote);
-	}
-    }
-    return ret;
+	return -1;
 }
     
 /* Updates the remote file list... site_fetch_callback is called for
@@ -1295,5 +1282,4 @@ void site_sock_progress_cb(void *userdata, off_t progress, off_t total)
 
 void fe_initialize(void)
 {
-    ne_sock_init();
 }
