@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1999-2000, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2004, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -253,7 +253,6 @@ struct site_file {
 
     enum file_type type:2;
     
-    unsigned int marked:1; /* whether the file is marked to be updated or not */
     unsigned int ignore:1; /* whether to ignore any changes to this file */
 
     /* Probably want to make the states into an array, so they can be
@@ -291,7 +290,6 @@ enum site_symlink_modes {
 enum site_protocol_modes {
     siteproto_ftp,
     siteproto_dav,
-    siteproto_rsh,
     siteproto_vfs,
     siteproto_unknown
 };
@@ -341,7 +339,8 @@ struct site_host {
 
 /* This represents a site */
 struct site {
-
+    void *user_data; /* added field for screem, points to the plugin */
+    
     char *name; /* symbolic name for site */
     char *url; /* URL for site - used by flatlist mode */
     
@@ -365,6 +364,7 @@ struct site {
 
     /* Options for the site */
     enum site_perm_modes perms; /* permissions maintenance mode */
+    int dirperms; /* directory permissions maintenance mode */
     enum site_symlink_modes symlinks; /* symlink handline mode */
 
     /* Protocol-driver specific options here */
@@ -389,7 +389,6 @@ struct site {
 
     /* These are parameters to site_update really. */
     unsigned int keep_going:1; /* if true, keep going past errors in updates */
-    unsigned int only_marked:1; /* if true, only act on marked files in updates */
 
     unsigned int use_this:1; /* whether the site is being operated on - handy
 			      * for the console FE */
@@ -496,10 +495,6 @@ int site_verify(struct site *site, int *numremoved);
  * called during the update. fe_can_update may be called during the
  * update if site->prompting is set.
  *
- * Only files with marked==true will be updated if site->only_marked == true.
- * WARNING: DO NOT USE only_marked==true LIGHTLY. You might mess up
- * the remote site. 
- *
  * Returns:
  *   SITE_ERRORS if an error occurred which was reported using
  *     the fe_update_* functions. site->last_error is undefined.
