@@ -1421,3 +1421,20 @@ egg_tree_model_union_remove (EggTreeModelUnion *umodel,
 
   model_map_free (map);
 }
+
+gboolean
+egg_tree_model_union_contains (EggTreeModelUnion *umodel,
+		               GtkTreeModel *model)
+{
+  GList *i;
+
+  g_return_val_if_fail (EGG_IS_TREE_MODEL_UNION (umodel), FALSE);
+  g_return_val_if_fail (GTK_IS_TREE_MODEL (model), FALSE);
+
+  for (i = umodel->root; i; i = i->next)
+    if (MODEL_MAP (i->data)->model == model)
+      break;
+
+  return (i != NULL);
+}
+
