@@ -82,6 +82,9 @@ gboolean copy_file( const gchar *source, const gchar *dest,
 
 	gboolean ret = FALSE;
 
+	g_return_val_if_fail( source != NULL, FALSE );
+	g_return_val_if_fail( dest != NULL, FALSE );
+	
 	if( ovr_mode == GNOME_VFS_XFER_OVERWRITE_MODE_QUERY ) {
 		if( ! overwrite( dest ) ) {
 			return FALSE;
