@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -26,10 +26,6 @@
 #include <config.h>
 
 #include <glib/gi18n.h>
-#include <libgnome/gnome-url.h>
-#include <libgnomeui/gnome-messagebox.h>
-#include <libgnomeui/gnome-stock-icons.h>
-#include <libgnomeui/gnome-entry.h>
 
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -59,41 +55,23 @@
 #include "screem-page.h"
 #include "screem-search.h"
 
+#include "gedit-close-confirmation-dialog.h"
+
 gboolean screem_page_open_with_filename( ScreemSite *site, 
 					 ScreemWindow *window,
 					 const gchar *filename )
 {
-	ScreemApplication *app;
+	GObject *app;
 	ScreemPage *page;
 	gchar *path;
 	const gchar *pathname;
-	GnomeVFSURI *uri;
 	GtkAction *action;
 	
 	g_return_val_if_fail( filename != NULL, FALSE );
 
-	uri = gnome_vfs_uri_new( filename );
-
-	if( ! uri ) {
-		return FALSE;
-	}
-	
 	g_object_get( G_OBJECT( site ), "app", &app, NULL );
-	
-	/* is the filename local? */
-	if( gnome_vfs_uri_is_local( uri ) ) {
-		/* yes */
-		/* HACK */
-		if( *filename == G_DIR_SEPARATOR ) {
-			path = g_strconcat( "file://", filename, NULL );
-		} else {
-			path = relative_to_full( filename, NULL );
-		}
-	} else {
-		path = g_strdup( filename );
-	}
-	gnome_vfs_uri_unref( uri );
-	
+
+	path = gnome_vfs_make_uri_from_input( filename );
 
 	page = screem_site_locate_page( site, path );
 
@@ -139,9 +117,9 @@ gboolean screem_page_open_with_filename( ScreemSite *site,
 }
 
 
-gint screem_page_save_as( ScreemPage *page, ScreemWindow *window )
+gint screem_page_save_as( ScreemPage *page, ScreemWindow *window,
+		const gchar *dir, ScreemApplication *app )
 {
-	ScreemApplication *app;
 	gchar *filename;
 	gint status;
 	ScreemSite *site;
@@ -157,10 +135,15 @@ gint screem_page_save_as( ScreemPage *page, ScreemWindow *window )
 	gint i;
 	static const gint num = G_N_ELEMENTS( filternames );
 
-	g_object_get( G_OBJECT( window ), "app", &app, NULL );
-	
-	site = screem_window_get_current( window );
-	pathname = screem_site_get_pathname( site );
+	site = NULL;
+	if( window ) {
+		site = screem_window_get_current( window );
+	}
+	if( site && ! dir ) {
+		pathname = screem_site_get_pathname( site );
+	} else {
+		pathname = dir;
+	}
 
 	filters = NULL;
 	for( i = 0; i < num; ++ i ) {
@@ -170,7 +153,7 @@ gint screem_page_save_as( ScreemPage *page, ScreemWindow *window )
 	filename = screem_fileselect( _( "Select filename to save as" ), 
 					GTK_WINDOW( window ),
 					GTK_FILE_CHOOSER_ACTION_SAVE,
-					pathname, filters );
+					pathname, filters, FALSE );
 	g_slist_free( filters );
 
 	/* we didn't select one */
@@ -187,9 +170,11 @@ gint screem_page_save_as( ScreemPage *page, ScreemWindow *window )
 				previous = g_strdup( previous );
 			}
 
-			/* FIXME: if we are overwriting an already existing file
+			/* FIXME: if we are overwriting an
+			 * already existing file
 			 *
-			 * 1) scan all windows, closing filename if it is open
+			 * 1) scan all windows, closing 
+			 * filename if it is open
 			 * 2) remove filename from site
 			 */
 			
@@ -219,20 +204,18 @@ gint screem_page_save_as( ScreemPage *page, ScreemWindow *window )
 				npage = screem_page_new( G_OBJECT( app ) );
 				screem_page_set_pathname( npage, previous );
 				screem_site_add_page( site, npage );
-				g_object_unref( page );
+				g_object_unref( npage );
 			}
 			g_free( previous );
 		}
 	}
-
-	g_object_unref( app );
 	
 	return status;
 }
 
-gint screem_page_save_confirmation( ScreemPage *page, gboolean closing )
+gint screem_page_save_confirmation( ScreemPage *page, 
+		ScreemWindow *window, gboolean closing )
 {
-	gchar *gladepath;
 	GladeXML *xml;
 	GtkWidget *widget;
 	const gchar *label;
@@ -247,6 +230,8 @@ gint screem_page_save_confirmation( ScreemPage *page, gboolean closing )
 	gchar *basename;
 
 	const gchar *name;
+
+	g_assert( closing == FALSE );
 	
 	if( ! screem_page_is_loaded( page ) ) {
 		return GTK_RESPONSE_NO;
@@ -254,15 +239,9 @@ gint screem_page_save_confirmation( ScreemPage *page, gboolean closing )
 		return GTK_RESPONSE_NO;
 	}
 
-	if( closing ) {
-		name = "save";
-	} else {
-		name = "save_cont";
-	}
+	name = "save_cont";
 
-	gladepath = screem_get_glade_path();
-	xml = glade_xml_new( gladepath, name, NULL );
-	g_free( gladepath );
+	xml = glade_xml_new( GLADE_PATH"/screem.glade", name, NULL );
 	
 	widget = glade_xml_get_widget( xml, "text" );
 	label = gtk_label_get_text( GTK_LABEL( widget ) );
@@ -318,16 +297,22 @@ gboolean screem_page_close_page( ScreemPage *page, ScreemWindow *window,
 			     gboolean confirm )
 {
 	gboolean ret;
+	ScreemApplication *app;
 	
 	/* request save confirmation */
+
+	g_object_get( G_OBJECT( window ), "app", &app, NULL );
+	
 	ret = FALSE;
 	if( ( ! confirm ) || 
-		screem_page_save_confirmation( page, TRUE ) != GTK_RESPONSE_CANCEL ) {
+		screem_page_save_page_confirmation( page, window, app ) != GTK_RESPONSE_CANCEL ) {
 		screem_page_set_open( page, FALSE );
 		screem_window_remove_document( window, page );
 		ret = TRUE;
 	}
 
+	g_object_unref( G_OBJECT( app ) );
+	
 	return ret;
 }
 
@@ -396,16 +381,18 @@ void external_view( ScreemPage *page )
 {
         const gchar *pathname;
 	gchar *uri;
+	GnomeVFSResult res;
 
         pathname = screem_page_get_pathname( page );
 
         g_return_if_fail( pathname != NULL );
 
-        if( screem_page_save_confirmation( page, FALSE ) != GTK_RESPONSE_CANCEL ) {
+        if( screem_page_save_confirmation( page, NULL, FALSE ) != GTK_RESPONSE_CANCEL ) {
 		uri = g_strconcat( "file://", pathname, NULL );
-		
-		gnome_url_show( uri, NULL );
-		
+	
+		res = gnome_vfs_url_show( uri );
+		/* FIXME: handle errors */
+
 		g_free( uri );
 	}
 }
@@ -415,7 +402,6 @@ void screem_page_edit_bookmarks( ScreemWindow *window, ScreemEditor *editor,
 				 ScreemPage *page )
 {
 	GladeXML *xml;
-	gchar *gladepath;
 
 	GtkWidget *dialog;
 	gint response;
@@ -429,13 +415,18 @@ void screem_page_edit_bookmarks( ScreemWindow *window, ScreemEditor *editor,
 	const gchar *pathname;
 	gchar *temp;
 
-	gladepath = screem_get_glade_path();
-	xml = glade_xml_new( gladepath, "bookmarks", NULL );
-	g_free( gladepath );
+	GtkWidget *treeview;
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *col;
+	GtkListStore *store;
+	
+	xml = glade_xml_new( GLADE_PATH"/screem.glade", 
+			"bookmarks", NULL );
 
 	dialog = glade_xml_get_widget( xml, "bookmarks" );
 	bookmarkname = glade_xml_get_widget( xml, "bookmarkname" );
-	bookmarkname = gnome_entry_gtk_entry( GNOME_ENTRY( bookmarkname ) );
+	screem_gtk_add_history( bookmarkname );
+	bookmarkname = GTK_BIN( bookmarkname )->child;
 
 	bookmarkline = glade_xml_get_widget( xml, "bookmarkline" );
 	line = gtk_text_buffer_get_line_count( GTK_TEXT_BUFFER( page ) );
@@ -446,38 +437,30 @@ void screem_page_edit_bookmarks( ScreemWindow *window, ScreemEditor *editor,
 	allowjump = ( page == screem_window_get_document( window ) );
 	gtk_widget_set_sensitive( jumpto, allowjump );
 
-	{
-		GtkWidget *treeview;
-		GtkCellRenderer *renderer;
-		GtkTreeViewColumn *col;
-		GtkListStore *store;
+	treeview = glade_xml_get_widget( xml, "bookmarkview" );
+	g_object_set_data( G_OBJECT( treeview ), "page", page );
+	
+	renderer = gtk_cell_renderer_text_new();
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title( col, "Name" );
+	gtk_tree_view_column_pack_start( col, renderer, TRUE );
+	gtk_tree_view_append_column( GTK_TREE_VIEW( treeview ),
+				     col );
+	gtk_tree_view_column_set_attributes( col, renderer,
+					     "text", 0, NULL );
 
-		treeview = glade_xml_get_widget( xml, "bookmarkview" );
-		g_object_set_data( G_OBJECT( treeview ), "page", page );
-		
-		renderer = gtk_cell_renderer_text_new();
-		col = gtk_tree_view_column_new();
-		gtk_tree_view_column_set_title( col, "Name" );
-		gtk_tree_view_column_pack_start( col, renderer, TRUE );
-		gtk_tree_view_append_column( GTK_TREE_VIEW( treeview ),
-					     col );
-		gtk_tree_view_column_set_attributes( col, renderer,
-						     "text", 0, NULL );
-
-		renderer = gtk_cell_renderer_text_new();
-		col = gtk_tree_view_column_new();
-		gtk_tree_view_column_set_title( col, "Line" );
-		gtk_tree_view_column_pack_start( col, renderer, TRUE );
-		gtk_tree_view_append_column( GTK_TREE_VIEW( treeview ),
-					     col );
-		gtk_tree_view_column_set_attributes( col, renderer,
-						     "text", 1, NULL );
-
-		store = screem_page_get_bookmark_model( page );
-		gtk_tree_view_set_model( GTK_TREE_VIEW( treeview ),
-					 GTK_TREE_MODEL( store ) );
-	}
+	renderer = gtk_cell_renderer_text_new();
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title( col, "Line" );
+	gtk_tree_view_column_pack_start( col, renderer, TRUE );
+	gtk_tree_view_append_column( GTK_TREE_VIEW( treeview ),
+				     col );
+	gtk_tree_view_column_set_attributes( col, renderer,
+					     "text", 1, NULL );
 
+	store = screem_page_get_bookmark_model( page );
+	gtk_tree_view_set_model( GTK_TREE_VIEW( treeview ),
+				 GTK_TREE_MODEL( store ) );
 
 	pathname = screem_page_get_pathname( page );
 	if( ! pathname ) {
@@ -558,7 +541,7 @@ void screem_page_bookmark_activate( GtkTreeView *view,
 		name = glade_xml_get_widget( xml, "bookmarkname" );
 		line = glade_xml_get_widget( xml, "bookmarkline" );
 
-		name = gnome_entry_gtk_entry( GNOME_ENTRY( name ) );
+		name = GTK_BIN( name )->child;
 		gtk_entry_set_text( GTK_ENTRY( name ), 
 				    bookmark->name->str );
 
@@ -567,4 +550,73 @@ void screem_page_bookmark_activate( GtkTreeView *view,
 	}
 }
 
+gint screem_page_save_page_confirmation( ScreemPage *page,
+		ScreemWindow *window, ScreemApplication *app )
+{
+	GSList *tmp;
+	gint ret;
+
+	ret = GTK_RESPONSE_NO;
+	
+	if( screem_page_is_loaded( page ) &&
+	    screem_page_get_changed( page ) ) {
+		tmp = g_slist_prepend( NULL, page );
+		ret = screem_page_save_pages_confirmation( tmp,
+				window, app );
+		g_slist_free( tmp );
+	}
+
+	return ret;
+}
+
+gint screem_page_save_pages_confirmation( GSList *pages,
+		ScreemWindow *window, ScreemApplication *app )
+{
+	GtkWindow *gwindow;
+	GtkWidget *confirm;
+	gint ret;
+	gboolean doclose;
+	GSList *saveme;
+	GSList *tmp;
+	ScreemPage *page;
+	const gchar *pathname;
+
+	g_return_val_if_fail( pages != NULL, GTK_RESPONSE_NO );
+	
+	gwindow = NULL;
+	if( window ) {
+		gwindow = GTK_WINDOW( window );
+	}
+	
+	if( ! pages->next ) {
+		confirm = gedit_close_confirmation_dialog_new_single( gwindow, pages->data );
+	} else {
+		confirm = gedit_close_confirmation_dialog_new( gwindow,
+				pages );
+	}
+	ret = GTK_RESPONSE_CANCEL;
+	saveme = NULL;
+	if( gedit_close_confirmation_dialog_run( confirm ) ) {
+		ret = GTK_RESPONSE_NO;
+		saveme = gedit_close_confirmation_dialog_get_selected_documents( confirm );
+	}
+	if( saveme ) {
+		for( tmp = saveme; tmp; tmp = tmp->next ) {
+			page = SCREEM_PAGE( tmp->data );
+			pathname = screem_page_get_pathname( page );
+			if( pathname ) {
+				screem_page_save( page, NULL );
+			} else {
+				screem_page_save_as( page,
+						window, NULL,
+						app );
+			}
+		}
+		g_slist_free( saveme );
+		ret = GTK_RESPONSE_YES;
+	}
+	gtk_widget_destroy( confirm );
+
+	return ret;
+}
 
