@@ -475,6 +475,7 @@ void screem_application_startup( ScreemApplication *application,
 
 	/* create the fake site for editing individual files */
 	site = screem_site_new( G_OBJECT( application ) );
+	screem_site_set_pathname( site, NULL );
 	screem_application_add_site( application, site );
 
 	window = NULL;
@@ -569,7 +570,7 @@ static gboolean screem_application_load( ScreemApplication *application )
 	ScreemApplicationPrivate *priv;
 	GSList *start_files;
 	gchar *sfile;
-	gchar *mime_type;
+	const gchar *mime_type;
 	gchar *file;
 	ScreemWindow *window;
 
@@ -617,7 +618,6 @@ static gboolean screem_application_load( ScreemApplication *application )
 			screem_page_open_with_filename( site, window, file );
 	      		gdk_threads_leave();
 		}
-		g_free( mime_type );
 		g_free( file );
 
 		g_free( sfile );
@@ -1515,6 +1515,9 @@ static void screem_application_finalize( GObject *object )
 		g_object_unref( G_OBJECT( lm ) );
 	}
 
+	/* causes the cache to free itself up */
+	(void)screem_get_mime_type_override( NULL, NULL );
+	
 	if( priv->encodings ) {
 		g_object_unref( priv->encodings );
 	}
