@@ -14,12 +14,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #include "config.h"
 
-#include <libgnome/gnome-macros.h>
 #include <glib/gi18n.h>
 
 #include <gtk/gtkcellrenderertext.h>
@@ -33,13 +32,6 @@
 
 #include "screemmarshal.h"
 
-static void screem_ctags_view_class_init( ScreemCtagsViewClass *klass );
-static void screem_ctags_view_instance_init( ScreemCtagsView *ctags_view );
-static void screem_ctags_view_finalize( GObject *object );
-static void screem_ctags_view_set_prop( GObject *object, guint prop_id,
-				const GValue *value, GParamSpec *spec );
-static void screem_ctags_view_get_prop( GObject *object, guint prop_id,
-				GValue *value, GParamSpec *spec );
 
 static void row_activated( GtkTreeView *view, GtkTreePath *path,
 			GtkTreeViewColumn *column, gpointer *data );
@@ -59,9 +51,13 @@ enum {
 
 static guint screem_ctags_view_signals[ LAST_SIGNAL ] = { 0 };
 
-GNOME_CLASS_BOILERPLATE( ScreemCtagsView, screem_ctags_view,
-			  GtkTreeView,
-			  GTK_TYPE_TREE_VIEW )
+G_DEFINE_TYPE( ScreemCtagsView, screem_ctags_view, GTK_TYPE_TREE_VIEW )
+
+static void screem_ctags_view_finalize( GObject *object );
+static void screem_ctags_view_set_prop( GObject *object, guint prop_id,
+				const GValue *value, GParamSpec *spec );
+static void screem_ctags_view_get_prop( GObject *object, guint prop_id,
+				GValue *value, GParamSpec *spec );
 
 static void screem_ctags_view_class_init( ScreemCtagsViewClass *klass )
 {
@@ -86,7 +82,7 @@ static void screem_ctags_view_class_init( ScreemCtagsViewClass *klass )
 			      G_TYPE_UINT );
 }
 
-static void screem_ctags_view_instance_init( ScreemCtagsView *ctags_view )
+static void screem_ctags_view_init( ScreemCtagsView *ctags_view )
 {
 	ScreemCtagsViewPrivate *priv;
 	GtkTreeView *view;
@@ -136,7 +132,7 @@ static void screem_ctags_view_finalize( GObject *object )
 {
 	ScreemCtagsView *ctags_view;
 	ScreemCtagsViewPrivate *priv;
-	
+				
 	g_return_if_fail( object != NULL );
 	g_return_if_fail( SCREEM_IS_CTAGS_VIEW( object ) );
 
@@ -146,7 +142,7 @@ static void screem_ctags_view_finalize( GObject *object )
 
 	g_free( priv );
 	
-	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
+	G_OBJECT_CLASS( screem_ctags_view_parent_class )->finalize( object );
 }
 
 static void screem_ctags_view_set_prop( GObject *object, guint prop_id,
@@ -188,8 +184,6 @@ static void row_activated( GtkTreeView *view, GtkTreePath *path,
 	ScreemCtagsViewPrivate *priv;
 	GtkTreeModel *model;
 	GtkTreeIter it;
-	gchar *mpath;
-	gchar *pathname;
 	gchar *file;
 	gchar *pattern;
 	guint line;
@@ -205,20 +199,11 @@ static void row_activated( GtkTreeView *view, GtkTreePath *path,
 				CTAGS_PATTERN_COL, &pattern,
 				CTAGS_LINE_COL, &line,
 				-1 );
-		mpath = NULL;
 		if( file ) {
-			mpath = screem_ctags_model_get_pathname( SCREEM_CTAGS_MODEL( model ) );
-		}
-		if( mpath ) {
-			pathname = g_build_path( G_DIR_SEPARATOR_S,
-					mpath, file, NULL );
-			g_free( mpath );
-			
 			g_signal_emit( G_OBJECT( view ),
 				screem_ctags_view_signals[ JUMPTO ], 0,
-				pathname, pattern, line );
+				file, pattern, line );
 
-			g_free( pathname );
 		}
 		g_free( file );
 		g_free( pattern );
