@@ -664,7 +664,10 @@ static gboolean screem_dtd_db_load_internal( ScreemDTDDB *db,
 
 	file = NULL;
 	local = FALSE;
-	uri = gnome_vfs_uri_new( systemid );
+	uri = NULL;
+	if( systemid ) {
+		uri = gnome_vfs_uri_new( systemid );
+	}
 	if( uri ) {
 		local = gnome_vfs_uri_is_local( uri );
 		if( ( ! priv->offline ) || local ) {
