@@ -16,14 +16,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <libgnome/gnome-macros.h>
+#include <config.h>
 
 #include <glib.h>
 
@@ -38,14 +34,6 @@
 
 #include "fileops.h"
 
-static void screem_dtd_class_init( ScreemDTDClass *klass );
-static void screem_dtd_instance_init( ScreemDTD *dtd );
-static void screem_dtd_finalize( GObject *object );
-static void screem_dtd_set_prop( GObject *object, guint prop_id,
-                                const GValue *value, GParamSpec *spec );
-static void screem_dtd_get_prop( GObject *object, guint prop_id,
-                                GValue *value, GParamSpec *spec );
-
 enum {
 	PROP_0,
 	PROP_PUBLIC,
@@ -72,6 +60,8 @@ struct ScreemDTDPrivate {
 
 	gchar *rooturi;
 	const gchar *toproot;
+
+	GStringChunk *chunk;
 };
 
 struct ScreemDTDEntity {
@@ -162,6 +152,8 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 					      const gchar *data );
 static const gchar *screem_dtd_parse_attlist( ScreemDTD *dtd,
 					      const gchar *data );
+static const gchar *screem_dtd_parse_notation( ScreemDTD *dtd,
+					      const gchar *data );
 
 static const gchar *screem_dtd_name_parse( ScreemDTD *dtd,
 					const gchar *pattern, 
@@ -197,13 +189,20 @@ static void screem_dtd_free_attribute( ScreemDTDAttribute *attr );
 
 static gint screem_dtd_sort_element_list( gconstpointer a,
 					gconstpointer b );
-static gint screem_dtd_sort_attr_list( gconstpointer a,
+static gint screem_dtd_reverse_sort_attr_list( gconstpointer a,
 					gconstpointer b,
 					gpointer data );
 static void screem_dtd_build_optional( gpointer data, 
 					gpointer user_data );
 
-GNOME_CLASS_BOILERPLATE( ScreemDTD, screem_dtd, GObject, G_TYPE_OBJECT )
+G_DEFINE_TYPE( ScreemDTD, screem_dtd, G_TYPE_OBJECT )
+
+static void screem_dtd_finalize( GObject *object );
+static void screem_dtd_set_prop( GObject *object, guint prop_id,
+                                const GValue *value, GParamSpec *spec );
+static void screem_dtd_get_prop( GObject *object, guint prop_id,
+                                GValue *value, GParamSpec *spec );
+
 
 static void screem_dtd_class_init( ScreemDTDClass *klass )
 {
@@ -237,15 +236,21 @@ static void screem_dtd_class_init( ScreemDTDClass *klass )
 			PROP_COMPARE, pspec );
 }
 
-static void screem_dtd_instance_init( ScreemDTD *dtd )
+static void screem_dtd_init( ScreemDTD *dtd )
 {
 	dtd->priv = g_new0( ScreemDTDPrivate, 1 );
+
+#define SENSIBLE_TAG_SIZE 16
+	dtd->priv->chunk = g_string_chunk_new( SENSIBLE_TAG_SIZE );
 }
 
 static void screem_dtd_finalize( GObject *object )
 {
 	ScreemDTD *dtd;
 	ScreemDTDPrivate *priv;
+			
+	g_return_if_fail( object != NULL );
+	g_return_if_fail( SCREEM_IS_DTD( object ) );
 
 	dtd = SCREEM_DTD( object );
 	priv = dtd->priv;
@@ -268,12 +273,14 @@ static void screem_dtd_finalize( GObject *object )
 	/* free attributes */
 	g_hash_table_destroy( priv->attlists );
 
+	g_string_chunk_free( priv->chunk );
+	
 	g_free( priv->root );
 	g_free( priv->rooturi );
 	
 	g_free( priv );	
 
-	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
+	G_OBJECT_CLASS( screem_dtd_parent_class )->finalize( object );
 }
 
 static void screem_dtd_set_prop( GObject *object, guint prop_id,
@@ -515,6 +522,7 @@ static const gchar *screem_dtd_bypass_comment( const gchar *data,
 {
 	const gchar *ret;
 	const gchar *start;
+	guint len;
 	
 	ret = data;
 
@@ -532,18 +540,23 @@ static const gchar *screem_dtd_bypass_comment( const gchar *data,
 	} else {
 		g_warning( "Non comment start passed to bypass comment\n" );
 	}
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 	start = data;
 
-	while( (ret = g_utf8_strchr( data, strlen( data ), '-' ) ) ) {
+	len = g_utf8_strlen( data, -1 );
+	
+	while( (ret = g_utf8_strchr( data, len, '-' ) ) ) {
+		len -= g_utf8_pointer_to_offset( data, ret );
 		data = g_utf8_next_char( ret );
+		len --;
 		if( *data == '-' ) {
 			ret = g_utf8_next_char( data );
+			len --;
 			break;
 		}
 	} 
 	if( ! ret ) {
-		ret = g_utf8_strchr( data, strlen( data ), '\0' );
+		ret = g_utf8_strchr( data, len, '\0' );
 	}
 	
 	if( txt ) {
@@ -560,7 +573,7 @@ static const gchar *screem_dtd_read_entity( const gchar *data,
 	gunichar c;
 	GString *str;
 
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 
 	if( *data != '%' ) {
 		g_warning( "read entity called on non entity" );
@@ -619,12 +632,12 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 	}
 
 	data += strlen( "ENTITY " );
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 	
 	general = (*data != '%');
 	if( ! general ) {
 		data = g_utf8_next_char( data );
-		data = g_utf8_skip_space( data );
+		data = screem_utf8_skip_space( data );
 	}
 
 	/* data should now point at the entity name */
@@ -641,12 +654,12 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 		}
 		data = g_utf8_next_char( data );
 	}
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 
 	/* check for a comment in the way */
 	if( *data == '-' && data[ 1 ] == '-' ) {
 		data = screem_dtd_bypass_comment( data, NULL );
-		data = g_utf8_skip_space( data );
+		data = screem_utf8_skip_space( data );
 	}
 	
 	/* data should now point at the entity value */
@@ -670,7 +683,7 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 	    value->str && ! strcmp( "PUBLIC", value->str ) ) {
 		/* read public id */
 		g_string_append_c( value, ' ' );
-		data = g_utf8_skip_space( data );
+		data = screem_utf8_skip_space( data );
 		term = g_utf8_get_char( data );
 		if( term != '"' && term != '\'' ) {
 			term = ' ';
@@ -689,7 +702,7 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 		if( *data != '\0' ) {
 			g_string_append_unichar( value, term );
 			data = g_utf8_next_char( data );
-			data = g_utf8_skip_space( data );
+			data = screem_utf8_skip_space( data );
 		}
 		c = g_utf8_get_char( data );
 		if( c == '-' && *( data + 1 ) == '-' ) {
@@ -711,7 +724,7 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 		! strcmp( "SYSTEM", value->str ) ) ) {
 		/* read system id */
 		g_string_append_c( value, ' ' );
-		data = g_utf8_skip_space( data );
+		data = screem_utf8_skip_space( data );
 		term = g_utf8_get_char( data );
 		if( term != '"' && term != '\'' ) {
 			term = ' ';
@@ -743,22 +756,19 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 		}
 	}
 
-	entity = NULL;
-	if( name->str && value->str ) {		
-		if( ! g_hash_table_lookup( priv->entities,
+	if( name->str && value->str &&
+		! g_hash_table_lookup( priv->entities,
 					   name->str ) ) {
-			entity = g_new0( ScreemDTDEntity, 1 );
-			entity->name = name->str;
-			entity->value = value->str;
-			entity->general = general;
+		entity = g_new0( ScreemDTDEntity, 1 );
+		entity->name = name->str;
+		entity->value = value->str;
+		entity->general = general;
 
-			g_hash_table_insert( priv->entities, entity->name,
-					     entity );
-			g_string_free( name, FALSE );
-			g_string_free( value, FALSE );
-		}
-	} 
-	if( ! entity ) {
+		g_hash_table_insert( priv->entities, entity->name,
+				entity );
+		g_string_free( name, FALSE );
+		g_string_free( value, FALSE );
+	} else  {
 		g_string_free( name, TRUE );
 		g_string_free( value, TRUE);
 		g_free( desc );
@@ -808,7 +818,7 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 	}
 
 	data += strlen( "ELEMENT " );
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 	
 	/* data should now be at the start of the element name */
 	group = NULL;
@@ -824,7 +834,7 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 	name_group = screem_dtd_group_flatten( group, NULL );
 	screem_dtd_group_free( group, NULL );
 
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 
 	/* check for open/close options,
 	 *
@@ -841,7 +851,16 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 		val = screem_dtd_lookup_entity( dtd, name, FALSE );
 		g_free( name );
 		if( val ) {
-			data = val;
+			if( *val != '\0' ) {
+				data = val;
+			} else {
+				g_free( val );
+				data = ndata;
+				ndata = NULL;
+			}
+		} else {
+			data = ndata;
+			ndata = NULL;
 		}
 	}
 	
@@ -883,7 +902,7 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 		data += offset;
 	}
 
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 
 	/* data should now be at the start of the allowed children */
 	data = screem_dtd_name_parse( dtd, data, &group );
@@ -891,19 +910,19 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 	/* group will contain allowed elements, it possibly doesn't
 	   contain all exclusions / inclusions however */
 
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 	exclude = include = NULL;
 	if( *data != '>' && ( *data =='-' && data[ 1 ] != '-' ) ) {
 		/* exclusion */
 		data = screem_dtd_name_parse( dtd, data, &exclude );
 	}
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 	if( *data != '>' && *data =='+' ) {
 		/* inclusion */
 		data = screem_dtd_name_parse( dtd, data, &include );
 	}
 
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 
 	/* check for a description of the element as a comment */
 	description = NULL;
@@ -928,11 +947,13 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 			if( tmp->type == SCREEM_DTD_EXCLUDE && 
 				allowed ) {
 				g_hash_table_remove( table, tmp->name );
-				g_free( allowed->name );
+				/* string chunk used do not free
+				 * this g_free( allowed->name ); */
 				g_free( allowed );
 			} else if( ! allowed ) {
 				allowed = g_new( ScreemDTDAllowed, 1 );
-				allowed->name = g_strdup( tmp->name );
+				allowed->name = g_string_chunk_insert_const( priv->chunk, tmp->name );
+					/*g_strdup( tmp->name )*/;
 				allowed->min = tmp->min;
 				allowed->max = tmp->max;
 				allowed->ref = 1;
@@ -956,7 +977,8 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 							tmp->name );
 			if( allowed ) {
 				g_hash_table_remove( table, tmp->name );
-				g_free( allowed->name );
+				/* string chunk used do not free this
+				 * g_free( allowed->name ); */
 				g_free( allowed );
 			}
 		}
@@ -974,7 +996,8 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 		    tmp->type != SCREEM_DTD_EXCLUDE &&
 		    ! g_hash_table_lookup( table, tmp->name ) ) {
 			allowed = g_new( ScreemDTDAllowed, 1 );
-			allowed->name = g_strdup( tmp->name );
+			allowed->name = g_string_chunk_insert_const( priv->chunk, tmp->name );
+				/*g_strdup( tmp->name );*/
 			allowed->min = tmp->min;
 			allowed->max = tmp->max;
 			allowed->ref = 1;
@@ -997,7 +1020,8 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 		if( tmp->name ) {
 			element = g_new0( ScreemDTDElement, 1 );
 			element->dtd = dtd;
-			element->name = g_strdup( tmp->name );
+			element->name = g_string_chunk_insert_const( priv->chunk, tmp->name );
+				/*g_strdup( tmp->name );*/
 			element->ostate = ostate;
 			element->cstate = cstate;
 			if( description ) {
@@ -1044,6 +1068,7 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 					const gchar *data, 
 					GSList **attrs )
 {
+	ScreemDTDPrivate *priv;
 	ScreemDTDAttribute *attr;
 	gunichar c;
 	gchar *ename;
@@ -1060,6 +1085,8 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 	g_return_val_if_fail( data != NULL, NULL );
 	g_return_val_if_fail( attrs != NULL, data );
 
+	priv = dtd->priv;
+	
 	alist = *attrs;
 
 	/* get current attribute or create new one */
@@ -1074,7 +1101,7 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 
 	str = g_string_new( NULL );
 	fixed = FALSE;
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 	while( *data != '\0' && *data != '>' ) {
 		c = g_utf8_get_char( data );
 		type = g_unichar_type( c );
@@ -1082,12 +1109,12 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 		if( c == '\0' || c == '>' ) {
 			break;
 		} else if( c == '-' && data[ 1 ] == '-' ) {
-			data = screem_dtd_bypass_comment( data,
-						&tmp );
 			if( ! attr->description ) {
-				attr->description = tmp;
+				data = screem_dtd_bypass_comment( data,
+						&attr->description );
 			} else {
-				g_free( tmp );
+				data = screem_dtd_bypass_comment( data,
+						NULL );
 			}
 		} else if( ! attr->name && c == '%' ) {
 			data = screem_dtd_read_entity( data, &ename );
@@ -1103,7 +1130,7 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 					val = g_strconcat( str->str,
 							tmp, NULL );
 					g_free( tmp );
-					g_string_assign( str, "" );
+					g_string_truncate( str, 0 );
 				}
 					
 				/* parse val */
@@ -1121,7 +1148,9 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 				if( attr && attr->name ) {
 					g_string_assign( str,
 							attr->name );
-					g_free( attr->name );
+					/* string chunk is used, do
+					 * not free 
+					 * g_free( attr->name ); */
 					attr->name = NULL;
 				}
 			}
@@ -1136,30 +1165,37 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 				g_string_append_unichar( str, c );
 			} else if( str->len > 0 ) {
 				/* name parse done */
-				attr->name = g_strdup( str->str );
-				g_string_assign( str, "" );
+				attr->name = g_string_chunk_insert_const( priv->chunk, str->str );
+					/*g_strdup( str->str );*/
+				g_string_truncate( str, 0 );
 
-				data = g_utf8_skip_space( data );
+				data = screem_utf8_skip_space( data );
 				data = g_utf8_prev_char( data );
 			}
 		} else if( ! attr->values ) {
 			/* parsing values, check for uri type, then
 			   do a normal name read */
-			data = g_utf8_skip_space( data );	
-			attr->isuri = ( ! strncmp( "%URI;", data, strlen( "%URI;" ) ) ||
-				  ! strncmp( "%URL;", data, strlen( "%URL;" ) ) ||
-				  ! strncmp( "%URL ", data, strlen( "%URL " ) ) ||
-				  ! strncmp( "%URI.datatype;", data, 
-					     strlen( "%URI.datatype;" ) ) );
+			data = screem_utf8_skip_space( data );	
 
-			/* some doctypes may not declare a URI type,
+			/* %URI; or %URL; or %URI  or %URL  or
+			 * %URI.* %URL.* 
+			 *
+			 * some doctypes may not declare a URI type,
 			 * look at the attribute name and guess
-			 * based on that */
-			if( ! attr->isuri ) {
-				attr->isuri = ( ! strcmp( "url",
-							attr->name ) );
-			}
-			
+			 * based on that 
+			 */
+
+			attr->isuri = ( ( data[ 0 ] == '%' &&
+					data[ 1 ] == 'U' &&
+					data[ 2 ] == 'R' &&
+					( data[ 3 ] == 'I' ||
+					  data[ 3 ] == 'L' ) &&
+					( data[ 4 ] == ';' ||
+					  data[ 4 ] == ' ' ||
+					  data[ 4 ] == '.' ) ) ||
+					( ! strcmp( "url", 
+						    attr->name ) ) );
+
 			data = screem_dtd_name_parse( dtd, data, 
 							&group );
 			flat = NULL;
@@ -1177,12 +1213,12 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 			if( flat ) {
 				screem_dtd_group_free( flat, NULL );
 				group = NULL;
-			} else {
+			} else { 
 				/* failed to get a single value,
 				   default to CDATA so we have something */
 				   attr->values = g_slist_prepend( NULL, g_strdup( "CDATA" ) );
 			}
-			data = g_utf8_skip_space( data );
+			data = screem_utf8_skip_space( data );
 			data = g_utf8_prev_char( data );
 		} else if( c == '#' ) {
 			/* #IMPLIED or #REQUIRED, or #FIXED */
@@ -1203,8 +1239,12 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 				c = g_utf8_get_char( data );
 			}
 			if( attr->required == SCREEM_DTD_FIXED ) {
-				data = g_utf8_skip_space( data );
-				/* parse default value */
+				data = screem_utf8_skip_space( data );
+				/* parse default value.
+				 *
+				 * FIXME: this is broken, we don't
+				 * want to call screem_dtd_name_parse()
+				 * */
 				data = screem_dtd_name_parse( dtd, data,
 							&group );
 				if( group && group->name ) {
@@ -1220,18 +1260,18 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 					data = g_utf8_next_char( data );
 				}
 			}
-			data = g_utf8_skip_space( data );
+			data = screem_utf8_skip_space( data );
 			data = g_utf8_prev_char( data );
 		} else if( attr->required == SCREEM_DTD_NO_TYPE &&
 			   ! attr->default_value ) {
-			data = g_utf8_skip_space( data );
+			data = screem_utf8_skip_space( data );
 			/* parse default value */
 			data = screem_dtd_name_parse( dtd, data,
 						&group );
-			if( group && group->name ) {
-				attr->default_value = g_strdup( group->name );
-			}
 			if( group ) {
+				if( group->name ) {
+					attr->default_value = g_strdup( group->name );
+				}
 				screem_dtd_group_free( group, NULL );
 				group = NULL;
 			}
@@ -1252,7 +1292,8 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 	if( ! attr->name ) {
 		/* have a string, so use that as the attr->name */
 		if( str->len ) {
-			attr->name = g_strdup( str->str );
+			attr->name = g_string_chunk_insert_const( priv->chunk, str->str );
+				/*g_strdup( str->str );*/
 		} else {
 			if( alist ) {
 				*attrs = alist->next;
@@ -1291,7 +1332,7 @@ static const gchar *screem_dtd_parse_attlist( ScreemDTD *dtd,
 	}
 
 	data += strlen( "ATTLIST " );
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 	
 	/* data should now be at the start of the element name */
 	group = NULL;
@@ -1305,28 +1346,18 @@ static const gchar *screem_dtd_parse_attlist( ScreemDTD *dtd,
 		screem_dtd_group_free( group, NULL );
 	}
 
-	data = g_utf8_skip_space( data );
+	data = screem_utf8_skip_space( data );
 	list = NULL;
 	data = screem_dtd_parse_attlist_attrs( dtd, data, &list );
 	if( list ) {
 		list = g_slist_sort_with_data( list, 
-				screem_dtd_sort_attr_list, dtd );
+				screem_dtd_reverse_sort_attr_list, dtd );
 	
 		/* copy attributes to all elements */ 
 		for( tmp = name_group; tmp; tmp = tmp->siblings ) {
 			if( tmp->name ) {
 				element = g_hash_table_lookup( priv->elements,
 								tmp->name );
-				/* a crash was occuring / valgrind
-				 * was giving errors when we
-				 * were using
-				 * copy = g_slist_copy( list );
-				 * theoretically that should be no
-				 * different to what is here now,
-				 * prepending each item, and reversing
-				 * should be identical, however the
-				 * crash goes, and valgrind stops
-				 * warning like this */
 				if( element ) {
 					copy = NULL;
 					for( tlist = list; tlist; 
@@ -1338,9 +1369,18 @@ static const gchar *screem_dtd_parse_attlist( ScreemDTD *dtd,
 							copy = g_slist_prepend( copy, attribute );
 						}
 					}
-					/* XMLResume has multiple ATTLISTs for
-					 * elements, we need to allow that */
-					copy = g_slist_reverse( copy );
+					/* XMLResume has multiple 
+					 * ATTLISTs for
+					 * elements, we need to allow 
+					 * that 
+					 *
+					 * we don't need to reverse
+					 * copy before the concat
+					 * as we sorted list into
+					 * reverse order before then
+					 * copy will be in correct order
+					 * now
+					 */
 					element->attrlist = g_slist_concat( element->attrlist, copy );
 				}
 			}
@@ -1356,6 +1396,160 @@ static const gchar *screem_dtd_parse_attlist( ScreemDTD *dtd,
 	return data;
 }
 
+static const gchar *screem_dtd_parse_notation( ScreemDTD *dtd,
+		const gchar *data )
+{
+	ScreemDTDPrivate *priv;
+	ScreemDTDEntity *entity;
+	GString *name;
+	GString *value;
+	gunichar c;
+	gunichar eterm;
+	gunichar term;
+	gchar *desc;
+	GUnicodeType type;
+
+	g_return_val_if_fail( SCREEM_IS_DTD( dtd ), data );
+	
+	priv = dtd->priv;
+	
+	/* check this is a notation */
+	
+	if( strncmp( "NOTATION ", data, strlen( "NOTATION " ) ) ) {
+		return data;
+	}
+
+	data += strlen( "NOTATION " );
+	data = screem_utf8_skip_space( data );
+	
+	/* data should now point at the notation name */
+	name = g_string_new( NULL );
+	while( *data != '\0' ) {
+		c = g_utf8_get_char( data );
+		type = g_unichar_type( c );
+		if( g_unichar_isalnum( c ) ||
+		    c == '.' || c == '-' || c == '_' ||
+		    type == G_UNICODE_COMBINING_MARK ) {
+			g_string_append_unichar( name, c );
+		} else {
+			break;
+		}
+		data = g_utf8_next_char( data );
+	}
+	data = screem_utf8_skip_space( data );
+
+	/* check for a comment in the way */
+	if( *data == '-' && data[ 1 ] == '-' ) {
+		data = screem_dtd_bypass_comment( data, NULL );
+		data = screem_utf8_skip_space( data );
+	}
+	
+	/* data should now point at the entity value */
+	value = g_string_new( NULL );
+	eterm = g_utf8_get_char( data );
+	if( eterm != '"' && eterm != '\'' ) {
+		eterm = ' ';
+	} else {
+		data = g_utf8_next_char( data );
+	}
+	while( *data != '\0' ) {
+		c = g_utf8_get_char( data );
+		if( c == eterm || ( eterm == ' ' && g_unichar_isspace( c ) ) ) {
+			break;
+		}
+		g_string_append_unichar( value, c );
+		data = g_utf8_next_char( data );
+	}
+
+	if( eterm == ' ' &&
+	    value->str && ! strcmp( "PUBLIC", value->str ) ) {
+		/* read public id */
+		g_string_append_c( value, ' ' );
+		data = screem_utf8_skip_space( data );
+		term = g_utf8_get_char( data );
+		if( term != '"' && term != '\'' ) {
+			term = ' ';
+		} else {
+			g_string_append_unichar( value, term );
+			data = g_utf8_next_char( data );
+		}
+		while( *data != '\0' ) {
+			c = g_utf8_get_char( data );
+			if( c == term || ( term == ' ' && g_unichar_isspace( c ) ) ) {
+				break;
+			}
+			g_string_append_unichar( value, c );
+			data = g_utf8_next_char( data );
+		}
+		if( *data != '\0' ) {
+			g_string_append_unichar( value, term );
+			data = g_utf8_next_char( data );
+			data = screem_utf8_skip_space( data );
+		}
+		c = g_utf8_get_char( data );
+		if( c == '-' && *( data + 1 ) == '-' ) {
+			/* local copy of html 4.0 trans, at least
+			 * under debian comments out the system
+			 * id for the entities, so we should
+			 * handle comments here */
+			data = screem_dtd_bypass_comment( data, 
+							 NULL );	
+			c = g_utf8_get_char( data );
+		}
+		/* prevent attempt to read non existant SYSTEM uri */
+		if( c == '>' || c == '\0' ) {
+			eterm = '\0';
+		}
+	}
+	if( eterm == ' ' && value->str && 
+	    ( ! strncmp( "PUBLIC", value->str, strlen( "PUBLIC" ) ) ||
+		! strcmp( "SYSTEM", value->str ) ) ) {
+		/* read system id */
+		g_string_append_c( value, ' ' );
+		data = screem_utf8_skip_space( data );
+		term = g_utf8_get_char( data );
+		if( term != '"' && term != '\'' ) {
+			term = ' ';
+		} else {
+			g_string_append_unichar( value, term );
+			data = g_utf8_next_char( data );
+		}
+		while( *data != '\0' ) {
+			c = g_utf8_get_char( data );
+			if( c == term || ( term == ' ' && g_unichar_isspace( c ) ) ) {
+				break;
+			}
+			g_string_append_unichar( value, c );
+			data = g_utf8_next_char( data );
+		}
+		g_string_append_unichar( value, term );
+	}
+
+	desc = NULL;
+	while( *data != '\0' && *data != '>' ) {
+		c = g_utf8_get_char( data );
+		if( *data == '-' && *( data + 1 ) == '-' ) {
+			/* don't bother getting the comment
+			   text as we don't use it anywhere */
+			data = screem_dtd_bypass_comment( data, NULL );	
+		} else {
+			data = g_utf8_next_char( data );
+		}
+	}
+
+	entity = NULL;
+	if( name->str && value->str ) {		
+		/* FIXME: have a notation hash table */
+	} 
+	if( ! entity ) {
+		g_string_free( name, TRUE );
+		g_string_free( value, TRUE);
+		g_free( desc );
+	}
+
+	return data;
+}
+
 static const gchar *screem_dtd_name_parse( ScreemDTD *dtd,
 					const gchar *pattern, 
 					ScreemDTDGroup **group )
@@ -1477,8 +1671,8 @@ static const gchar *screem_dtd_name_parse2( ScreemDTD *dtd,
 			   if string->len > 0 */
 			if( string->len > 0 ) {
 				group->name = g_strdup( string->str );	
+				g_string_truncate( string, 0 );
 			}
-			g_string_assign( string, "" );
 		}
 
 		if( c == '*' ) {
@@ -1543,8 +1737,8 @@ static const gchar *screem_dtd_name_parse2( ScreemDTD *dtd,
 			if( string->len > 0 ) {
 				group->name = g_strdup( string->str );	
 				group = screem_dtd_group_new_sibling( group, SCREEM_DTD_OR );
+				g_string_truncate( string, 0 );
 			}
-			g_string_assign( string, "" );
 		}
 	}
 	if( string->len > 0 ) {
@@ -1685,13 +1879,22 @@ static gboolean screem_dtd_free_allowed_table( gpointer key,
 	allowed = (ScreemDTDAllowed*)value;
 	allowed->ref --;
 	if( allowed->ref < 1 ) {
-		g_free( allowed->name );
+	/* string chunk used do not free this
+	 * g_free( allowed->name );*/
 		g_free( allowed );
 	}
 
 	return FALSE;
 }
 
+static void screem_dtd_free_entity( ScreemDTDEntity *entity )
+{
+	g_free( entity->name );
+	g_free( entity->value );
+	g_free( entity->description );
+	g_free( entity );
+}
+
 static gboolean screem_dtd_free_entity_table( gpointer key,
 					gpointer value,
 					gpointer user_data )
@@ -1699,10 +1902,7 @@ static gboolean screem_dtd_free_entity_table( gpointer key,
 	ScreemDTDEntity *entity;
 
 	entity = (ScreemDTDEntity*)value;
-	g_free( entity->name );
-	g_free( entity->value );
-	g_free( entity->description );
-	g_free( entity );
+	screem_dtd_free_entity( entity );
 
 	return FALSE;
 }
@@ -1714,7 +1914,8 @@ static gboolean screem_dtd_free_element_table( gpointer key,
 	ScreemDTDElement *element;
 
 	element = (ScreemDTDElement*)value;
-	g_free( element->name );
+	/* string chunk used do not free this
+	 * g_free( element->name ); */
 	g_free( element->description );
 
 	if( element->allowedlist ) {
@@ -1755,7 +1956,8 @@ static gboolean screem_dtd_free_attribute_table( gpointer key,
 static void screem_dtd_free_attribute( ScreemDTDAttribute *attr )
 {
 	if( -- attr->ref < 1 ) {
-		g_free( attr->name );
+		/* do not free, string chunk in use
+		 * g_free( attr->name ); */
 		g_free( attr->default_value );
 		g_free( attr->description );
 		if( attr->values ) {
@@ -1783,7 +1985,7 @@ static gint screem_dtd_sort_element_list( gconstpointer a,
 }
 
 
-static gint screem_dtd_sort_attr_list( gconstpointer a,
+static gint screem_dtd_reverse_sort_attr_list( gconstpointer a,
 					gconstpointer b,
 					gpointer data )
 {
@@ -1798,11 +2000,11 @@ static gint screem_dtd_sort_attr_list( gconstpointer a,
 	dtd = SCREEM_DTD( data );
 
 	if( aattr->name && ! battr->name ) {
-		ret = 1;
-	} else if( battr->name && ! aattr->name ) {
 		ret = -1;
+	} else if( battr->name && ! aattr->name ) {
+		ret = 1;
 	} else {
-		ret = dtd->priv->compare( aattr->name, battr->name );
+		ret = dtd->priv->compare( battr->name, aattr->name );
 	}
 
 	return ret;
@@ -1862,6 +2064,24 @@ static void screem_dtd_find_root( ScreemDTD *dtd )
 	}
 }
 
+static gboolean screem_dtd_remove_param_entities( gpointer key,
+		gpointer value, gpointer data )
+{
+	ScreemDTDEntity *ent;
+	gboolean ret;
+	
+	ent = (ScreemDTDEntity*)value;
+
+	ret = FALSE;
+	if( ! ent->general ) {
+		ret = TRUE;
+
+		screem_dtd_free_entity( ent );
+	}
+
+	return ret;
+}
+
 /* public stuff */
 
 ScreemDTD *screem_dtd_new( const gchar *publicid, 
@@ -1898,7 +2118,7 @@ const gchar *screem_dtd_get_root_name( const ScreemDTD *dtd )
 	return dtd->priv->root;
 }
 
-void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
+gboolean screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 			const gchar *data )
 {
 	ScreemDTDPrivate *priv;
@@ -1913,9 +2133,11 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 
 	gchar *rooturi;
 	
-	g_return_if_fail( SCREEM_IS_DTD( dtd ) );
-	g_return_if_fail( parse != NULL );
-	g_return_if_fail( data != NULL );
+	gboolean success;
+
+	g_return_val_if_fail( SCREEM_IS_DTD( dtd ), FALSE );
+	g_return_val_if_fail( parse != NULL, FALSE );
+	g_return_val_if_fail( data != NULL, FALSE );
 
 	priv = dtd->priv;
 	db = SCREEM_DTD_DB( parse->userdata );
@@ -1928,7 +2150,9 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 		}
 	}
 	
-	while( *data != '\0' ) {
+	success = TRUE;
+	
+	while( success && *data != '\0' ) {
 
 		while( *data != '\0' && *data != '<' && *data != '%' ) {
 			data = g_utf8_next_char( data );
@@ -1972,8 +2196,9 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 				g_free( systemid );
 				g_free( rooturi );
 				if( data2 ) {
-					screem_dtd_parse( dtd, parse,
-							  data2 );
+					success = screem_dtd_parse( dtd,
+							parse,
+							data2 );
 					g_free( data2 );
 
 				}
@@ -1991,7 +2216,7 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 			if( ! strncmp( "--", data, strlen( "--" ) ) ) {
 				/* comment */
 				data = screem_dtd_bypass_comment( data,
-								 NULL );
+						NULL );
 			} else if( ! strncmp( "ELEMENT ", data,
 						strlen( "ELEMENT " ) ) ) {
 				/* element declaration */
@@ -2004,6 +2229,9 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 						strlen( "ATTLIST " ) ) ) {
 				/* attlist declaration */
 				data = screem_dtd_parse_attlist( dtd, data );
+			} else if( ! strncmp( "NOTATION ", data,
+						strlen( "NOTATION " ) ) ) {
+				data = screem_dtd_parse_notation( dtd, data );
 			} else if( *data == '[' ) {
 				/* feature switch */
 				data = g_utf8_next_char( data );	
@@ -2025,6 +2253,10 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 						data = tmp + strlen( "]]>" );
 					}
 				}
+			} else {
+				/* no idea what this tag is, bail out */
+				success = FALSE;
+				break;
 			}
 		} else if( *data != '\0' && *data != '>' ) {
 			data = g_utf8_next_char( data );
@@ -2039,7 +2271,15 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 		screem_dtd_find_root( dtd );
 		g_free( priv->rooturi );
 		priv->rooturi = NULL;
+
+		/* clean up non general entities, they are
+		 * no longer needed */
+		g_hash_table_foreach_remove( priv->entities,
+				(GHRFunc)screem_dtd_remove_param_entities,
+				NULL );
 	}
+
+	return success;
 }
 
 gchar *screem_dtd_dump( const ScreemDTD *dtd )
@@ -2101,6 +2341,7 @@ gchar *screem_dtd_lookup_entity( const ScreemDTD *dtd,
 			c = g_utf8_get_char( val );
 			if( c != '%' || entity->general ) {
 				exp = g_string_append_unichar( exp, c );
+				val = g_utf8_next_char( val );
 			} else { 
 				tmp = NULL;
 				val = screem_dtd_read_entity( val, &tmp );		
@@ -2110,9 +2351,9 @@ gchar *screem_dtd_lookup_entity( const ScreemDTD *dtd,
 					g_free( ret );
 					g_free( tmp );
 				}
-			}
-			if( val != '\0' ) {
-				val = g_utf8_next_char( val );
+				if( val != '\0' ) {
+					val = g_utf8_next_char( val );
+				}
 			}
 		}
 		ret = exp->str;
@@ -2341,14 +2582,14 @@ const gchar *screem_dtd_attribute_get_description( const ScreemDTDAttribute *att
 	return attr->description;
 }
 
-const gboolean screem_dtd_attribute_get_required( const ScreemDTDAttribute *attr )
+gboolean screem_dtd_attribute_get_required( const ScreemDTDAttribute *attr )
 {
 	g_return_val_if_fail( attr != NULL, FALSE );
 
 	return attr->required == SCREEM_DTD_REQUIRED;
 }
 
-const gboolean screem_dtd_attribute_is_uri( const ScreemDTDAttribute *attr )
+gboolean screem_dtd_attribute_is_uri( const ScreemDTDAttribute *attr )
 {
 	g_return_val_if_fail( attr != NULL, FALSE );
 
