@@ -57,12 +57,8 @@ enum {
 
 static guint screem_file_browser_signals[ LAST_SIGNAL ] = { 0 };
 
-/* lock access to the icon cache */
-static GMutex *cache_mutex = NULL;
-
-
 /* cache mime type strings */
-static GHashTable *mime_cache = NULL;
+static GStringChunk *mime_cache = NULL;
 
 typedef enum {
 	BROWSER_NODE_DIRECTORY,
@@ -231,8 +227,8 @@ void screem_file_browser_scan_iter( ScreemFileBrowser *browser,
 	gchar *uri;
 	GtkTreeIter filtered_it;
 	GtkTreeIter realit;
-	GtkTreeIter *parent = NULL;
-	GtkTreeIter pit;
+		
+	GtkTreeIter cit;
 
 	g_return_if_fail( SCREEM_IS_FILE_BROWSER( browser ) );
 	g_return_if_fail( iter != NULL );
@@ -259,28 +255,50 @@ void screem_file_browser_scan_iter( ScreemFileBrowser *browser,
 			mode = details->mode;
 		}
 
-	
-		if( gtk_tree_model_iter_has_child( model, iter ) ) {
-			GtkTreeIter cit;
-			while( gtk_tree_model_iter_children( model, &cit, iter ) ) {
-				BrowserNode *node;
 
+		/* remove BrowserNode objects from current children,
+		 * this marks them for removeal */
+		if( gtk_tree_model_iter_children( model, &cit, iter ) ) {
+			do {
 				gtk_tree_model_get( model, &cit,
 						FILE_BROWSER_NODE_COL, 
 						&node, -1 );
 				if( node ) {
 					browser_node_free( node );
+					gtk_tree_store_set( GTK_TREE_STORE( model ), &cit,
+							FILE_BROWSER_NODE_COL, NULL, -1 );
 				}
+			} while( gtk_tree_model_iter_next( model, &cit ) );
 			
-				gtk_tree_store_remove( GTK_TREE_STORE( model ),
-						&cit );
-			}
 		}
 	
 		scan_directory( browser, iter, 
 				uri,  "x-directory/normal", 0, 
 				mode );
-	
+
+		/* now new children are added, remove all children
+		 * that have no BrowserNode object */
+		if( gtk_tree_model_iter_children( model, &cit, iter ) ) {
+			gboolean stop = FALSE;
+			
+			do {
+				GtkTreeIter next;
+				
+				gtk_tree_model_get( model, &cit,
+						FILE_BROWSER_NODE_COL, 
+						&node, -1 );
+				next = cit;
+				stop = ! gtk_tree_model_iter_next( model, &next );
+				
+				if( ! node  ) {
+					gtk_tree_store_remove( GTK_TREE_STORE( model ),
+						&cit );
+				} 
+				cit = next;
+			} while( ! stop  );
+			
+		}
+
 		g_free( uri );
 	}
 }
@@ -430,11 +448,8 @@ void screem_file_browser_set_backup_suffix( ScreemFileBrowser *browser,
 
 void screem_file_browser_cleanup( void )
 {
-	if( cache_mutex ) {
-		g_mutex_free( cache_mutex );
-	}
 	if( mime_cache ) {
-		g_hash_table_destroy( mime_cache );
+		g_string_chunk_free( mime_cache );
 	}
 }
 
@@ -472,23 +487,10 @@ static BrowserNode *browser_node_new( ScreemFileBrowser *browser,
 		node->basename = g_strdup( G_DIR_SEPARATOR_S );
 	}
 	
-	g_mutex_lock( cache_mutex );
-
-	mime_type = g_hash_table_lookup( mime_cache, node->mime_type );
-	if( ! mime_type ) {
-		node->mime_type = g_strdup( node->mime_type );
-		/* safe to cast to gpointer, node->mime_type will
-		 * stay valid while mime_cache is around */
-		g_hash_table_insert( mime_cache, 
-				(gpointer)node->mime_type,
-				(gpointer)node->mime_type );
-		mime_type = node->mime_type;
-	} else if( mime_type != node->mime_type ) {
-		node->mime_type = mime_type;
-	}
+	mime_type = g_string_chunk_insert_const( mime_cache,
+			node->mime_type );
+	node->mime_type = mime_type;
 	
-	g_mutex_unlock( cache_mutex );
-
        	if( type == BROWSER_NODE_DIRECTORY && depth == 0 ) {
 		/* don't monitor /dev, /proc, or /sys */
 		gchar *monuri;
@@ -631,6 +633,7 @@ static void scan_directory( ScreemFileBrowser *browser,
 	result = GNOME_VFS_ERROR_NOT_A_DIRECTORY;
 	handle = NULL;
 	info = NULL;
+
 	if( mime_type && ! strcmp( "x-directory/normal", mime_type ) ) {
 		result = GNOME_VFS_OK;
 		if(  (mode & FILE_BROWSE_RECURSE) || ( depth == 0 ) ) {
@@ -1163,19 +1166,10 @@ screem_file_browser_init( ScreemFileBrowser *browser )
 {
 	ScreemFileBrowserDetails *details;
 	
-	if( ! cache_mutex ) {
-		cache_mutex = g_mutex_new();
-	}
-	g_mutex_lock( cache_mutex );
 	if( ! mime_cache ) {
-		mime_cache = g_hash_table_new_full( g_str_hash, 
-				g_str_equal, 
-				(GDestroyNotify)g_free, 
-				NULL );
+		mime_cache = g_string_chunk_new( 128 );
 	}
 
-	g_mutex_unlock( cache_mutex );
-
 	details = browser->details = g_new0( ScreemFileBrowserDetails, 1 );
 
 	details->show_hidden = FALSE;
