@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -41,34 +41,20 @@
 #include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <glib.h>
 
 #include "screem-window.h"
 #include "screem-helper.h"
 #include "screem-page.h"
 
+/* needed for special case */
+#include "screem-cvs.h"
+
 #include "fileops.h"
 #include "support.h"
 
-#include "eggdesktopentries.h"
-
 #include "screemmarshal.h"
 
-static void screem_helper_class_init( ScreemHelperClass *klass );
-static void screem_helper_init( ScreemHelper *helper );
-static void screem_helper_finalize( GObject *object );
-
-static void screem_helper_set_property(GObject *object, guint prop_id,
-				       const GValue *value, GParamSpec *pspec);
-static void screem_helper_get_property( GObject *object, guint prop_id,
-					GValue *value, GParamSpec *pspec );
-static void screem_helper_verb_cb( GtkAction *action, gpointer user_data );
-
-static void child_setup( gpointer data );
-static void child_watch( GPid pid, gint status, gpointer data );
-static gboolean child_stdout( GIOChannel *source,
-		GIOCondition condition, gpointer data );
-static gboolean child_stderr( GIOChannel *source,
-		GIOCondition condition, gpointer data );
 
 enum {
 	PROP_BOGUS,
@@ -78,7 +64,9 @@ enum {
 	PROP_INPUT_MODE,
 	PROP_OUTPUT_MODE,
 	PROP_EXEC_ON_SAVE,
-	PROP_EXEC_ON_SAVE_TYPE
+	PROP_EXEC_ON_SAVE_TYPE,
+	PROP_UIPATH,
+	PROP_FILE
 };
 
 enum {
@@ -88,22 +76,9 @@ enum {
 };
 static guint screem_helper_signals[LAST_SIGNAL] = { 0 };
 
-struct ScreemHelperPrivate {
-	/* a list of ScreemWindows */
-	GSList *list;
-
-	gchar *name;
-	gchar *pathname;
-	gchar *iconpath;
-
-	HelperInputMode imode;
-	HelperOutputMode omode;
-
-	gboolean exec;
-	gchar *exec_type;
+typedef struct {
+	ScreemHelper *helper;
 	
-	/* the following two variables are set in screem_helper_execute(), 
-	   only valid in that context + the source cb */
 	ScreemPage *page;
 
 	gint std_out;
@@ -115,12 +90,56 @@ struct ScreemHelperPrivate {
 	GIOChannel *err;
 	gint owatch;
 	gint ewatch;
+	gint pulse;
 
 	GString *input;
 	ScreemWindow *window;
 	GtkWidget *dialog;
+} ScreemHelperData;
+
+struct ScreemHelperPrivate {
+	/* a list of ScreemWindows */
+	GSList *list;
+
+	/* set to FALSE if the helper .desktop file contains
+	   a TryExec field and the executable doesn't exist */
+	gboolean exists;
+
+	gchar *name;
+	gchar *pathname;
+	gchar *iconpath;
+
+	HelperInputMode imode;
+	HelperOutputMode omode;
+
+	gboolean exec;
+	gchar *exec_type;
+
+	gchar *uipath;
+		
+	gboolean nonblocking;
+	
+	/* file that the helper was loaded from */
+	gchar *filename;
+
+	/* a list of ScreemHelperData for running instances
+	 * of the helper */
+	GSList *running;
 };
 
+static void screem_helper_verb_cb( GtkAction *action, gpointer user_data );
+
+static void child_setup( gpointer data );
+static void child_watch( GPid pid, gint status, gpointer data );
+static gboolean child_stdout( GIOChannel *source,
+		GIOCondition condition, gpointer data );
+static gboolean child_stderr( GIOChannel *source,
+		GIOCondition condition, gpointer data );
+static gboolean screem_helper_pulse( ScreemHelperData *hdata );
+static gboolean exec_exists( const char *exec );
+static gchar *screem_helper_build_action_name( const gchar *name );
+
+
 
 ScreemHelper* screem_helper_new( const gchar *name, 
 				 const gchar *pathname,
@@ -148,9 +167,9 @@ ScreemHelper *screem_helper_new_from_file( const gchar *file )
 {
 	ScreemHelper *helper;
 
-	EggDesktopEntries *entries;
+	GKeyFile *entries;
 	GError *error;
-
+	
 	gchar *name;
 	gchar *pathname;
 	gchar *icon;
@@ -158,65 +177,158 @@ ScreemHelper *screem_helper_new_from_file( const gchar *file )
 	HelperOutputMode omode;
 	gboolean exec;
 	gchar *type;
+	gchar *uipath;
+
+	gchar *tryexec;
+	gboolean exists;
+	gchar *tmp;
+	gint argc;
+	gchar **argv;
 
 	helper = NULL;
 	error = NULL;
-	entries = eggy_desktop_entries_new_from_file( file,
-			NULL,
-			EGG_DESKTOP_ENTRIES_DISCARD_COMMENTS |
-			EGG_DESKTOP_ENTRIES_DISCARD_TRANSLATIONS,
-			&error );
-
-	if( entries ) {
-		name = eggy_desktop_entries_get_string( entries,
+	entries = g_key_file_new();
+	if( g_key_file_load_from_file( entries, file,
+				G_KEY_FILE_NONE, &error ) ) {
+		tryexec = g_key_file_get_string( entries,
+				"Desktop Entry",
+				"TryExec",
+				&error );
+		if( error ) {
+			g_error_free( error );
+		}
+	
+		error = NULL;
+		name = g_key_file_get_string( entries,
 				"Desktop Entry",
 				"Name",
 				&error );
+		if( error ) {
+			g_error_free( error );
+		}
+
 		error = NULL;
-		pathname = eggy_desktop_entries_get_string( entries,
+		pathname = g_key_file_get_string( entries,
 				"Desktop Entry",
 				"Exec",
 				&error );
+		if( error ) {
+			g_error_free( error );
+		}
+		
 		error = NULL;
-		icon = eggy_desktop_entries_get_string( entries,
+		icon = g_key_file_get_string( entries,
 				"Desktop Entry",
 				"Icon",
 				&error );
+		if( error ) {
+			g_error_free( error );
+		}
+
 		error = NULL;
-		imode = eggy_desktop_entries_get_integer( entries,
+		imode = g_key_file_get_integer( entries,
 				"Desktop Entry",
 				"X-SCREEM-imode",
 				&error );
+		if( error ) {
+			g_error_free( error );
+		}
+
 		error = NULL;
-		omode = eggy_desktop_entries_get_integer( entries,
+		omode = g_key_file_get_integer( entries,
 				"Desktop Entry",
 				"X-SCREEM-omode",
 				&error );
+		if( error ) {
+			g_error_free( error );
+		}
+		
 		error = NULL;
-		exec = eggy_desktop_entries_get_boolean( entries,
+		exec = g_key_file_get_boolean( entries,
 				"Desktop Entry",
 				"X-SCREEM-exec",
 				&error );
+		if( error ) {
+			g_error_free( error );
+		}
+
 		error = NULL;
-		type = eggy_desktop_entries_get_string( entries,
+		type = g_key_file_get_string( entries,
 				"Desktop Entry",
 				"X-SCREEM-exectype",
 				&error );
-		
-		eggy_desktop_entries_free( entries );
+		if( error ) {
+			g_error_free( error );
+		}
 
+		error = NULL;
+		uipath = g_key_file_get_string( entries,
+				"Desktop Entry",
+				"X-SCREEM-uipath",
+				&error );
+		/* ignore missing X-SCREEM-uipath */
+		if( error ) {
+			g_error_free( error );
+			error = NULL;
+		}
+		
 		helper = screem_helper_new( name, pathname,
 				icon, imode, omode );
 
 		g_object_set( G_OBJECT( helper ),
 				"exec_on_save", exec,
 				"exec_type", type,
+				"uipath", uipath,
 				NULL );
+
+		if( tryexec ) {
+			exists = exec_exists( tryexec );
+			g_free( tryexec );
+		} else {
+			/* problem: this prevents helpers that use a relative path from
+			   being active */
+			if( ! g_shell_parse_argv( pathname, &argc, &argv, NULL ) ) {
+				exists = FALSE;	
+			} else if( argc < 1 ) {
+				g_strfreev( argv );
+				exists = FALSE;
+			} else {
+				exists = exec_exists( argv[ 0 ] );
+				g_strfreev( argv );
+			}
+		}
+		helper->priv->exists = exists;
+
+		error = NULL;
+		tmp = g_key_file_get_string( entries,
+				"Desktop Entry",
+				"X-SCREEM-nonblocking",
+				&error );
+		/* ignore missing X-SCREEM-nonblocking */
+		if( error ) {
+			g_error_free( error );
+			error = NULL;
+		}
+		helper->priv->nonblocking = ( tmp && *tmp == '1' );
+		g_free( tmp );
+		
 		g_free( name );
 		g_free( pathname );
 		g_free( icon );
-		g_free( type );
+		g_free( type );	
+		g_free( uipath );
+		
+	}	
+	
+	if( error ) {
+		g_printerr( "Failed to load helper %s : %s\n",
+				file, error->message );
+		g_error_free( error );
+	} else {
+		helper->priv->filename = g_strdup( file );
 	}
+	
+	g_key_file_free( entries );
 
 	return helper;
 }
@@ -278,7 +390,7 @@ gboolean screem_helper_save( ScreemHelper *helper )
 
 		ret = save_file( tmp, output->str,
 				GNOME_VFS_PERM_USER_ALL, 
-				FALSE, NULL );
+				FALSE, TRUE, NULL );
 		g_string_free( output, TRUE );
 		g_free( name );
 		g_free( path );
@@ -296,6 +408,7 @@ void screem_helper_execute( ScreemHelper *helper,
 		ScreemPage *page, ScreemWindow *window )
 {
 	ScreemHelperPrivate *priv;
+	const gchar *pathname;
 	const gchar *pagepath;
 	gchar *dirname;
 
@@ -310,7 +423,6 @@ void screem_helper_execute( ScreemHelper *helper,
 	const gchar *charset;
 	
 	GladeXML *xml;
-	gchar *gladepath;
 	GtkWidget *widget;
 	
 	gchar *label;
@@ -328,28 +440,53 @@ void screem_helper_execute( ScreemHelper *helper,
 
 	ScreemPage *wpage;
 	
-	priv = helper->private;
+	ScreemSite *site;
+	const gchar *sitepath;
+	const gchar *remote;
 
-	screem_window_clear_messages( window );
-	screem_window_clear_errors( window );
+	GSpawnFlags spawnflags;
+
+	ScreemHelperData *newdata;
+	
+	priv = helper->priv;
 
 	wpage = NULL;
+	site = NULL;
 	if( window ) {
+		screem_window_clear_messages( window );
+		screem_window_clear_errors( window );
 		wpage = screem_window_get_document( window );
+		site = screem_window_get_current( window );
 	}
 	
 	if( wpage && ! page ) {
-		 page = wpage;
+		page = wpage;
 	}
 	
-	if( ! page ) {
+	if( ( ! page ) &&  
+	    ( priv->imode != SCREEM_HELPER_STDIN_NONE &&
+	      priv->omode != SCREEM_HELPER_STDOUT_NONE ) ) {
 		return;
 	}
 
-	priv->page = page;
+	sitepath = NULL;
+	if( site ) {
+		sitepath = screem_site_get_pathname( site );
+		/* strip file: from local methods */
+		if( sitepath ) {
+			if( ! strncmp( "file://", sitepath,
+				strlen( "file://" ) ) ) {
+				sitepath += strlen( "file://" );	
+			}
+		}
+	}
 	
 	/* get pathname, and strip file: from local methods */
-	pagepath = screem_page_get_pathname( page );
+	if( page ) {
+		pathname = pagepath = screem_page_get_pathname( page );
+	} else {
+		pathname = pagepath = NULL;
+	}
 	dirname = NULL;
 	if( pagepath ) {
 		/* need local path for working directory, for
@@ -361,9 +498,17 @@ void screem_helper_execute( ScreemHelper *helper,
 			dirname = g_path_get_dirname( pagepath );
 		}
 	}
+
+	/* special case CVS helper */
+	if( SCREEM_IS_CVS( helper ) ) {
+		g_free( dirname );
+		dirname = NULL;	
+	}
+	
+	remote = screem_site_get_http_url( site );
 	
 	/* replace pathnames in the pattern to exec */
-	path = g_string_new( "" );
+	path = g_string_new( NULL );
 	exec = priv->pathname;
 	do {
 		tmp = strchr( exec, '%' );
@@ -374,12 +519,36 @@ void screem_helper_execute( ScreemHelper *helper,
 			tmp = g_utf8_next_char( tmp );
 			c = g_utf8_get_char( tmp );
 			switch( c ) {
+				case 'S':
+					if( sitepath ) {
+						g_string_append_printf( path, "\"%s\"", 
+							sitepath );
+					}
+					break;
+				case 'U':
+					if( pathname ) {
+						g_string_append_printf( path, "\"%s\"",
+							pathname );
+					}
+					break;
 				case 'f':
 					if( pagepath ) {
-						g_string_append( path,
+						g_string_append_printf( path, "\"%s\"",
 								pagepath );
 					}
 					break;
+				case 'd':
+					if( dirname ) {
+						g_string_append_printf( path, "\"%s\"",
+								dirname );
+					}
+					break;
+				case 'p':
+					if( remote && pagepath && sitepath ) {
+						g_string_append_printf( path, "\"%s%s\"",
+							remote, pagepath + strlen( sitepath ) );
+					}
+					break;
 				case '%':
 					g_string_append_unichar( path,
 							c );
@@ -402,23 +571,34 @@ void screem_helper_execute( ScreemHelper *helper,
 	if( ! g_shell_parse_argv( path->str, &argc, &argv, &error ) ) {
 		g_string_free( path, TRUE );
 		if( error ) {
+			if( window ) {
+				screem_window_show_error( window, 
+						error->message, TRUE );
+			}
 			g_error_free( error );
+		} else if( window ) {
+			screem_window_show_error( window,
+					path->str, TRUE );
 		}
 		return;
 	}
 	g_string_free( path, TRUE );
 	
-	priv->window = window;
-
+	newdata = g_new0( ScreemHelperData, 1 );
+	newdata->page = page;
+	newdata->window = window;
+	
+	spawnflags = G_SPAWN_SEARCH_PATH;
+	spawnflags |= G_SPAWN_DO_NOT_REAP_CHILD;
+	
 	if( ! g_spawn_async_with_pipes( dirname,
-			argv, NULL,
-			G_SPAWN_DO_NOT_REAP_CHILD,
+			argv, NULL, spawnflags,
 			child_setup,
 			helper,
 			&pid,
 			&std_in,
-			&priv->std_out,
-			&priv->std_err,
+			&newdata->std_out,
+			&newdata->std_err,
 			&error ) ) {
 		/* error starting helper */
 		g_signal_emit( G_OBJECT( helper ),
@@ -426,28 +606,46 @@ void screem_helper_execute( ScreemHelper *helper,
 			       0, error->message );
 		g_strfreev( argv );
 		if( error ) {
+			if( window ) {
+				screem_window_show_error( newdata->window, error->message,
+					TRUE );
+			}
 			g_error_free( error );
 		}
+
+		/* free newdata up, it hasn't gone into the list
+		 * yet */
+		g_free( newdata );
+		
 		return;
 	}
 	g_strfreev( argv );
-	priv->pid = pid;
 	
-	flags = fcntl( priv->std_out, F_GETFL, 0 );
+	newdata->pid = pid;
+	
+	flags = fcntl( newdata->std_out, F_GETFL, 0 );
 	if( flags != -1 ) {
-		fcntl( priv->std_out, F_SETFL, flags | O_NONBLOCK );
+		fcntl( newdata->std_out, F_SETFL, flags | O_NONBLOCK );
 	} else {
+		/* free newdata up, it hasn't gone into the list
+		 * yet */
+		g_free( newdata );
+		
+		/* FIXME: close pipes */
 		return;
 	}
-	flags = fcntl( priv->std_err, F_GETFL, 0 );
+	flags = fcntl( newdata->std_err, F_GETFL, 0 );
 	if( flags != -1 ) {
-		fcntl( priv->std_err, F_SETFL, flags | O_NONBLOCK );
+		fcntl( newdata->std_err, F_SETFL, flags | O_NONBLOCK );
 	} else {
+		/* free newdata up, it hasn't gone into the list
+		 * yet */
+		g_free( newdata );
+
+		/* FIXME: close pipes */
 		return;
 	}
 	
-	gladepath = screem_get_glade_path();
-	
 	switch( priv->imode ) {
 		case SCREEM_HELPER_STDIN_SELECTION:
 			out = fdopen( std_in, "w" );
@@ -464,7 +662,7 @@ void screem_helper_execute( ScreemHelper *helper,
 			break;
 		case SCREEM_HELPER_STDIN_USER:
 			out = fdopen( std_in, "w" );
-			xml = glade_xml_new( gladepath, 
+			xml = glade_xml_new( GLADE_PATH"/screem.glade", 
 					"helper_user_input", NULL );
 			widget = glade_xml_get_widget( xml, 
 					"helper_name" );
@@ -512,112 +710,172 @@ void screem_helper_execute( ScreemHelper *helper,
 	}
 	close( std_in );
 
-	priv->input = g_string_new( "" );
+	newdata->input = g_string_new( NULL );
 	
 	/* we want to listen for child exits */
-	priv->watch = g_child_watch_add( pid, child_watch, helper );
+	newdata->watch = g_child_watch_add( pid, child_watch, newdata );
 
 	/* listen to priv->std_out and priv->std_err */
-	priv->out = g_io_channel_unix_new( priv->std_out );
-	priv->err = g_io_channel_unix_new( priv->std_err );
+	newdata->out = g_io_channel_unix_new( newdata->std_out );
+	newdata->err = g_io_channel_unix_new( newdata->std_err );
 
 	if( ! g_get_charset( &charset ) ) {
-		g_io_channel_set_encoding( priv->out, charset, &error );
+		g_io_channel_set_encoding( newdata->out, charset, &error );
 		if( error ) {
 			g_error_free( error );
 			error = NULL;
 		}
-		g_io_channel_set_encoding( priv->err, charset, &error );
+		g_io_channel_set_encoding( newdata->err, charset, &error );
 		if( error ) {
 			g_error_free( error );
 			error = NULL;
 		}
 	}
 
-	priv->owatch = g_io_add_watch( priv->out, 
+	newdata->helper = helper;
+	newdata->owatch = g_io_add_watch( newdata->out, 
 			G_IO_IN | G_IO_ERR | G_IO_HUP, 
-			child_stdout, helper );
-	priv->ewatch = g_io_add_watch( priv->err, 
+			child_stdout, newdata );
+	newdata->ewatch = g_io_add_watch( newdata->err, 
 			G_IO_IN | G_IO_ERR | G_IO_HUP, 
-			child_stderr, helper );
-
-	/* display helper running dialog */
-	xml = glade_xml_new( gladepath, "helper_run", NULL );
-
-	widget = glade_xml_get_widget( xml, "helperlabel" );
-	tmp = g_strdup_printf( "Running %s", priv->name );
-	label = g_strconcat( "<span weight=\"bold\" size=\"large\">",
-			tmp, "</span>", NULL );
-	g_free( tmp );
-	gtk_label_set_markup( GTK_LABEL( widget ), label );
-	g_free( label );
+			child_stderr, newdata );
 	
-	widget = glade_xml_get_widget( xml, "helper_run" );
-
-	priv->dialog = widget;
-
-	do {
-		button = gtk_dialog_run( GTK_DIALOG( widget ) );
-		switch( button ) {
-			case GTK_RESPONSE_CANCEL:
-				kill( pid, SIGKILL );
-				break;
-			default:
-				break;
-		}
-	} while( button != GTK_RESPONSE_CLOSE );
-
-	gtk_widget_destroy( widget );
-	priv->dialog = NULL;
+	priv->running = g_slist_prepend( priv->running, newdata );
 	
-	g_free( gladepath );
+	if( ! priv->nonblocking ) {
+		/* display helper running dialog */
+		xml = glade_xml_new( GLADE_PATH"/screem.glade", "helper_run", NULL );
+
+		widget = glade_xml_get_widget( xml, "helperlabel" );
+		tmp = g_strdup_printf( "Running %s", priv->name );
+		label = g_strconcat( "<span weight=\"bold\" size=\"large\">",
+				tmp, "</span>", NULL );
+		g_free( tmp );
+		gtk_label_set_markup( GTK_LABEL( widget ), label );
+		g_free( label );
+	
+		widget = glade_xml_get_widget( xml, "helperprogress" );
+		newdata->pulse = g_timeout_add( 250, (GSourceFunc)screem_helper_pulse,
+			newdata );
+	
+		widget = glade_xml_get_widget( xml, "helper_run" );
+
+		newdata->dialog = widget;
+
+		do {
+			button = gtk_dialog_run( GTK_DIALOG( widget ) );
+			switch( button ) {
+				case GTK_RESPONSE_CANCEL:
+					kill( pid, SIGKILL );
+					break;
+				default:
+					break;
+			}
+		} while( button != GTK_RESPONSE_CLOSE );
+	
+		gtk_widget_destroy( widget );
+	}
 	g_free( dirname );
 }
 
 void screem_helper_add_to_window( ScreemHelper *helper, ScreemWindow *window )
 {
+	ScreemHelperPrivate *priv;
 	gchar *name;
 	gchar *pathname;
 	gchar *iconpath;
 
 	gchar *uiname;
-	gchar *ui;
 	GtkAction *action;
 	guint merge_id;
 	gchar *tmp;
 
-	helper->private->list = 
-		g_slist_append( helper->private->list, window );
+	GString *ui;
+	const gchar *uipath;
+	gchar **paths;
+	gchar **tpaths;
+	GtkActionGroup *group;
+	gchar *accel_path;
+
+	GtkAction *subaction;
+	
+	g_return_if_fail( window != NULL );
+	
+	priv = helper->priv;
+	
+	priv->list = g_slist_append( priv->list, window );
 	
 	g_object_get( G_OBJECT( helper ),
 		      "name", &name, "pathname", &pathname, 
 		      "iconpath", &iconpath, NULL );
 
-	tmp = screem_escape_char( name, ' ' );
-	uiname = g_strconcat( "Exec_Helper_", tmp, NULL );
-	g_free( tmp );
-
+	uiname = screem_helper_build_action_name( name );
+	
 	tmp = screem_escape_underlines( name );
 	action = gtk_action_new( uiname, tmp, pathname, 
 			GTK_STOCK_EXECUTE );
+	gtk_action_set_sensitive( action, priv->exists );
+	
 	g_free( tmp );
 	g_signal_connect( G_OBJECT( action ), "activate",
 			G_CALLBACK( screem_helper_verb_cb ), helper );
-	gtk_action_group_add_action( GTK_ACTION_GROUP( window->action_group ),
-			action );
-	ui = g_strconcat( "<ui><menubar><menu action=\"Tools\"><menu action=\"Helpers\">",
-			"<menuitem action=\"", uiname, "\" />",
-			"</menu></menu></menubar></ui>",
-			NULL );
+
+	accel_path = g_strconcat( "<Actions>", "/ScreemHelperActions/",
+			name, NULL );
+	gtk_action_set_accel_path( action, accel_path );
+	g_free( accel_path );
+	
+	group = GTK_ACTION_GROUP( window->action_group );
+	gtk_action_group_add_action( group, action );
+
+
+	/* build ui path */
+	uipath = priv->uipath;
+	if( ( ! uipath ) || *uipath == '\0'  ) {
+		uipath = "Tools/Helpers";
+	}
+	ui = g_string_new( "<ui><menubar>" );
+	paths = g_strsplit( uipath, "/", -1 );
+	for( tpaths = paths; tpaths && *tpaths; tpaths ++ ) {
+		if( **tpaths ) {
+			g_string_append_printf( ui,
+					"<menu action=\"%s\">",
+					*tpaths );
+
+			/* do we have an action *tpaths ? if
+			 * not create one */
+			subaction = gtk_action_group_get_action( group,
+					*tpaths );
+			if( ! subaction ) {
+				tmp = screem_escape_underlines( *tpaths );
+				subaction = gtk_action_new( *tpaths,
+						tmp, "", "" );
+				gtk_action_group_add_action( group, 
+						subaction );
+			}
+		}
+	}
+	g_string_append_printf( ui, "<menuitem action=\"%s\" />",
+			uiname );
+	for( tpaths = paths; tpaths && *tpaths; tpaths ++ ) {
+		if( **tpaths ) {
+			g_string_append( ui, "</menu>" );
+		}
+	}
+	g_strfreev( paths );
+	g_string_append( ui, "</menubar></ui>" );
+	
 	merge_id = gtk_ui_manager_add_ui_from_string( GTK_UI_MANAGER( window->merge ),
-							ui, strlen( ui ), NULL );
+							ui->str, 
+							ui->len,
+							NULL );
 	g_object_set_data( G_OBJECT( action ), "merge_id",
 				GUINT_TO_POINTER( merge_id ) );
 
 	g_object_set_data( G_OBJECT( action ), "window", window );
 
 	g_free( uiname );
-	g_free( ui );
+	g_string_free( ui, TRUE );
 
 	g_free( pathname );
 	g_free( name );
@@ -632,18 +890,15 @@ void screem_helper_remove_from_window( ScreemHelper *helper,
 	gchar *uiname;
 	GtkAction *action;
 	guint merge_id;
-	gchar *tmp;
 	
 	g_return_if_fail( SCREEM_IS_HELPER( helper ) );
 	
-	helper->private->list = g_slist_remove( helper->private->list, 
+	helper->priv->list = g_slist_remove( helper->priv->list, 
 						window );
 
 	g_object_get( G_OBJECT( helper ), "name", &name, NULL );
 
-	tmp = screem_escape_char( name, ' ' );
-	uiname = g_strconcat( "Exec_Helper_", tmp, NULL );
-	g_free( tmp );
+	uiname = screem_helper_build_action_name( name );
 
 	g_free( name );
 	
@@ -662,14 +917,26 @@ void screem_helper_remove_from_window( ScreemHelper *helper,
 
 }
 
+gchar *screem_helper_get_uiname( ScreemHelper *helper )
+{
+	gchar *ret;
+	
+	g_return_val_if_fail( SCREEM_IS_HELPER( helper ), NULL );
+
+	ret = screem_helper_build_action_name( helper->priv->name );
+
+	return ret;
+}
+
 /* static stuff */
 static void screem_helper_verb_cb( GtkAction *action, gpointer user_data )
 {
 	ScreemHelper *helper;
 	ScreemWindow *window;
-	
+
 	helper = SCREEM_HELPER( user_data );
-	window = SCREEM_WINDOW( g_object_get_data( G_OBJECT( action ), "window" ) );
+	window = g_object_get_data( G_OBJECT( action ), "window" );
+
 	screem_helper_execute( helper, NULL, window );
 }
 
@@ -680,37 +947,48 @@ static void child_setup( gpointer data )
 }
 static void child_watch( GPid pid, gint status, gpointer data )
 {
+	ScreemHelperData *hdata;
 	ScreemHelper *helper;
 	ScreemHelperPrivate *priv;
 	ScreemPage *wpage;
 	gchar *tmp;
+	GSList *temp;
+	ScreemView *view;
+	
+	hdata = (ScreemHelperData*)data;
+	
+	helper = hdata->helper;
+	priv = helper->priv;
 
-	helper = SCREEM_HELPER( data );
-	priv = helper->private;
-
-	if( pid != priv->pid ) {
+	if( pid != hdata->pid ) {
 		return;
 	}
 		
-	wpage = screem_window_get_document( priv->window );
+	wpage = screem_window_get_document( hdata->window );
+	view = screem_window_get_view( hdata->window );
 	
 	g_spawn_close_pid( pid );
 
-	g_source_remove( priv->watch );
+	g_source_remove( hdata->watch );
 	
-	g_source_remove( priv->owatch );
-	g_source_remove( priv->ewatch );
+	g_source_remove( hdata->owatch );
+	g_source_remove( hdata->ewatch );
 
-	/* read any remaining data */
-	child_stdout( priv->out, G_IO_IN, data );
-	child_stderr( priv->err, G_IO_IN, data );
-		
-	g_io_channel_shutdown( priv->out, FALSE, NULL );
-	g_io_channel_shutdown( priv->err, FALSE, NULL );
-	g_io_channel_unref( priv->out );
-	g_io_channel_unref( priv->err );
+	if( hdata->pulse ) {
+		g_source_remove( hdata->pulse );
+		hdata->pulse = 0;
+	}
 	
-	priv->out = priv->err = NULL;
+	/* read any remaining data */
+	child_stdout( hdata->out, G_IO_IN, data );
+	child_stderr( hdata->err, G_IO_IN, data );
+		
+	g_io_channel_shutdown( hdata->out, FALSE, NULL );
+	g_io_channel_shutdown( hdata->err, FALSE, NULL );
+	g_io_channel_unref( hdata->out );
+	g_io_channel_unref( hdata->err );
+	
+	hdata->out = hdata->err = NULL;
 	
 	gdk_threads_enter();
 	
@@ -719,15 +997,16 @@ static void child_watch( GPid pid, gint status, gpointer data )
 			/* success */
 			switch( priv->omode ) {
 				case SCREEM_HELPER_STDOUT_SELECTION:
-					if( wpage == priv->page ) {
-						screem_window_replace_selection( priv->window,
-							priv->input->str );
+					if( wpage == hdata->page ) {
+						screem_window_replace_selection( hdata->window,
+							hdata->input->str );
 					} else {
-						screem_page_set_data( priv->page, priv->input->str );
+						screem_page_set_data( hdata->page, hdata->input->str );
 					}
-					
 					break;
 				case SCREEM_HELPER_STDOUT_INSERT:
+					screem_view_insert( view, -1,
+							hdata->input->str );
 					break;
 				case SCREEM_HELPER_STDOUT_NONE:
 					/* will have been displayed in
@@ -750,26 +1029,38 @@ static void child_watch( GPid pid, gint status, gpointer data )
 		g_free( tmp );
 	}
 	
-	g_string_free( priv->input, TRUE );
-	if( priv->dialog ) {
-		gtk_dialog_response( GTK_DIALOG( priv->dialog ), GTK_RESPONSE_CLOSE );
+	g_string_free( hdata->input, TRUE );
+	if( hdata->dialog ) {
+		gtk_dialog_response( GTK_DIALOG( hdata->dialog ), GTK_RESPONSE_CLOSE );
+		hdata->dialog = NULL;
 	}
 
+	/* free up newdata, removing it from the list */
+	temp = g_slist_find( priv->running, hdata );
+	if( temp ) {
+		priv->running = g_slist_remove( priv->running, hdata );
+		g_free( hdata );
+	} else {
+		g_warning( "Orphan ScreemHelperData\n" );
+	}
+	
 	gdk_threads_leave();
 }
 
 static gboolean child_stdout( GIOChannel *source,
 		GIOCondition condition, gpointer data )
 {
+	ScreemHelperData *hdata;
 	ScreemHelper *helper;
 	ScreemHelperPrivate *priv;
 	GIOStatus status;
 	gchar buf[ BUFSIZ + 1 ];
 	GError *error;
-	gint read;
+	gsize read;
 
-	helper = SCREEM_HELPER( data );
-	priv = helper->private;
+	hdata = (ScreemHelperData*)data;
+	helper = hdata->helper;
+	priv = helper->priv;
 
 	error = NULL;
 	status = G_IO_STATUS_AGAIN;
@@ -777,33 +1068,36 @@ static gboolean child_stdout( GIOChannel *source,
 	gdk_threads_enter();
 	
 	if( condition == G_IO_IN ) {
-		status = g_io_channel_read_chars( source,
-				buf, BUFSIZ, &read, &error );
-		if( status == G_IO_STATUS_NORMAL ||
-			status == G_IO_STATUS_EOF ) {
+		do {
+			read = 0;
+			status = g_io_channel_read_chars( source,
+					buf, BUFSIZ, &read, &error );
+			if( status == G_IO_STATUS_NORMAL ||
+				status == G_IO_STATUS_EOF ) {
 	
-			buf[ read ] = '\0';
-			if( priv->omode == SCREEM_HELPER_STDOUT_NONE ) {
-				screem_window_show_message( priv->window,
-						buf, TRUE );
-			} else {	
-				g_string_append_len( priv->input,
-						buf, read );
+				buf[ read ] = '\0';
+				if( priv->omode == SCREEM_HELPER_STDOUT_NONE ) {
+					screem_window_show_message( hdata->window,
+							buf, TRUE );
+				} else {	
+					g_string_append_len( hdata->input,
+							buf, read );
+				}
 			}
-		}
+		} while( read > 0 );
 		if( error  ) {
 			g_error_free( error );
 		}
 	}
 	if( condition == G_IO_ERR ) {
-		if( priv->dialog ) {
-			gtk_dialog_response( GTK_DIALOG( priv->dialog ),
+		if( hdata->dialog ) {
+			gtk_dialog_response( GTK_DIALOG( hdata->dialog ),
 					GTK_RESPONSE_CANCEL );
 		}
 				
 	} else if( condition == G_IO_HUP || status == G_IO_STATUS_EOF ) {
-		if( priv->dialog ) {
-			gtk_dialog_response( GTK_DIALOG( priv->dialog ),
+		if( hdata->dialog ) {
+			gtk_dialog_response( GTK_DIALOG( hdata->dialog ),
 					GTK_RESPONSE_CLOSE );
 		}
 	}
@@ -816,15 +1110,17 @@ static gboolean child_stdout( GIOChannel *source,
 static gboolean child_stderr( GIOChannel *source,
 		GIOCondition condition, gpointer data )
 {
+	ScreemHelperData *hdata;
 	ScreemHelper *helper;
 	ScreemHelperPrivate *priv;
 	GIOStatus status;
 	gchar buf[ BUFSIZ + 1 ];
 	GError *error;
-	gint read;
+	gsize read;
 
-	helper = SCREEM_HELPER( data );
-	priv = helper->private;
+	hdata = (ScreemHelperData*)data;
+	helper = hdata->helper;
+	priv = helper->priv;
 
 	error = NULL;
 	status = G_IO_STATUS_AGAIN;
@@ -832,15 +1128,18 @@ static gboolean child_stderr( GIOChannel *source,
 	gdk_threads_enter();
 	
 	if( condition == G_IO_IN ) {
-		status = g_io_channel_read_chars( source,
-				buf, BUFSIZ, &read, &error );
-		if( status == G_IO_STATUS_NORMAL ||
-			status == G_IO_STATUS_EOF ) {
+		do {
+			read = 0;
+			status = g_io_channel_read_chars( source,
+					buf, BUFSIZ, &read, &error );
+			if( status == G_IO_STATUS_NORMAL ||
+				status == G_IO_STATUS_EOF ) {
 			
-			buf[ read ] = '\0';
-			screem_window_show_error( priv->window, buf,
-					TRUE );
-		}
+				buf[ read ] = '\0';
+				screem_window_show_error( hdata->window, buf,
+						TRUE );
+			}
+		} while( read > 0 );
 		if( error  ) {
 			g_error_free( error );
 		}
@@ -857,13 +1156,64 @@ static gboolean child_stderr( GIOChannel *source,
 	return TRUE;
 }
 
+static gboolean screem_helper_pulse( ScreemHelperData *hdata )
+{
+	GladeXML *xml;
+	GtkWidget *widget;
 
+	gdk_threads_enter();
+	
+	xml = glade_get_widget_tree( hdata->dialog );
+	
+	widget = glade_xml_get_widget( xml, "helperprogress" );
+	gtk_progress_bar_pulse( GTK_PROGRESS_BAR( widget ) );
+
+	gdk_threads_leave();
+	
+	return TRUE;
+}
+
+/* take from libgnome-desktop/gnome-desktop-item.c */
+static gboolean exec_exists( const char *exec )
+{
+	if( g_path_is_absolute( exec ) ) {
+		if( access( exec, X_OK ) == 0 )
+			return TRUE;
+		else
+			return FALSE;
+	} else {
+		char *tryme;
+
+		tryme = g_find_program_in_path( exec );
+		if( tryme != NULL ) {
+			g_free( tryme );
+			return TRUE;
+		}
+		return FALSE;
+	}
+}
+
+static gchar *screem_helper_build_action_name( const gchar *name )
+{
+	gchar *tmp;
+	gchar *uiname;
+	
+	tmp = screem_escape_char( name, ' ' );
+	uiname = g_strconcat( "Exec_Helper_", tmp, NULL );
+	g_free( tmp );
+	
+	return uiname;
+}
 
 /* G Object stuff */
 
-#define PARENT_TYPE G_TYPE_OBJECT
+G_DEFINE_TYPE( ScreemHelper, screem_helper, G_TYPE_OBJECT )
 
-static gpointer parent_class;
+static void screem_helper_finalize( GObject *object );
+static void screem_helper_set_property(GObject *object, guint prop_id,
+				       const GValue *value, GParamSpec *pspec);
+static void screem_helper_get_property( GObject *object, guint prop_id,
+					GValue *value, GParamSpec *pspec );
 
 static void screem_helper_class_init( ScreemHelperClass *klass )
 {
@@ -871,7 +1221,7 @@ static void screem_helper_class_init( ScreemHelperClass *klass )
 	GParamSpec *spec;
 		
 	object_class = G_OBJECT_CLASS( klass );
-	parent_class = g_type_class_peek_parent( klass );
+	screem_helper_parent_class = g_type_class_peek_parent( klass );
 
 	object_class->finalize = screem_helper_finalize;
 
@@ -928,6 +1278,18 @@ static void screem_helper_class_init( ScreemHelperClass *klass )
 			G_PARAM_READWRITE );
 	g_object_class_install_property( object_class,
 					 PROP_EXEC_ON_SAVE_TYPE, spec );
+
+	spec = g_param_spec_string( "uipath", "UI menu path",
+			"The menu path to use when adding the helper to a GtkUIManager", "",
+			G_PARAM_READWRITE );
+	g_object_class_install_property( object_class,
+					 PROP_UIPATH, spec );
+	
+	spec = g_param_spec_string( "filename", "Helper File",
+			"The .desktop file the helper was loaded from", "",
+			G_PARAM_READWRITE );
+	g_object_class_install_property( object_class,
+					 PROP_FILE, spec );
 	
 	screem_helper_signals[ HELPER_FAILED ] = 
 		g_signal_new( "helper_failed",
@@ -954,8 +1316,8 @@ static void screem_helper_class_init( ScreemHelperClass *klass )
 
 static void screem_helper_init( ScreemHelper *helper )
 {
-	helper->private = g_new0( ScreemHelperPrivate, 1 );
-
+	helper->priv = g_new0( ScreemHelperPrivate, 1 );
+	helper->priv->exists = TRUE;
 }
 
 static void screem_helper_finalize( GObject *object )
@@ -965,23 +1327,29 @@ static void screem_helper_finalize( GObject *object )
 	helper = SCREEM_HELPER( object );
 
 	/* we need to remove outselves from all windows */
-	while( helper->private->list ) {
+	while( helper->priv->list ) {
 		ScreemWindow *window = 
-			SCREEM_WINDOW( helper->private->list->data );
+			SCREEM_WINDOW( helper->priv->list->data );
 
 		screem_helper_remove_from_window( helper, window );
 	}
 
-	if( helper->private->name )
-		g_free( helper->private->name );
-	if( helper->private->pathname )
-		g_free( helper->private->pathname );
-	if( helper->private->iconpath )
-		g_free( helper->private->iconpath );
+	if( helper->priv->name ) {
+		g_free( helper->priv->name );
+	}
+	if( helper->priv->pathname ) {
+		g_free( helper->priv->pathname );
+	}
+	if( helper->priv->iconpath ) {
+		g_free( helper->priv->iconpath );
+	}
+	if( helper->priv->exec_type ) {
+		g_free( helper->priv->exec_type );
+	}
 
-	g_free( helper->private );
+	g_free( helper->priv );
 
-	G_OBJECT_CLASS( parent_class )->finalize( object );
+	G_OBJECT_CLASS( screem_helper_parent_class )->finalize( object );
 }
 
 
@@ -989,43 +1357,58 @@ static void screem_helper_set_property( GObject *object, guint prop_id,
 					const GValue *value, GParamSpec *pspec)
 {
 	ScreemHelper *helper;
-	ScreemHelperPrivate *private;
+	ScreemHelperPrivate *priv;
 	const gchar *val;
 	gint mode;
+	gboolean exists;
+	gint argc;
+	gchar **argv;
 	
 	helper = SCREEM_HELPER( object );
-	private = helper->private;
+	priv = helper->priv;
 
 	switch( prop_id ) {
 	case PROP_PATHNAME:
 		val = g_value_get_string( value );
-		if( private->pathname )
-			g_free( private->pathname );
+		if( priv->pathname )
+			g_free( priv->pathname );
 		if( val ) {
-			private->pathname = g_strdup( val );
+			priv->pathname = g_strdup( val );
+			if( ! g_shell_parse_argv( priv->pathname, &argc, &argv, NULL ) ) {
+			exists = FALSE;	
+			} else if( argc < 1 ) {
+				g_strfreev( argv );
+				exists = FALSE;
+			} else {
+				exists = exec_exists( argv[ 0 ] );
+				g_strfreev( argv );
+			}
 		} else {
-			private->pathname = NULL;
+			priv->pathname = NULL;
+			exists = FALSE;
 		}
+		priv->exists = exists;
+	
 		break;
 	case PROP_ICONPATH:
 		val = g_value_get_string( value );
-		if( private->iconpath )
-			g_free( private->iconpath );
+		if( priv->iconpath )
+			g_free( priv->iconpath );
 		if( val ) {
-			private->iconpath = g_strdup( val );
+			priv->iconpath = g_strdup( val );
 		} else {
-			private->iconpath = NULL;
+			priv->iconpath = NULL;
 		}
 		break;
 	case PROP_HELPER_NAME:
 		val = g_value_get_string( value );
-		if( private->name ) {
-			g_free( private->name );
+		if( priv->name ) {
+			g_free( priv->name );
 		}
 		if( val ) {
-			private->name = g_strdup( val );
+			priv->name = g_strdup( val );
 		} else {
-			private->name = NULL;
+			priv->name = NULL;
 		}
 		break;
 	case PROP_INPUT_MODE:
@@ -1033,22 +1416,32 @@ static void screem_helper_set_property( GObject *object, guint prop_id,
 		if( mode < 0 || mode > SCREEM_HELPER_STDIN_USER ) {
 			mode = SCREEM_HELPER_STDIN_NONE;
 		}
-		private->imode = mode;
+		priv->imode = mode;
 		break;
 	case PROP_OUTPUT_MODE:
 		mode = g_value_get_int( value );
 		if( mode < 0 || mode > SCREEM_HELPER_STDOUT_INSERT ) {
 			mode = SCREEM_HELPER_STDOUT_NONE;
 		}
-		private->omode = mode;
+		priv->omode = mode;
 		break;
 	case PROP_EXEC_ON_SAVE:
-		private->exec = g_value_get_boolean( value );
+		priv->exec = g_value_get_boolean( value );
 		break;
 	case PROP_EXEC_ON_SAVE_TYPE:
 		val = g_value_get_string( value );
-		g_free( private->exec_type );
-		private->exec_type = g_strdup( val );
+		g_free( priv->exec_type );
+		priv->exec_type = g_strdup( val );
+		break;
+	case PROP_UIPATH:
+		val = g_value_get_string( value );
+		g_free( priv->uipath );
+		priv->uipath = g_strdup( val );
+		break;
+	case PROP_FILE:
+		val = g_value_get_string( value );
+		g_free( priv->filename );
+		priv->filename = g_strdup( val );
 		break;
 	default:
 		break;
@@ -1059,45 +1452,59 @@ static void screem_helper_get_property( GObject *object, guint prop_id,
 					GValue *value, GParamSpec *pspec )
 {
 	ScreemHelper *helper;
-	ScreemHelperPrivate *private;
+	ScreemHelperPrivate *priv;
 
 	helper = SCREEM_HELPER( object );
-	private = helper->private;
+	priv = helper->priv;
 
 	switch( prop_id ) {
 	case PROP_PATHNAME:
-		if( private->pathname ) {
-			g_value_set_string( value, private->pathname );
+		if( priv->pathname ) {
+			g_value_set_string( value, priv->pathname );
 		} else {
 			g_value_set_string( value, "" );
 		}
 		break;
 	case PROP_ICONPATH:
-		if( private->iconpath ) {
-			g_value_set_string( value, private->iconpath );
+		if( priv->iconpath ) {
+			g_value_set_string( value, priv->iconpath );
 		} else {
 			g_value_set_string( value, "" );
 		}
 		break;
 	case PROP_HELPER_NAME:
-		if( private->name ) {
-			g_value_set_string( value, private->name );
+		if( priv->name ) {
+			g_value_set_string( value, priv->name );
 		} else {
 			g_value_set_string( value, "" );
 		}
 		break;
 	case PROP_INPUT_MODE:
-		g_value_set_int( value, private->imode );
+		g_value_set_int( value, priv->imode );
 		break;
 	case PROP_OUTPUT_MODE:
-		g_value_set_int( value, private->omode );
+		g_value_set_int( value, priv->omode );
 		break;
 	case PROP_EXEC_ON_SAVE:
-		g_value_set_boolean( value, private->exec );
+		g_value_set_boolean( value, priv->exec );
 		break;
 	case PROP_EXEC_ON_SAVE_TYPE:
-		if( private->exec_type ) {
-			g_value_set_string( value, private->exec_type );
+		if( priv->exec_type ) {
+			g_value_set_string( value, priv->exec_type );
+		} else {
+			g_value_set_string( value, "" );
+		}
+		break;
+	case PROP_UIPATH:
+		if( priv->uipath ) {
+			g_value_set_string( value, priv->uipath );
+		} else {
+			g_value_set_string( value, "" );
+		}
+		break;
+	case PROP_FILE:
+		if( priv->filename ) {
+			g_value_set_string( value, priv->filename );
 		} else {
 			g_value_set_string( value, "" );
 		}
@@ -1108,29 +1515,3 @@ static void screem_helper_get_property( GObject *object, guint prop_id,
 	}
 }
 
-
-
-GType screem_helper_get_type()
-{
-	static GType type = 0;
-	
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemHelperClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_helper_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemHelper ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_helper_init
-		};
-
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemHelper",
-					       &info, 0 );
-	}
-
-	return type;
-}
