@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -36,18 +36,14 @@
 
 #include <gconf/gconf-client.h>
 
+#include <glade/glade.h>
+
 #include <glib/gi18n.h>
-#include <libgnome/gnome-util.h>
 
 #include "screem-hint.h"
 
 #include "support.h"
 
-static GList *hint_windows = NULL;
-
-static void screem_hint_init( ScreemHint *hint );
-static void screem_hint_class_init( ScreemHintClass *hint );
-static void screem_hint_finalize( GObject *hint );
 static void screem_hint_show( GtkWidget *widget );
 static void screem_hint_load( ScreemHint *hint );
 static void screem_hint_store( ScreemHint *hint, gchar *str );
@@ -66,25 +62,18 @@ ScreemHint *screem_hint_new()
 
 gboolean screem_hint_next_hint( GtkWidget *widget )
 {
-	GladeXML *xml;
-	GList *list;
 	ScreemHint *hint;
 	GtkWidget *label;
 	GConfClient *client;
 
 	client = gconf_client_get_default();
 
-	xml = glade_get_widget_tree( widget );
-	hint = NULL;
-
-	for( list = hint_windows; list; list = list->next ) {
-		hint = SCREEM_HINT( list->data );
-		if( xml == hint->xml ) {
-			break;
-		}
+	while( gtk_widget_get_parent( widget ) ) {
+		widget = gtk_widget_get_parent( widget );
 	}
 
-	g_assert( hint );
+	g_assert( SCREEM_IS_HINT( widget ) );
+	hint = SCREEM_HINT( widget );
 
 	label = hint->label;
 
@@ -106,25 +95,18 @@ gboolean screem_hint_next_hint( GtkWidget *widget )
 
 gboolean screem_hint_prev_hint( GtkWidget *widget )
 {
-	GladeXML *xml;
-	GList *list;
 	ScreemHint *hint;
 	GtkWidget *label;
 	GConfClient *client;
 
 	client = gconf_client_get_default();
 
-	xml = glade_get_widget_tree( widget );
-	hint = NULL;
-
-	for( list = hint_windows; list; list = list->next ) {
-		hint = SCREEM_HINT( list->data );
-		if( xml == hint->xml ) {
-			break;
-		}
+	while( gtk_widget_get_parent( widget ) ) {
+		widget = gtk_widget_get_parent( widget );
 	}
 
-	g_assert( hint );
+	g_assert( SCREEM_IS_HINT( widget ) );
+	hint = SCREEM_HINT( widget );
 
 	label = hint->label;
 
@@ -146,28 +128,20 @@ gboolean screem_hint_prev_hint( GtkWidget *widget )
 
 gboolean screem_hint_hide_hint( GtkWidget *widget )
 {
-	GladeXML *xml;
-	GList *list;
 	ScreemHint *hint;
 	gint x;
 	gint y;
 	
-	xml = glade_get_widget_tree( widget );
-	hint = NULL;
-
-	for( list = hint_windows; list; list = list->next ) {
-		hint = SCREEM_HINT( list->data );
-		if( xml == hint->xml ) {
-			gtk_window_get_position( GTK_WINDOW( hint ), 
-					&x, &y );
-			gtk_widget_hide( GTK_WIDGET( hint ) );
-			gtk_window_move( GTK_WINDOW( hint ), x, y );
-
-			break;
-		}
+	while( gtk_widget_get_parent( widget ) ) {
+		widget = gtk_widget_get_parent( widget );
 	}
 
-	g_assert( hint );
+	g_assert( SCREEM_IS_HINT( widget ) );
+	hint = SCREEM_HINT( widget );
+	
+	gtk_window_get_position( GTK_WINDOW( hint ), &x, &y );
+	gtk_widget_hide( GTK_WIDGET( hint ) );
+	gtk_window_move( GTK_WINDOW( hint ), x, y );
 
 	return TRUE;
 }
@@ -189,16 +163,15 @@ gboolean screem_hint_hide_on_startup( GtkWidget *widget )
 
 /* G Object stuff */
 
-#define PARENT_TYPE GTK_TYPE_WINDOW
+G_DEFINE_TYPE( ScreemHint, screem_hint, GTK_TYPE_WINDOW )
 
-static gpointer parent_class;
+static void screem_hint_finalize( GObject *hint );
 
 static void screem_hint_class_init( ScreemHintClass *klass )
 {
 	GObjectClass *object_class;
 
 	object_class = G_OBJECT_CLASS( klass );
-	parent_class = g_type_class_peek_parent( klass );
 
 	object_class->finalize = screem_hint_finalize;
 
@@ -207,39 +180,37 @@ static void screem_hint_class_init( ScreemHintClass *klass )
 
 static void screem_hint_init( ScreemHint *hint )
 {
+	GladeXML *xml;
 	GConfClient *client;
 	GtkWidget *table;
 	GtkWidget *toggle;
 	GtkWidget *image;
-	gchar *gladepath;
 
 	gchar *temp;
 
 	gtk_window_set_title( GTK_WINDOW( hint ),
 			      _( "Tip of the day - Screem" ) );
 
-	gladepath = screem_get_glade_path();
-	hint->xml = glade_xml_new( gladepath, "hint_table",
-				   NULL );
-	g_free( gladepath );
+	xml = glade_xml_new( GLADE_PATH"/screem.glade", "hint_table", NULL );
 
 	client = gconf_client_get_default();
 	hint->show_hint = gconf_client_get_bool( client,
 						 "/apps/screem/general/show_tips",
 						 NULL );
 
-	toggle = glade_xml_get_widget( hint->xml, "show_tips" );
+	toggle = glade_xml_get_widget( xml, "show_tips" );
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( toggle ),
 				      hint->show_hint );
-	glade_xml_signal_autoconnect( hint->xml );
+	glade_xml_signal_autoconnect( xml );
 
-	table = glade_xml_get_widget( hint->xml, "hint_table" );
+	table = glade_xml_get_widget( xml, "hint_table" );
+	g_object_set_data( G_OBJECT( table ), "hint", hint );
 	gtk_container_add( GTK_CONTAINER( hint ), table );
 	gtk_widget_show_all( table );
 
-	hint->label = glade_xml_get_widget( hint->xml, "hint_label" );
+	hint->label = glade_xml_get_widget( xml, "hint_label" );
 
-	image = glade_xml_get_widget( hint->xml, "hint_icon" );
+	image = glade_xml_get_widget( xml, "hint_icon" );
 	gtk_image_set_from_file( GTK_IMAGE( image ), 
 			DATADIR"/pixmaps/screem.png" );
 	
@@ -268,7 +239,6 @@ static void screem_hint_init( ScreemHint *hint )
 	gtk_window_set_type_hint( GTK_WINDOW( hint ), GDK_WINDOW_TYPE_HINT_DIALOG );
 	gtk_window_set_wmclass( GTK_WINDOW( hint ),
 				"Screem", "hint_window" );
-	hint_windows = g_list_append( hint_windows, hint );
 
 	hint->last_hint = gconf_client_get_int( client,
 						"/apps/screem/hint/number",
@@ -279,6 +249,7 @@ static void screem_hint_init( ScreemHint *hint )
 			NULL );
 
 	g_object_unref( G_OBJECT( client ) );
+	g_object_unref( xml );
 }
 
 static void screem_hint_finalize( GObject *hint )
@@ -288,14 +259,13 @@ static void screem_hint_finalize( GObject *hint )
 
 	h = SCREEM_HINT( hint );
 	
-	g_object_unref( h->xml );
 	g_free( h->hint_file );
 	for( i = 0; i < h->hint_count; ++ i ) {
 		g_free( h->hint_text[ i ] );
 	}
 	g_free( h->hint_text );
 
-	G_OBJECT_CLASS( parent_class )->finalize( hint );
+	G_OBJECT_CLASS( screem_hint_parent_class )->finalize( hint );
 }
 
 static void screem_hint_show( GtkWidget *widget )
@@ -310,33 +280,9 @@ static void screem_hint_show( GtkWidget *widget )
 	gtk_window_set_position( GTK_WINDOW( hint ), 
 				GTK_WIN_POS_CENTER_ON_PARENT );
 	
-	GTK_WIDGET_CLASS( parent_class )->show( widget );
+	GTK_WIDGET_CLASS( screem_hint_parent_class )->show( widget );
 }
 
-GType screem_hint_get_type()
-{
-	static GType type = 0;
-
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemHintClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_hint_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemHint ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_hint_init
-		};
-
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemHint",
-					       &info, 0 );
-	}
-
-	return type;
-}
 
 static void screem_hint_load( ScreemHint *hint )
 {
