@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -86,7 +86,7 @@ ScreemLinkViewHtml *screem_link_view_html_new( ScreemLinkView *view,
 	g_assert( SCREEM_IS_SITE( site ) );
 
 	islocal = TRUE;
-	if( pathname && strncmp( "file:", pathname, strlen( "file:" ) ) ) {
+	if( pathname && ! g_str_has_prefix( pathname, "file://" ) ) {
 		islocal = FALSE;
 		uri = gnome_vfs_uri_new( pathname );
 		if( uri ) {
@@ -366,11 +366,6 @@ static gint item_event( GnomeCanvasItem *group, GdkEvent *event,
 							"IgnoreFlag" );
 		gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
 						status );
-		status = screem_site_is_ascii( site, path );
-		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
-							"ASCIIFlag" );
-		gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
-						status );
 	} else if( popup ) {
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"ExcludeFlag" );
@@ -378,9 +373,6 @@ static gint item_event( GnomeCanvasItem *group, GdkEvent *event,
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"IgnoreFlag" );
 		g_object_set( G_OBJECT( action ), "visible", FALSE, NULL );
-		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
-							"ASCIIFlag" );
-		g_object_set( G_OBJECT( action ), "visible", FALSE, NULL );
 	}
 
 	switch( event->type ) {
@@ -452,13 +444,6 @@ static gint item_event( GnomeCanvasItem *group, GdkEvent *event,
 		} else {
 			screem_site_remove_ignore( site, path );
 		}
-		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
-							"ASCIIFlag" );
-		if( gtk_toggle_action_get_active( GTK_TOGGLE_ACTION( action ) ) ) {
-			screem_site_add_ascii( site, path );
-		} else {
-			screem_site_remove_ascii( site, path );
-		}
 	} else if( popup ) {
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"ExcludeFlag" );
@@ -466,9 +451,6 @@ static gint item_event( GnomeCanvasItem *group, GdkEvent *event,
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"IgnoreFlag" );
 		g_object_set( G_OBJECT( action ), "visible", TRUE, NULL );
-		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
-							"ASCIIFlag" );
-		g_object_set( G_OBJECT( action ), "visible", TRUE, NULL );
 	}
 
 	g_object_unref( site );
@@ -525,7 +507,7 @@ static void screem_link_view_html_finalize( GObject *link_view_html )
 
 GType screem_link_view_html_get_type()
 {
-	static guint type = 0;
+	static GType type = 0;
 
 	if( ! type ) {
 		static const GTypeInfo info = {
