@@ -485,7 +485,7 @@ static GNode *build_link_tree( ScreemLinkView *view,
 	GNode *ret = NULL;
 
 	ScreemPage *page;
-	gchar *mime_type;
+	const gchar *mime_type;
 	GNode *node = NULL;
 	ScreemDTD *dtd;
 
@@ -536,9 +536,6 @@ static GNode *build_link_tree( ScreemLinkView *view,
 			node = g_node_new( html );
 		}
 	}
-	if( mime_type ) {
-		g_free( mime_type );
-	}
 	
 	/* page is NULL for external links, non existant link or non text files,
 	   node should null for images
