@@ -1087,6 +1087,11 @@ GSList *screem_markup_build_attributes_list( const gchar *tag,
 				equals = g_utf8_next_char( equals );
 				tmp = equals;
 				do {
+					/* FIXME: hack to avoid
+					 * infinite loop on \" */
+					if( *(tmp-1) == '\\' ) {
+						tmp = g_utf8_next_char( tmp );
+					}
 					tmp = g_utf8_strchr( tmp,
 							-1, c );
 				} while( tmp && *(tmp-1) == '\\' );
@@ -1261,6 +1266,8 @@ static void process_attrs( GString *str, const gchar *tag,
 	
 	GCompareFunc compare;
 
+	gchar *unescaped_value;
+	
 	compare = NULL;
 	if( dtd ) {
 		g_object_get( G_OBJECT( dtd ), 
@@ -1280,10 +1287,17 @@ static void process_attrs( GString *str, const gchar *tag,
 		aname = tmp->data;
 		tmp = tmp->next;
 		value = tmp->data;
+		unescaped_value = NULL;
 
 		aoffset = strstr( offset, aname );
 
-		g_assert( aoffset != NULL );
+		if( ! aoffset ) {
+			/* this can occur with inline script where
+			 * screem can get confused,
+			 * we shouldn't be calling this function
+			 * in script tags though */
+			g_assert( aoffset != NULL );
+		}
 		
 		if( aoffset != offset ) {
 			chars = g_utf8_pointer_to_offset( offset,
@@ -1325,6 +1339,8 @@ static void process_attrs( GString *str, const gchar *tag,
 			       ! compare("data", name ) ||
 			       ! compare("href", name ) ||
 			       ! compare("url", name ) ) ) ) ) {
+				
+				unescaped_value = gnome_vfs_unescape_string( value, "/" );
 
 				uri = gnome_vfs_uri_new( value );
 				fragment = "";
@@ -1333,7 +1349,7 @@ static void process_attrs( GString *str, const gchar *tag,
 				}
 				if( ! update_all ) {
 					match = paths_match( pageroot,
-							value,
+							unescaped_value,
 							source );
 					newpath = relative_path( dest,
 							pageroot );
@@ -1344,7 +1360,7 @@ static void process_attrs( GString *str, const gchar *tag,
 						g_free( temp );
 					}
 				} else {
-					match = relative_to_full( value,
+					match = relative_to_full( unescaped_value,
 							pageroot );
 					newpath = relative_path( match,
 							destroot );
@@ -1371,11 +1387,14 @@ static void process_attrs( GString *str, const gchar *tag,
 		}
 		g_free( aname );
 		g_free( value );
+		g_free( unescaped_value );
 
 	}
 	g_slist_free( attrs );
+
 	/* add any extra spacing extra that there was, and also
 	 * the possible / */
+
 	g_string_append( str, offset );
 }
 
@@ -1406,6 +1425,7 @@ static void screem_markup_fix_html_links( ScreemSite *site,
 	gboolean ineditable;
 	
 	GString *str;
+	gboolean inscript;
 
 	text = screem_page_get_data( page );
 	offset_text = text;
@@ -1425,7 +1445,7 @@ static void screem_markup_fix_html_links( ScreemSite *site,
 					strlen( dest ) ) );
 	}
 	dtd = screem_page_get_dtd( page );
-	ineditable = FALSE;
+	ineditable = inscript = FALSE;
 	
 	if( ! dtd ) {
 		g_warning( "Page without assigned dtd\n" );
@@ -1470,9 +1490,8 @@ static void screem_markup_fix_html_links( ScreemSite *site,
 					ineditable = FALSE;
 				}
 
-			}
-
-			if( ( ! ineditable ) &&
+			} 
+			if( ( ! ineditable ) && ( ! inscript ) &&
 				*name != '/' &&
 				*name != '!' &&
 				*name != '?' &&
@@ -1490,6 +1509,15 @@ static void screem_markup_fix_html_links( ScreemSite *site,
 				g_string_append( str, tag );
 			}
 			
+			/* we don't want to decend into script tags */
+			if( ! ineditable ) {
+				if( ! g_strcasecmp( name, "script" ) ) {
+					inscript = TRUE;
+				} else if( ! g_strcasecmp( name, "/script" ) ) {
+					inscript = FALSE;
+				}
+			}
+			
 			g_free( tag );
 			g_free( name );
 		} 
@@ -1745,11 +1773,15 @@ void screem_markup_update_from_template( ScreemSite *site,
 	
 	const gchar *template_path;
 	const gchar *page_path;
+
+	guint len;
 	
 	if( ! screem_page_load( page, NULL ) ) {
 		return;
 	}
-		
+	
+	len = strlen( template_tag );
+	
 	template_path = screem_page_get_pathname( template_page );
 	page_path = screem_page_get_pathname( page );
 	
@@ -1766,7 +1798,7 @@ void screem_markup_update_from_template( ScreemSite *site,
 	}
 	
 	newdata = g_string_new( NULL );
-	start += strlen( template_tag );
+	start += len;
 	
 	g_string_append_len( newdata, text, start - text );
 	
