@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -23,9 +23,6 @@
 #include "config.h"
 
 #include <glib/gi18n.h>
-#include <libgnome/gnome-macros.h>
-
-#include <libgnomeui/gnome-popup-menu.h>
 
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -44,7 +41,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "screem-file-browser.h"
 #include "screem-mdi.h"
 #include "screem-page.h"
 #include "screem-window.h"
@@ -57,16 +53,12 @@
 
 #include "gdl-stock.h"
 
-static void screem_mdi_class_init( ScreemMDIClass *klass );
-static void screem_mdi_instance_init( ScreemMDI *mdi );
-static void screem_mdi_finalize( GObject *object );
-static void screem_mdi_set_prop( GObject *object, guint prop_id,
-				const GValue *value, GParamSpec *spec );
-static void screem_mdi_get_prop( GObject *object, guint prop_id,
-				GValue *value, GParamSpec *spec );
+#include "screem-application.h"
+#include "screem-icon-cache.h"
 
+extern ScreemApplication *app;
 
-static void screem_mdi_window_set( ScreemMDI *mdi );
+static gboolean screem_mdi_window_set( ScreemMDI *mdi );
 static void screem_mdi_switch_doc( GtkAction *action, gpointer data );
 static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi, 
 					 ScreemPage *page );
@@ -115,8 +107,6 @@ enum {
 
 static guint screem_mdi_signals[ LAST_SIGNAL ] = { 0 };
 
-static ScreemFileBrowser *browser = NULL;
-
 enum {
 	MDI_PATHNAME,
 	MDI_PIXBUF,
@@ -126,7 +116,8 @@ enum {
 };
 
 struct ScreemMDIPrivate {
-
+	guint set;
+	
 	GList *documents;
 	GtkNotebook *notebook;
 	GtkTooltips *tip;
@@ -143,9 +134,17 @@ struct ScreemMDIPrivate {
 	GtkWidget *dialog;
 
 	GtkListStore *store;
+
+	ScreemIconCache *cache;
 };
 
-GNOME_CLASS_BOILERPLATE( ScreemMDI, screem_mdi, GObject, G_TYPE_OBJECT )
+G_DEFINE_TYPE( ScreemMDI, screem_mdi, G_TYPE_OBJECT )
+
+static void screem_mdi_finalize( GObject *object );
+static void screem_mdi_set_prop( GObject *object, guint prop_id,
+				const GValue *value, GParamSpec *spec );
+static void screem_mdi_get_prop( GObject *object, guint prop_id,
+				GValue *value, GParamSpec *spec );
 
 static void screem_mdi_class_init( ScreemMDIClass *klass )
 {
@@ -178,7 +177,7 @@ static void screem_mdi_class_init( ScreemMDIClass *klass )
 			      G_TYPE_OBJECT );
 }
 
-static void screem_mdi_instance_init( ScreemMDI *mdi )
+static void screem_mdi_init( ScreemMDI *mdi )
 {
 	ScreemMDIPrivate *priv;
 	GtkPositionType position;
@@ -209,16 +208,14 @@ static void screem_mdi_instance_init( ScreemMDI *mdi )
 					  G_TYPE_STRING,
 					  G_TYPE_POINTER );
 
-	if( ! browser ) {
-		browser = screem_file_browser_new();
-	}
+	priv->cache = screem_application_get_icon_cache( app );
 }
 
 static void screem_mdi_finalize( GObject *object )
 {
 	ScreemMDI *mdi;
 	ScreemMDIPrivate *priv;
-	
+		
 	g_return_if_fail( object != NULL );
 	g_return_if_fail( SCREEM_IS_MDI( object ) );
 
@@ -236,6 +233,10 @@ static void screem_mdi_finalize( GObject *object )
 	
 	g_list_free( priv->documents );
 
+	if( priv->cache ) {
+		g_object_unref( priv->cache );
+	}
+	
 	if( priv->dialog ) {
 		gtk_widget_destroy( priv->dialog );
 	}
@@ -243,7 +244,7 @@ static void screem_mdi_finalize( GObject *object )
 	
 	g_free( priv );
 	
-	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
+	G_OBJECT_CLASS( screem_mdi_parent_class )->finalize( object );
 }
 
 static void screem_mdi_set_prop( GObject *object, guint prop_id,
@@ -258,7 +259,8 @@ static void screem_mdi_set_prop( GObject *object, guint prop_id,
 	switch( prop_id ) {
 		case PROP_WINDOW:
 			priv->window = g_value_get_object( value );
-			screem_mdi_window_set( mdi );
+			priv->set = g_idle_add( (GSourceFunc)screem_mdi_window_set,
+					mdi );
 			break;
 		default:
 			break;
@@ -284,7 +286,7 @@ static void screem_mdi_get_prop( GObject *object, guint prop_id,
 }
 
 /* static stuff */
-static void screem_mdi_window_set( ScreemMDI *mdi )
+static gboolean screem_mdi_window_set( ScreemMDI *mdi )
 {
 	ScreemMDIPrivate *priv;
 	GConfClient *client;
@@ -299,6 +301,7 @@ static void screem_mdi_window_set( ScreemMDI *mdi )
 	gchar *name;
 	
 	priv = mdi->priv;
+	priv->set = 0;
 
 	priv->merge = GTK_UI_MANAGER( priv->window->merge );
 	
@@ -365,15 +368,15 @@ static void screem_mdi_window_set( ScreemMDI *mdi )
 							NULL );
 
 	action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
-	g_object_set( G_OBJECT( action ),
-			"sensitive", FALSE, NULL );
+	gtk_action_set_sensitive( GTK_ACTION( action ), FALSE );
 	action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
-	g_object_set( G_OBJECT( action ),
-			"sensitive", FALSE, NULL );
+	gtk_action_set_sensitive( GTK_ACTION( action ), FALSE );
 	
 	g_string_free( str, TRUE );
 	
 	g_object_unref( client );
+
+	return FALSE;
 }
 
 static void screem_mdi_switch_doc( GtkAction *action, gpointer data )
@@ -412,8 +415,13 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 	gint h;
 	
 	const gchar *pathname;
+	const gchar *mime;
+	const gchar *showtype;
 	gchar *tiptext;
-	
+	gchar *tmp;
+	ScreemDTD *dtd;
+	const gchar *doctype;
+
 	g_return_val_if_fail( SCREEM_IS_MDI( mdi ), NULL );
 	g_return_val_if_fail( SCREEM_IS_PAGE( page ), NULL );
 	
@@ -435,8 +443,11 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 			  G_CALLBACK( screem_mdi_page_change ), 
 			  clabel );
 	g_signal_connect( G_OBJECT( page ), "notify::mime-type",
-			  G_CALLBACK( screem_mdi_page_change ), 
-			  clabel );
+			  G_CALLBACK( screem_mdi_page_path_change ), 
+			  label );
+	g_signal_connect( G_OBJECT( page ), "notify::dtd",
+			  G_CALLBACK( screem_mdi_page_path_change ), 
+			  label );
 	
 	g_signal_connect( G_OBJECT( page ), "can_undo",
 			  G_CALLBACK( screem_mdi_can_undo ),
@@ -473,8 +484,8 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 			GTK_ICON_SIZE_MENU );
 	g_object_set_data( G_OBJECT( clabel ), "img", img );
 	gtk_box_pack_start( GTK_BOX( box ), img, FALSE, FALSE, 0 );
-	gtk_box_pack_start( GTK_BOX( box ), label, TRUE, TRUE, 0 );
 	gtk_box_pack_start( GTK_BOX( box ), clabel, FALSE, FALSE, 0 );
+	gtk_box_pack_start( GTK_BOX( box ), label, TRUE, TRUE, 0 );
 	gtk_widget_show_all( box );
 	gtk_container_add( GTK_CONTAINER( event ), box );
 	g_object_set_data( G_OBJECT( event ), "document", 
@@ -489,9 +500,33 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 	if( ! pathname ) {
 		pathname = _( "Unsaved Document" );
 	}
-	tiptext = gnome_vfs_unescape_string_for_display( pathname );
+	tmp = screem_page_get_title_name( page );
+	mime = screem_page_get_mime_type( page );
+	showtype = gnome_vfs_mime_get_description( mime );
+	if( ! showtype ) {
+		showtype = mime;
+	}
+	if( ! screem_page_is_markup( page ) ) {
+		doctype = _( "N/A" );
+	} else {
+		dtd = screem_page_get_dtd( page );
+		if( ! dtd ) {
+			doctype = _( "None" );
+		} else {
+			doctype = screem_dtd_get_public_id( dtd );
+		}
+	}
+	
+	tiptext = g_strconcat( _( "File: " ), tmp, "\n",
+			_( "Type: " ), showtype, "\n",
+			_( "Doctype: " ), doctype,
+			NULL );
+	g_free( tmp );
+	
 	gtk_tooltips_set_tip( priv->tip, event, tiptext, 
 			      NULL );
+	g_object_set_data_full( G_OBJECT( event ), "tipobj",
+			priv->tip, (GDestroyNotify)gtk_object_destroy );
 	g_free( tiptext );
 
 	gtk_widget_show_all( tbox );
@@ -533,6 +568,7 @@ static gboolean screem_mdi_notebook_add( ScreemMDI *mdi,
 					screem_mdi_notebook_change,
 					mdi );
 
+	error = NULL;
 	if( ! pathname || screem_page_load( page, &error ) ) {
 		tab = screem_mdi_create_tab( mdi, page );
 		hack = gtk_hbox_new( 0, FALSE );
@@ -578,7 +614,7 @@ static gboolean screem_mdi_notebook_add( ScreemMDI *mdi,
 		}
 		primary = g_strdup_printf( _( "Failed to load %s" ),
 				temp );
-		screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+		screem_hig_alert( GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 				primary, secondary, 
 				GTK_WIDGET( priv->window ) );
 		g_free( primary );
@@ -596,7 +632,6 @@ static gboolean screem_mdi_notebook_add( ScreemMDI *mdi,
 					0, 0, NULL, 
 					screem_mdi_notebook_change,
 					mdi );
-	
 	return loaded;
 }
 
@@ -607,19 +642,11 @@ static void screem_mdi_get_page_labels( ScreemPage *page,
 	const gchar *pathname;
 	gchar *filename;
 	gchar *temp;
-	GnomeVFSURI *uri;
 
 	pathname = screem_page_get_pathname( page );
 	
-	if( pathname && ( strlen( pathname ) > 0 ) ) {
-		uri = gnome_vfs_uri_new( pathname );
-		filename = gnome_vfs_uri_extract_short_name( uri );
-		gnome_vfs_uri_unref( uri );
-		temp = gnome_vfs_unescape_string_for_display( filename );
-		g_free( filename );
-		filename = temp;
-	} else {
-		filename = g_strdup( _( "Untitled" ) );
+	filename = screem_page_get_short_name( page );
+	if( ( ! pathname ) || ( *pathname == '\0' ) ) {
 		pathname = _( "Unsaved Document" );
 	}
 		
@@ -647,7 +674,6 @@ static void screem_mdi_page_path_change( ScreemPage *page,
 	GtkWidget *event;
 	GtkTooltipsData *ttip;
 	gchar *filename;
-	gchar *title;
 	gchar *temp;
 	gint i;
 	gchar *name;
@@ -657,7 +683,14 @@ static void screem_mdi_page_path_change( ScreemPage *page,
 	ScreemPage *cpage;
 	GdkPixbuf *pixbuf;
 	const gchar *pathname;
-	
+	const gchar *type;
+	gchar *title;
+
+	const gchar *showtype;
+	gchar *tiptext;
+	ScreemDTD *dtd;
+	const gchar *doctype;
+
 	label = GTK_WIDGET( data );
 	
 	mdi = SCREEM_MDI( g_object_get_data( G_OBJECT( label ), "mdi" ) );
@@ -668,8 +701,9 @@ static void screem_mdi_page_path_change( ScreemPage *page,
 
 	pathname = screem_page_get_pathname( page );
 	modified = screem_page_get_changed( page );
+	type = screem_page_get_mime_type( page );
 	screem_mdi_get_page_labels( page, &filename, &temp );
-	
+
 	gtk_label_set_text( GTK_LABEL( label ), filename );
 		
 	if( spec ) {
@@ -711,16 +745,19 @@ static void screem_mdi_page_path_change( ScreemPage *page,
 	}
 	pixbuf = NULL;
 	if( pathname ) {
-		pixbuf = screem_file_browser_get_icon( browser, 
-					pathname, 16, 16, FALSE,
-					NULL );
+		pixbuf = screem_icon_cache_get_pixbuf( priv->cache,
+				pathname, type, 16 );
 	}
+		
+	title = screem_page_get_title_name( page );
+	
 	gtk_list_store_set( priv->store, &it,
-			MDI_PATHNAME, temp,
+			MDI_PATHNAME, title,
 			MDI_MODIFIED, modified ? "*" : "",
 			MDI_PIXBUF, pixbuf,
 			MDI_PAGE, page,
 			-1 );
+	g_free( title );
 	if( pixbuf ) {
 		g_object_unref( G_OBJECT( pixbuf ) );
 	}
@@ -730,22 +767,42 @@ static void screem_mdi_page_path_change( ScreemPage *page,
 	if( event ) {
 		event = event->parent;
 
+		showtype = gnome_vfs_mime_get_description( type );
+		if( ! showtype ) {
+			showtype = type;
+		}
+		if( ! screem_page_is_markup( page ) ) {
+			doctype = _( "N/A" );
+		} else {
+			dtd = screem_page_get_dtd( page );
+			if( ! dtd ) {
+				doctype = _( "None" );
+			} else {
+				doctype = screem_dtd_get_public_id( dtd );
+			}
+		}
+		
+		g_free( temp );
+		temp = screem_page_get_title_name( page );
+	
+		tiptext = g_strconcat( _( "File: " ), temp, "\n",
+			_( "Type: " ), showtype, "\n",
+			_( "Doctype: " ), doctype,
+			NULL );
 		ttip = gtk_tooltips_data_get( event );
-		gtk_tooltips_set_tip( ttip->tooltips, event,
-					temp, "" );
-	}
-
-	if( modified ) {
-		title = g_strdup_printf( "%s * - Screem", temp );
-	} else {
-		title = g_strdup_printf( "%s - Screem", temp );
+		if( ttip ) {
+			gtk_tooltips_set_tip( ttip->tooltips, event,
+						tiptext, "" );
+		} else {
+			g_warning( "Missing tooltip on file tab\n" );
+		}
+		g_free( tiptext );
 	}
 
 	if( page == priv->current ) {
-		gtk_window_set_title( GTK_WINDOW( window ), title );
+		screem_window_set_title( window, page );
 	}
 	g_free( temp );
-	g_free( title );
 }
 
 static void screem_mdi_page_change( ScreemPage *page, 
@@ -758,14 +815,11 @@ static void screem_mdi_page_change( ScreemPage *page,
 	GdkPixbuf *pixbuf;
 	ScreemWindow *window;
 	const gchar *pathname;
-	gchar *temp;
-	gchar *title;
 	gboolean modified;
 	GtkTreeIter it;
 	ScreemPage *cpage;
 
 	const gchar *type;
-	const gchar *showtype;
 	
 	label = GTK_WIDGET( data );
 	img = g_object_get_data( G_OBJECT( label ), "img" );
@@ -774,38 +828,26 @@ static void screem_mdi_page_change( ScreemPage *page,
 	window = priv->window;
 	
 	type = screem_page_get_mime_type( page );
-	showtype = gnome_vfs_mime_get_description( type );
-	if( ! showtype ) {
-		showtype = type;
-	}
 
-	pixbuf = screem_file_browser_get_icon( browser,
-			NULL, 16, 16, FALSE, (gchar**)&type );
+	pixbuf = screem_icon_cache_get_pixbuf( priv->cache,
+				NULL, type, 16 );
+
 	if( pixbuf ) {
 		gtk_image_set_from_pixbuf( GTK_IMAGE( img ), pixbuf );
 		g_object_unref( pixbuf );
 	}
 
 	pathname = screem_page_get_pathname( page );
-	if( ! pathname ) {
-		pathname = _( "Untitled" );
-		temp = g_strconcat( pathname, " ", showtype, NULL );
-	} else {
-		temp = gnome_vfs_unescape_string_for_display( pathname );
-	}
-	
 	modified = screem_page_get_changed( page );
-		
-	if( modified ) {
-		gtk_label_set_text( GTK_LABEL( label ), "*" );
-		title = g_strdup_printf( "* %s - Screem", temp );
-	} else {
-		gtk_label_set_text( GTK_LABEL( label ), "" );
-		title = g_strdup_printf( "%s - Screem", temp );
-	}
 
 	if( page == priv->current ) {
-		gtk_window_set_title( GTK_WINDOW( window ), title );
+		screem_window_set_title( window, page );
+	}
+
+	if( modified ) {
+		gtk_label_set_text( GTK_LABEL( label ), " *" );
+	} else {
+		gtk_label_set_text( GTK_LABEL( label ), "" );
 	}
 
 	if( spec && 
@@ -822,9 +864,7 @@ static void screem_mdi_page_change( ScreemPage *page,
 		gtk_list_store_set( priv->store, &it,
 				MDI_MODIFIED, modified ? "*" : "",
 				-1 );
-	}	
-	g_free( temp );
-	g_free( title );
+	}
 }
 
 static void screem_mdi_notebook_change( GtkNotebook *notebook, 
@@ -892,18 +932,11 @@ static void screem_mdi_tab_clicked( GtkWidget *label,
 		
 		for( docs = priv->documents; docs; docs = docs->next ) {
 			GtkWidget *submenuitem;
-			const gchar *pathname;
-			gchar *base;
 			gchar *temp;
 			
 			page = SCREEM_PAGE( docs->data );
-			pathname = screem_page_get_pathname( page );
-			
-			if( ! pathname ) {
-				pathname = _( "Untitled" );
-			}
-			base = g_path_get_basename( pathname );
-			temp = gnome_vfs_unescape_string_for_display( base );
+		
+			temp = screem_page_get_short_name( page );
 			submenuitem = gtk_menu_item_new_with_label( temp );
 			g_free( temp );
 			gtk_widget_show( submenuitem );
@@ -914,7 +947,6 @@ static void screem_mdi_tab_clicked( GtkWidget *label,
 					mdi );
 			gtk_menu_shell_append( GTK_MENU_SHELL( submenu ),
 						submenuitem );
-			g_free( base );
 		}
 		
 		screem_popup_menu_do_popup_modal( menu, 0, 0, event, 0, label );
@@ -1038,13 +1070,9 @@ static void screem_mdi_real_remove_document( ScreemMDI *mdi,
 	   page is closed, so we need handle it here */
 	if( ! priv->documents ) {
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
-		g_object_set( G_OBJECT( action ),
-				"sensitive", FALSE,
-				NULL );
+		gtk_action_set_sensitive( GTK_ACTION( action ), FALSE );
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
-		g_object_set( G_OBJECT( action ),
-				"sensitive", FALSE,
-				NULL );
+		gtk_action_set_sensitive( GTK_ACTION( action ), FALSE );
 		g_signal_emit( G_OBJECT( mdi ), 
 				screem_mdi_signals[ CHANGED ],
 				0, NULL, page );
@@ -1064,7 +1092,6 @@ static void screem_mdi_document_dialog( GtkAction *action,
 	ScreemMDIPrivate *priv;
 	ScreemApplication *app;
 	ScreemSession *session;
-	gchar *glade_path;
 	GladeXML *xml;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *col;
@@ -1078,9 +1105,8 @@ static void screem_mdi_document_dialog( GtkAction *action,
 	session = screem_application_get_session( app );
 	
 	if( ! priv->dialog ) {
-		glade_path = screem_get_glade_path();
-		xml = glade_xml_new( glade_path, "documents", NULL );
-		g_free( glade_path );
+		xml = glade_xml_new( GLADE_PATH"/screem.glade",
+				"documents", NULL );
 
 		priv->dialog = glade_xml_get_widget( xml, "documents" );
 		g_signal_connect( G_OBJECT( priv->dialog ),
@@ -1319,8 +1345,8 @@ static void screem_mdi_can_undo( GtkSourceBuffer *buffer,
 	
 	if( page && ( GTK_SOURCE_BUFFER( page ) == buffer ) ) {
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "Undo" );
-		g_object_set( G_OBJECT( action ), "sensitive", can_undo,
-				NULL );
+		gtk_action_set_sensitive( GTK_ACTION( action ), can_undo );
+
 	}
 }
 
@@ -1339,8 +1365,7 @@ static void screem_mdi_can_redo( GtkSourceBuffer *buffer,
 	page = screem_mdi_get_current( mdi );
 	if( page && ( GTK_SOURCE_BUFFER( page ) == buffer ) ) {
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "Redo" );
-		g_object_set( G_OBJECT( action ), "sensitive", can_redo,
-				NULL );
+		gtk_action_set_sensitive( GTK_ACTION( action ), can_redo );
 	}
 }
 
@@ -1493,12 +1518,11 @@ gboolean screem_mdi_set_current( ScreemMDI *mdi, ScreemPage *page )
 		len = g_list_length( priv->documents );
 
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
-		g_object_set( G_OBJECT( action ),
-				"sensitive", index > 0, NULL );
+		gtk_action_set_sensitive( GTK_ACTION( action ), 
+				index > 0 );
 		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
-		g_object_set( G_OBJECT( action ),
-				"sensitive", index < ( len - 1 ), 
-				NULL );
+		gtk_action_set_sensitive( GTK_ACTION( action ), 
+				index < ( len - 1 ) );
 		
 		g_signal_handlers_unblock_matched( G_OBJECT( priv->notebook ),
 						G_SIGNAL_MATCH_FUNC|
@@ -1513,12 +1537,12 @@ gboolean screem_mdi_set_current( ScreemMDI *mdi, ScreemPage *page )
 		if( page ) {
 			action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "Undo" );
 			can = gtk_source_buffer_can_undo( GTK_SOURCE_BUFFER( page ) );
-			g_object_set( G_OBJECT( action ), "sensitive", can,
-				NULL );
+			gtk_action_set_sensitive( GTK_ACTION( action ),
+					can );
 			action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "Redo" );
 			can = gtk_source_buffer_can_redo( GTK_SOURCE_BUFFER( page ) );
-			g_object_set( G_OBJECT( action ), "sensitive", can,
-					NULL );
+			gtk_action_set_sensitive( GTK_ACTION( action ),
+					can );
 		}
 	}
 
@@ -1536,6 +1560,12 @@ gboolean screem_mdi_add_document( ScreemMDI *mdi, ScreemPage *page )
 
 	priv = mdi->priv;
 
+	if( priv->set ) {
+		g_source_remove( priv->set );
+		priv->set = 0;
+		screem_mdi_window_set( mdi );
+	}
+	
 	/* lookup page in list */
 	index = g_list_index( priv->documents, page );
 	ret = TRUE;
