@@ -1524,6 +1524,9 @@ static void screem_page_model_build_open( ScreemPageModelParser *parser )
 			if( ! value ) {
 				value = g_strdup( name );
 			}
+			/* FIXME: 
+			 * hmm seems to be causing massive leaking
+			 * here */
 			xmlSetProp( node, (xmlChar*)name, 
 					(xmlChar*)value );
 			
@@ -1602,6 +1605,7 @@ static void screem_page_model_build_open( ScreemPageModelParser *parser )
 	xmlclosecheck = g_utf8_prev_char( xmlclosecheck );
 	xmlclosecheck = g_utf8_prev_char( xmlclosecheck );
 	if(  ( firstc != '!' ) && ( firstc != '?' ) &&
+	     ( firstc != '%' ) &&
 	    ( *xmlclosecheck != '/' ) &&
 		( close_state != SCREEM_DTD_MUST_NOT ) ) {
 		parser->parent = gtk_tree_iter_copy( &it );
