@@ -209,41 +209,48 @@ void screem_page_set_pathname( ScreemPage *page, const gchar *path )
 
 static void screem_page_set_pathname_real( ScreemPage *page, const gchar *path )
 {	
-	if( page->private->pathname && path != page->private->pathname ) {
-		g_free( page->private->pathname );
+	ScreemPagePrivate *priv;
+
+	priv = page->private;
+	
+	if( priv->pathname && path != priv->pathname ) {
+		g_free( priv->pathname );
 	}
 
-	if( page->private->mime_type ) {
-		g_free( page->private->mime_type );
+	if( priv->mime_type ) {
+		g_free( priv->mime_type );
 	}
-	page->private->mime_type = NULL;
+	priv->mime_type = NULL;
 	
 	if( path ) {
-		page->private->mime_type = 
+		priv->mime_type = 
 			screem_get_mime_type( path, FALSE );
+		if( priv->mime_type ) {
+			priv->mime_type = g_strdup( priv->mime_type );
+		}
 	}
-	if( ! page->private->mime_type ) {
-		page->private->mime_type = gconf_client_get_string( page->private->client, "/apps/screem/default_mimetype", NULL );
-		if( ! page->private->mime_type ) {
-			page->private->mime_type = g_strdup( "text/html" );
+	if( ! priv->mime_type ) {
+		priv->mime_type = gconf_client_get_string( priv->client, "/apps/screem/default_mimetype", NULL );
+		if( ! priv->mime_type ) {
+			priv->mime_type = g_strdup( "text/html" );
 		}
 	}
 
-	if( path != page->private->pathname ) {
+	if( path != priv->pathname ) {
 		if( path ) {
-			page->private->pathname = g_strdup( path );
+			priv->pathname = g_strdup( path );
 		} else {
-			page->private->pathname = NULL;
+			priv->pathname = NULL;
 		}
 	}
 	
-	if( page->private->monitor ) {
-		gnome_vfs_monitor_cancel( page->private->monitor );
-		page->private->monitor = NULL;
+	if( priv->monitor ) {
+		gnome_vfs_monitor_cancel( priv->monitor );
+		priv->monitor = NULL;
 	}
-	if( page->private->pathname ) {
-		gnome_vfs_monitor_add( &page->private->monitor,
-				       page->private->pathname,
+	if( priv->pathname ) {
+		gnome_vfs_monitor_add( &priv->monitor,
+				       priv->pathname,
 				       GNOME_VFS_MONITOR_FILE,
 				       screem_page_monitor_cb, page );
 	}
@@ -658,7 +665,7 @@ glong screem_page_get_seconds_since_last_save_or_load( ScreemPage *page )
 
 gboolean screem_page_is_file_page( const gchar *path )
 {
-	gchar *mime_type;
+	const gchar *mime_type;
 	gboolean ret;
 	
 	g_return_val_if_fail( path != NULL, FALSE );
@@ -666,7 +673,6 @@ gboolean screem_page_is_file_page( const gchar *path )
 	mime_type = screem_get_mime_type( path, FALSE );
 
 	ret =  screem_page_is_mime_type_page( mime_type );
-	g_free( mime_type );
 
 	return ret;
 }
