@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -57,15 +57,6 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 static void url_clicked(  HtmlDocument *document, const gchar *url,
 			  ScreemPreview *preview );
 
-static void screem_preview_class_init( ScreemPreviewClass *klass );
-static void screem_preview_init( ScreemPreview *preview );
-static void screem_preview_finalize( GObject *preview );
-static void screem_preview_window_set( ScreemView *view );
-static void screem_preview_size_request( GtkWidget *widget, GtkRequisition *req );
-static void screem_preview_realize( GtkWidget *widget );
-static void screem_preview_show( GtkWidget *widget );
-
-
 static void screem_preview_async_cb( GnomeVFSAsyncHandle *handle,
 				     GnomeVFSResult result,
 				     gpointer data );
@@ -141,10 +132,10 @@ static void screem_preview_display_view( ScreemView *view )
 		      "online", &online,
 		      NULL );
 
-	g_mutex_lock( preview->private->streams_lock );
-	g_hash_table_foreach( preview->private->streams,
+	g_mutex_lock( preview->priv->streams_lock );
+	g_hash_table_foreach( preview->priv->streams,
 				(GHFunc)screem_preview_cancel_all, preview );
-	g_mutex_unlock( preview->private->streams_lock );
+	g_mutex_unlock( preview->priv->streams_lock );
 	
 	if( site && ! screem_site_get_fake_flag( site ) ) {
 		baseuri = screem_site_get_http_url( site );
@@ -273,16 +264,16 @@ void screem_preview_display( ScreemPreview *preview, const gchar *text,
 	}
 
 	document = screem_preview_new_document( preview );
-	old = HTML_VIEW( preview->private->html )->document;
-	html_view_set_document( HTML_VIEW( preview->private->html ), document );
+	old = HTML_VIEW( preview->priv->html )->document;
+	html_view_set_document( HTML_VIEW( preview->priv->html ), document );
 	if( old ) {
 		g_object_unref( old );
 	}
 	
-	if( preview->private->base_uri ) {
-		g_free( preview->private->base_uri );
+	if( preview->priv->base_uri ) {
+		g_free( preview->priv->base_uri );
 	}
-	preview->private->base_uri = g_strdup( base_uri );
+	preview->priv->base_uri = g_strdup( base_uri );
 	
 	html_document_open_stream( document, "text/html" );
 	html_document_write_stream( document, text, strlen( text ) );
@@ -297,12 +288,12 @@ void screem_preview_display( ScreemPreview *preview, const gchar *text,
 		base_uri = "file:///";
 	}
 
-	if( preview->private->base_uri ) {
-		g_free( preview->private->base_uri );
+	if( preview->priv->base_uri ) {
+		g_free( preview->priv->base_uri );
 	}
-	preview->private->base_uri = g_strdup( base_uri );
+	preview->priv->base_uri = g_strdup( base_uri );
 
-	gtk_moz_embed_render_data( GTK_MOZ_EMBED( preview->private->html ),
+	gtk_moz_embed_render_data( GTK_MOZ_EMBED( preview->priv->html ),
 				   text, strlen( text ),
 				   base_uri, "text/html" );
 #endif
@@ -316,16 +307,16 @@ void screem_preview_uri( ScreemPreview *preview, const gchar *uri,
 	HtmlDocument *old;
 
 	document = screem_preview_new_document( preview );
-	old = HTML_VIEW( preview->private->html )->document;
-	html_view_set_document( HTML_VIEW( preview->private->html ), document );
+	old = HTML_VIEW( preview->priv->html )->document;
+	html_view_set_document( HTML_VIEW( preview->priv->html ), document );
 	if( old ) {
 		g_object_unref( old );
 	}
 	
-	if( preview->private->base_uri ) {
-		g_free( preview->private->base_uri );
+	if( preview->priv->base_uri ) {
+		g_free( preview->priv->base_uri );
 	}
-	preview->private->base_uri = g_path_get_dirname( uri );
+	preview->priv->base_uri = g_path_get_dirname( uri );
 
 	html_document_open_stream( document, "text/html" );
 
@@ -351,7 +342,7 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 	gchar *msg;
 	const gchar *sitebase;
 	
-	if( document != HTML_VIEW( preview->private->html )->document ) {
+	if( document != HTML_VIEW( preview->priv->html )->document ) {
 		html_stream_close( stream );
 		g_object_unref( document );
 		return;
@@ -378,7 +369,7 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 				sitebase = screem_site_get_pathname( site );
 			}
 		} else if( ! uri_exists( url, NULL ) ) {
-			sitebase = preview->private->base_uri;
+			sitebase = preview->priv->base_uri;
 		}
 		if( sitebase ) {
 			full = g_strconcat( sitebase, url, NULL );
@@ -386,7 +377,7 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 	}
 	if( ! full ) {
 		full = relative_to_full( url, 
-				preview->private->base_uri );
+				preview->priv->base_uri );
 	}
 	
 	if( full ) {
@@ -406,10 +397,10 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 		g_free( full );
 		full = gnome_vfs_uri_to_string( uri, 0 );
 		if( gnome_vfs_uri_is_local( uri ) || online ) {
-			g_mutex_lock( preview->private->streams_lock );
+			g_mutex_lock( preview->priv->streams_lock );
 			
 			/* async open, the callback will wait on
-			   preview->private->streams_lock before
+			   preview->priv->streams_lock before
 			   doing anything */
 			g_object_ref( document );
 			gnome_vfs_async_open( &handle, full, GNOME_VFS_OPEN_READ,
@@ -419,12 +410,12 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 					      preview );
 			
 			/* put the stream in the hash table */
-			g_hash_table_insert( preview->private->streams,
+			g_hash_table_insert( preview->priv->streams,
 						handle, stream );
 			g_object_set_data( G_OBJECT( stream ), "document",
 					   document );
 			
-			g_mutex_unlock( preview->private->streams_lock );
+			g_mutex_unlock( preview->priv->streams_lock );
 		} else {
 			msg = g_strconcat( "Not fetching remote uri: ",
 						full, NULL );
@@ -457,11 +448,11 @@ static void screem_preview_async_close_cb( GnomeVFSAsyncHandle *handle,
 	preview = SCREEM_PREVIEW( data );
 	g_object_get( G_OBJECT( preview ), "window", &window, NULL );
 	
-	g_mutex_lock( preview->private->streams_lock );
+	g_mutex_lock( preview->priv->streams_lock );
 
-	g_hash_table_remove( preview->private->streams, handle );
+	g_hash_table_remove( preview->priv->streams, handle );
 	
-	g_mutex_unlock( preview->private->streams_lock );
+	g_mutex_unlock( preview->priv->streams_lock );
 }
 
 static void url_clicked( HtmlDocument *document, const gchar *url, 
@@ -483,9 +474,9 @@ static void screem_preview_async_cb( GnomeVFSAsyncHandle *handle,
 	preview = SCREEM_PREVIEW( data );
 	g_object_get( G_OBJECT( preview ), "window", &window, NULL );
 	
-	g_mutex_lock( preview->private->streams_lock );
+	g_mutex_lock( preview->priv->streams_lock );
 	
-	stream = HTML_STREAM( g_hash_table_lookup( preview->private->streams,
+	stream = HTML_STREAM( g_hash_table_lookup( preview->priv->streams,
 						   handle ) );
 	document = NULL;
 	if( ! stream ) {
@@ -494,11 +485,11 @@ static void screem_preview_async_cb( GnomeVFSAsyncHandle *handle,
 		document = g_object_get_data( G_OBJECT( stream ), "document" );
 	}
 
-	if( document != HTML_VIEW( preview->private->html )->document ) {
+	if( document != HTML_VIEW( preview->priv->html )->document ) {
 		result = GNOME_VFS_ERROR_GENERIC;
 	}
 	
-	g_mutex_unlock( preview->private->streams_lock );
+	g_mutex_unlock( preview->priv->streams_lock );
 	
 	switch( result ) {
 		case GNOME_VFS_OK:
@@ -538,9 +529,9 @@ static void screem_preview_async_read_cb( GnomeVFSAsyncHandle *handle,
 	preview = SCREEM_PREVIEW( data );
 	g_object_get( G_OBJECT( preview ), "window", &window, NULL );
 	
-	g_mutex_lock( preview->private->streams_lock );
+	g_mutex_lock( preview->priv->streams_lock );
 
-	stream = HTML_STREAM( g_hash_table_lookup( preview->private->streams,
+	stream = HTML_STREAM( g_hash_table_lookup( preview->priv->streams,
 						   handle ) );
 	document = NULL;
 	if( ! stream ) {
@@ -549,11 +540,11 @@ static void screem_preview_async_read_cb( GnomeVFSAsyncHandle *handle,
 		document = g_object_get_data( G_OBJECT( stream ), "document" );
 	}
 
-	if( document != HTML_VIEW( preview->private->html )->document ) {
+	if( document != HTML_VIEW( preview->priv->html )->document ) {
 		result = GNOME_VFS_ERROR_GENERIC;
 	}
 	
-	g_mutex_unlock( preview->private->streams_lock );
+	g_mutex_unlock( preview->priv->streams_lock );
 	
 	switch( result ) {
 		case GNOME_VFS_OK:
@@ -597,9 +588,13 @@ static HtmlDocument *screem_preview_new_document( ScreemPreview *preview )
 
 /* G Object stuff */
 
-#define PARENT_TYPE SCREEM_TYPE_VIEW
+static void screem_view_iface_init( ScreemView *iface );
 
-static gpointer parent_class;
+G_DEFINE_TYPE( ScreemPreview, screem_preview, SCREEM_TYPE_VIEW )
+
+static void screem_preview_finalize( GObject *preview );
+static void screem_preview_window_set( ScreemView *view );
+static void screem_preview_show( GtkWidget *widget );
 
 static void screem_preview_class_init( ScreemPreviewClass *klass )
 {
@@ -608,13 +603,16 @@ static void screem_preview_class_init( ScreemPreviewClass *klass )
 
 	object_class = G_OBJECT_CLASS( klass );
 	widget_class = (GtkWidgetClass *)klass;
-	parent_class = g_type_class_peek_parent( klass );
 
 	object_class->finalize = screem_preview_finalize;
 
 	widget_class->show = screem_preview_show;
-	widget_class->realize = screem_preview_realize;
-	widget_class->size_request = screem_preview_size_request;
+}
+
+static void screem_view_iface_init( ScreemView *iface )
+{
+	iface->display = screem_preview_display_view;
+	iface->print = screem_preview_print;
 }
 
 static void screem_preview_init( ScreemPreview *preview )
@@ -623,19 +621,21 @@ static void screem_preview_init( ScreemPreview *preview )
 	GtkWidget *sw;
 	gchar *profile;
 	
-	ScreemPreviewPrivate *private;
+	ScreemPreviewPrivate *priv;
 
-	SCREEM_VIEW( preview )->window_set = screem_preview_window_set;
-	SCREEM_VIEW( preview )->display = screem_preview_display_view;
-	SCREEM_VIEW( preview )->print = screem_preview_print;
+	screem_view_iface_init( SCREEM_VIEW( preview ) );
+	
+	g_signal_connect( G_OBJECT( preview ), "notify::window",
+			G_CALLBACK( screem_preview_window_set ),
+			NULL );
 
-	private = preview->private = g_new0( ScreemPreviewPrivate, 1 );
+	priv = preview->priv = g_new0( ScreemPreviewPrivate, 1 );
 #ifndef SCREEM_MOZ
-	html = private->html = html_view_new();
+	html = priv->html = html_view_new();
 
-	gtk_widget_show( private->html );
+	gtk_widget_show( priv->html );
 
-	sw = preview->private->sw = 
+	sw = preview->priv->sw = 
 		gtk_scrolled_window_new( GTK_LAYOUT( html )->hadjustment,
 					 GTK_LAYOUT( html )->vadjustment );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( sw ),
@@ -650,14 +650,14 @@ static void screem_preview_init( ScreemPreview *preview )
 #else
 	gtk_moz_embed_set_profile_path( "/home/david/.gnome2/epiphany/mozilla", "epiphany" );
 	gtk_moz_embed_set_comp_path( g_getenv( "MOZILLA_FIVE_HOME" ) );
-	html = private->html = gtk_moz_embed_new();
+	html = priv->html = gtk_moz_embed_new();
 
 	gtk_moz_embed_set_chrome_mask( html, GTK_MOZ_EMBED_FLAG_ALLCHROME );
 	
 	gtk_container_add( GTK_CONTAINER( preview ), html );
 #endif
-	preview->private->streams = g_hash_table_new( NULL, NULL );
-	preview->private->streams_lock = g_mutex_new();
+	preview->priv->streams = g_hash_table_new( NULL, NULL );
+	preview->priv->streams_lock = g_mutex_new();
 }
 
 static void screem_preview_finalize( GObject *object )
@@ -666,14 +666,14 @@ static void screem_preview_finalize( GObject *object )
 
 	preview = SCREEM_PREVIEW( object );
 
-	g_mutex_lock( preview->private->streams_lock );
-	g_hash_table_foreach( preview->private->streams,
+	g_mutex_lock( preview->priv->streams_lock );
+	g_hash_table_foreach( preview->priv->streams,
 				(GHFunc)screem_preview_cancel_all, preview );
-	g_mutex_unlock( preview->private->streams_lock );
+	g_mutex_unlock( preview->priv->streams_lock );
 
 	/* FIXME: leaking the mutex / hashtable here */
 		
-	G_OBJECT_CLASS( parent_class )->finalize( object );
+	G_OBJECT_CLASS( screem_preview_parent_class )->finalize( object );
 }
 
 static gboolean screem_preview_cancel_all( gpointer key,
@@ -690,52 +690,18 @@ static void screem_preview_window_set( ScreemView *view )
 
 }
 
-static void screem_preview_size_request( GtkWidget *widget, GtkRequisition *req )
-{
-	GTK_WIDGET_CLASS( parent_class )->size_request( widget, req );
-}
-
-static void screem_preview_realize( GtkWidget *widget )
-{
-	GTK_WIDGET_CLASS( parent_class )->realize( widget );
-}
-
 static void screem_preview_show( GtkWidget *widget )
 {
 	ScreemPreview *preview;
 
 	preview = SCREEM_PREVIEW( widget );
 
-	GTK_WIDGET_CLASS( parent_class )->show( widget );
+	GTK_WIDGET_CLASS( screem_preview_parent_class )->show( widget );
 
-	gtk_widget_show( preview->private->html );
+	gtk_widget_show( preview->priv->html );
 
-	if( preview->private->sw ) {
-		gtk_widget_show( preview->private->sw );
+	if( preview->priv->sw ) {
+		gtk_widget_show( preview->priv->sw );
 	}
 }
 
-GType screem_preview_get_type()
-{
-	static GType type = 0;
-
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemPreviewClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_preview_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemPreview ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_preview_init
-		};
-		
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemPreview",
-					       &info, 0 );
-	}
-
-	return type;
-}
