@@ -14,18 +14,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #include "config.h"
 
-#include <libgnome/gnome-macros.h>
 #include <glib/gi18n.h>
 
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-#include <libgnomeui/gnome-entry.h>
-
 #include <glib/glist.h>
 
 #include <glade/glade.h>
@@ -33,29 +30,25 @@
 #include <regex.h>
 #include <string.h>
 
-#include "screem-editor.h"
-#include "screem-file-browser.h"
 #include "screemmarshal.h"
 #include "screem-page.h"
 #include "screem-search.h"
 #include "screem-window.h"
 
+#include "fileops.h"
 #include "support.h"
 
-static void screem_search_class_init( ScreemSearchClass *klass );
-static void screem_search_instance_init( ScreemSearch *search );
-static void screem_search_finalize( GObject *object );
-static void screem_search_set_prop( GObject *object, guint prop_id,
-				const GValue *value, GParamSpec *spec );
-static void screem_search_get_prop( GObject *object, guint prop_id,
-				GValue *value, GParamSpec *spec );
+#include "screem-application.h"
+#include "screem-icon-cache.h"
+
+extern ScreemApplication *app;
 
 struct ScreemSearchPrivate {
 	gint found_at;		/* this is a byte offset */
-	gint found_c;		/* this is a utf8 char offset */
+	glong found_c;		/* this is a utf8 char offset */
 	
 	guint length;		/* this is the byte length */
-	guint length_c;		/* this is the utf8 char length */
+	glong length_c;		/* this is the utf8 char length */
 	
 	gboolean matches;
 	
@@ -63,6 +56,9 @@ struct ScreemSearchPrivate {
 	gchar *find_pattern;
 	gchar *replace_pattern;
 
+	size_t nmatch;
+	regmatch_t *pmatch;
+
 	ScreemSearchType search_type;
 	ScreemSearchFrom search_from;
 
@@ -70,7 +66,9 @@ struct ScreemSearchPrivate {
 	GtkListStore *match_model;
 
 	ScreemWindow *window;
-	ScreemEditor *editor;
+	ScreemView *editor;
+
+	ScreemIconCache *cache;
 };
 
 typedef enum {
@@ -92,12 +90,15 @@ enum {
 	PROP_EDITOR
 };
 
-static ScreemFileBrowser *browser = NULL;
 static guint screem_search_signals[ LAST_SIGNAL ] = { 0 };
 	
-GNOME_CLASS_BOILERPLATE( ScreemSearch, screem_search,
-			  GObject,
-			  G_TYPE_OBJECT )
+G_DEFINE_TYPE( ScreemSearch, screem_search, G_TYPE_OBJECT )
+
+static void screem_search_finalize( GObject *object );
+static void screem_search_set_prop( GObject *object, guint prop_id,
+				const GValue *value, GParamSpec *spec );
+static void screem_search_get_prop( GObject *object, guint prop_id,
+				GValue *value, GParamSpec *spec );
 
 static void screem_search_class_init( ScreemSearchClass *klass )
 {
@@ -152,7 +153,7 @@ static void screem_search_class_init( ScreemSearchClass *klass )
 }
 
 
-static void screem_search_instance_init( ScreemSearch *search )
+static void screem_search_init( ScreemSearch *search )
 {
 	ScreemSearchPrivate *priv;
 	
@@ -165,10 +166,7 @@ static void screem_search_instance_init( ScreemSearch *search )
 				    GDK_TYPE_PIXBUF,
 				    G_TYPE_POINTER,
 				    NULL );
-
-	if( ! browser ) {
-		browser = screem_file_browser_new();
-	}
+	priv->cache = screem_application_get_icon_cache( app );
 }
 
 static void screem_search_finalize( GObject *object )
@@ -188,10 +186,18 @@ static void screem_search_finalize( GObject *object )
 	}
 	g_free( priv->find_pattern );
 	g_free( priv->replace_pattern );
+
+	if( priv->pmatch ) {
+		g_free( priv->pmatch );
+	}
+
+	if( priv->cache ) {
+		g_object_unref( G_OBJECT( priv->cache ) );
+	}
 	
 	g_free( priv );
 	
-	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
+	G_OBJECT_CLASS( screem_search_parent_class )->finalize( object );
 }
 
 static void screem_search_set_prop( GObject *object, guint prop_id,
@@ -258,9 +264,10 @@ static void screem_search_change_file( GtkTreeView *view,
 	gtk_tree_model_get( model, &it, 
 			    SCREEM_SEARCH_MATCH_PATH, &pathname,
 			    -1 );
-
+	
 	site = screem_window_get_current( window );
 	page = screem_site_locate_page( site, pathname );
+
 	g_free( pathname );
 
 	if( page ) {
@@ -330,22 +337,20 @@ static void screem_search_add_match( ScreemSearch *search,
 	pixbuf = NULL;
 	if( path ) {
 		temp = gnome_vfs_unescape_string_for_display( path );
-		
-		pixbuf = screem_file_browser_get_icon( browser, path,
-							16, 16,
-							FALSE,
-							NULL );
+	
+		pixbuf = screem_icon_cache_get_pixbuf( priv->cache,
+				path, NULL, 16 );
 	} else {
 		temp = g_strdup( _( "Untitled" ) );
 	}
 	
-	gtk_list_store_append( GTK_LIST_STORE( store ), &it );
-	
-	gtk_list_store_set( GTK_LIST_STORE( store ), &it,
-			    SCREEM_SEARCH_MATCH_PATH, temp, 
-			    SCREEM_SEARCH_MATCH_PIXBUF, pixbuf, 
-			    SCREEM_SEARCH_MATCH_OBJECT, search,
-			    -1 );
+	/* use large pos value to ensure append */	
+	gtk_list_store_insert_with_values( GTK_LIST_STORE( store ), 
+			&it, G_MAXINT32,
+			SCREEM_SEARCH_MATCH_PATH, temp, 
+			SCREEM_SEARCH_MATCH_PIXBUF, pixbuf, 
+			SCREEM_SEARCH_MATCH_OBJECT, search,
+			-1 );
 	g_free( temp );
 
 	if( pixbuf ) {
@@ -379,6 +384,7 @@ static void screem_search_has_match( gpointer data,
 	GError *error;
 	gchar *temp;
 	gchar *primary;
+	gboolean loaded;
 	
 	g_return_if_fail( SCREEM_IS_SEARCH( search ) );
 	g_return_if_fail( SCREEM_IS_PAGE( data ) );
@@ -395,19 +401,25 @@ static void screem_search_has_match( gpointer data,
 
 	/* is path excluded? */
 	error = NULL;
-	if( ( ! screem_site_is_excluded( site, path ) ) &&
-		screem_page_load( page, &error ) ) {
-
-		text = screem_page_get_data( page );
-		if( screem_search_static( text,	priv->find_pattern, 
-				priv->regexp, 0, &priv->length ) != -1 ) {
-			screem_search_add_match( search, page );
+	loaded = screem_page_is_loaded( page ) || screem_page_is_open( page );
+	if( ( ! screem_site_is_excluded( site, path ) ) ) {
+		if( loaded ) {
+			text = screem_page_get_data( page );
+		} else {
+			text = load_file( path, NULL, NULL, NULL );
 		}
+		/* we don't care about submatches here, we only want
+		   to know if there is a match */
+		if( screem_search_static( text,	priv->find_pattern, 
+				priv->regexp, 0, &priv->length,
+				NULL, NULL ) != -1 ) {
+			screem_search_add_match( search, page );
+		} 
 		g_free( text );
 	} else if( error ) {
 		temp = gnome_vfs_unescape_string_for_display( path );
 		primary = g_strdup_printf( _( "Failed to load: %s" ), temp );
-		screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+		screem_hig_alert( GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 				primary, error->message, GTK_WIDGET( window ) );
 		g_free( primary );
 		g_free( temp );
@@ -415,26 +427,39 @@ static void screem_search_has_match( gpointer data,
 	}
 }
 
-static void screem_search_find_change( GtkWidget *widget,
-					ScreemSearch *search )
+static gboolean screem_search_find_change( GtkWidget *widget,
+		GdkEventFocus *event, ScreemSearch *search )
 {
 	const gchar *pattern;
 	
-	g_return_if_fail( SCREEM_IS_SEARCH( search ) );
+	g_return_val_if_fail( SCREEM_IS_SEARCH( search ), FALSE );
 	
 	pattern = gtk_entry_get_text( GTK_ENTRY( widget ) );
 	screem_search_set_find_pattern( search, pattern );
+	
+	return FALSE;
 }
 
-static void screem_search_replace_change( GtkWidget *widget,
-					ScreemSearch *search )
+static void screem_search_find_activate( GtkWidget *widget,
+		ScreemSearch *search )
+{
+	g_return_if_fail( SCREEM_IS_SEARCH( search ) );
+
+	screem_search_find_change( widget, NULL, search );		
+}
+
+
+static gboolean screem_search_replace_change( GtkWidget *widget,
+		GdkEventFocus *event, ScreemSearch *search )
 {
 	const gchar *pattern;
 	
-	g_return_if_fail( SCREEM_IS_SEARCH( search ) );
+	g_return_val_if_fail( SCREEM_IS_SEARCH( search ), FALSE );
 	
 	pattern = gtk_entry_get_text( GTK_ENTRY( widget ) );
 	screem_search_set_replace_pattern( search, pattern );
+	
+	return FALSE;
 }
 
 static void screem_search_regexp_toggle( GtkToggleButton *widget,
@@ -486,10 +511,10 @@ static void screem_search_pos_toggle( GtkToggleButton *widget,
 	
 	if( gtk_toggle_button_get_active( widget ) ) {
 		screem_search_set_from( search,
-				SCREEM_SEARCH_START );
+				SCREEM_SEARCH_START, TRUE );
 	} else {
 		screem_search_set_from( search,
-					SCREEM_SEARCH_CURRENT );
+					SCREEM_SEARCH_CURRENT, TRUE );
 	}
 }
 
@@ -499,7 +524,7 @@ static void screem_search_do_replace_all( ScreemSearch *search )
 	
 	g_return_if_fail( SCREEM_IS_SEARCH( search ) );
 	
-	screem_search_set_from( search, SCREEM_SEARCH_START );
+	screem_search_set_from( search, SCREEM_SEARCH_START, TRUE );
 
 	priv = search->priv;
 	
@@ -526,17 +551,19 @@ static void screem_search_do_replace_all( ScreemSearch *search )
  *         performed
  */
 gchar *find_text( const gchar *string, const gchar *find, 
-		  const gchar *replace, gint *length )
+		  const gchar *replace, guint *length )
 {
 	gchar *chunk1;
 	gchar *chunk2;
 	gchar *newChunk;
 	gchar *text = NULL;
 	gint found_at;
-	gint find_length;
+	guint find_length;
 
-       	found_at = screem_search_static( string, find, 
-			TRUE, 0, &find_length );
+	/* find_text is called from code rather user based search,
+	   no need for submatches yet */
+	found_at = screem_search_static( string, find, 
+			TRUE, 0, &find_length, NULL, NULL );
     
 	if( found_at == -1 ) {
 		return NULL;
@@ -567,39 +594,71 @@ gchar *find_text( const gchar *string, const gchar *find,
  *
  * @text:      the text to search
  * @find:      the string to find
- * @use_regex: search uses regular expressions
  * @start:     offset in string to start searching at
+ * @use_regex:	use regexp
  * @length:    a pointer to an integer to store the length of the match in,
  *             can be NULL if the value isn't interested in
+ * @nmatch:		out value, number of submatches in the regexp
+ * @pmatch:		 out value, positions of matches
  *
  * finds a string inside a piece of text
  *
  * returns: the position in text that find was found
  */
+
 gint screem_search_static( const gchar *text, const gchar *find, 
-			gboolean use_regex, gint start, gint *length )
+			gboolean use_regexp, gint start, guint *length, 
+			size_t *nmatch, regmatch_t **pmatch )
 {
 	gint retval = start;
 
+	g_return_val_if_fail( text != NULL, -1 );
+	g_return_val_if_fail( find != NULL, -1 );
+	
 	g_assert( length != NULL );
 
-	if( use_regex ) {
+	if( use_regexp ) {
+		int failed;
 		regex_t *expressions;
-		size_t nmatch = 1;
-		regmatch_t pmatch[ 1 ];
+		size_t junk;
+		regmatch_t *pjunk;
+		
+		if( ! nmatch ) {
+			nmatch = &junk;
+		}
+		if( ! pmatch ) {
+			pmatch = &pjunk;
+		}
 		
 		expressions = g_new0( regex_t, 1 );
 		
-		if( regcomp( expressions, find, REG_EXTENDED | REG_NEWLINE ) ||
-		    regexec( expressions, text + start, nmatch, pmatch, 0 ) ) {
-			retval = -1;
-		} else {
-			 retval += pmatch[ 0 ].rm_so;
+		failed = ( regcomp( expressions, find,
+					REG_EXTENDED | REG_NEWLINE ) );
+		*pmatch = NULL;
+		if( ! failed ) {
+			*nmatch = (expressions)->re_nsub + 1;
+			*pmatch = g_new0( regmatch_t, *nmatch );
+			
+		    failed = regexec( expressions, 
+				    text + start, *nmatch, *pmatch, 0 );
 		}
+		
+		if( failed ) {
+			retval = -1;
+			*length = 0;
+			g_free( *pmatch );
+			*pmatch = NULL;
+		} else {
+			 retval += (*pmatch)[ 0 ].rm_so;
+			 *length = (*pmatch)[0].rm_eo - (*pmatch)[0].rm_so;
+			 
+			 if( pmatch == &pjunk ) {
+				g_free( *pmatch );
+			}
+		}
+			
 		regfree( expressions );
 		g_free( expressions );
-		
-		*length = pmatch[0].rm_eo - pmatch[0].rm_so;
 	} else {
 		/* not using regular expressions */
 		gchar *pos;
@@ -645,7 +704,7 @@ ScreemSearchType screem_search_get_search_type( const ScreemSearch *search )
 }
 
 void screem_search_set_from( ScreemSearch *search,
-		ScreemSearchFrom from )
+		ScreemSearchFrom from, gboolean reset )
 {
 	GladeXML *xml;
 	GtkWidget *widget;
@@ -655,12 +714,14 @@ void screem_search_set_from( ScreemSearch *search,
 			  from <= SCREEM_SEARCH_CURRENT );
 
 	search->priv->search_from = from;
-	search->priv->found_at = -1;
-
-	if( search->priv->dialog ) {
-		xml = glade_get_widget_tree( search->priv->dialog );
-		widget = glade_xml_get_widget( xml, "replace" );
-		gtk_widget_set_sensitive( widget, FALSE );
+	if( reset ) {
+		search->priv->found_at = -1;
+	
+		if( search->priv->dialog ) {
+			xml = glade_get_widget_tree( search->priv->dialog );
+			widget = glade_xml_get_widget( xml, "replace" );
+			gtk_widget_set_sensitive( widget, FALSE );
+		}
 	}
 }
 
@@ -746,7 +807,6 @@ gchar *screem_search_get_replace_pattern( const ScreemSearch *search )
 void screem_search_show_dialog( ScreemSearch *search, 
 				gboolean doreplace )
 {
-	gchar *glade_path;
 	GladeXML *xml;
 
 	GtkWidget *widget;
@@ -762,9 +822,8 @@ void screem_search_show_dialog( ScreemSearch *search,
 	g_return_if_fail( SCREEM_IS_WINDOW( search->priv->window ) );
 	
 	if( ! search->priv->dialog ) {
-		glade_path = screem_get_glade_path();
-		xml = glade_xml_new( glade_path, "find_dialog", NULL );
-		g_free( glade_path );
+		xml = glade_xml_new( GLADE_PATH"/screem.glade",
+				"find_dialog", NULL );
 	
 		search->priv->dialog = glade_xml_get_widget( xml,
 				"find_dialog" );
@@ -805,15 +864,21 @@ void screem_search_show_dialog( ScreemSearch *search,
 		
 		/* setup field change handlers */
 		widget = glade_xml_get_widget( xml, "find_entry" );
-		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+		screem_gtk_add_history( widget );
+		widget = GTK_BIN( widget )->child;
+		g_signal_connect( G_OBJECT( widget ), "activate",
+				G_CALLBACK( screem_search_find_activate ),
+				search );
+		gtk_entry_set_activates_default( GTK_ENTRY( widget ), TRUE );
 		g_signal_connect( G_OBJECT( widget ),
-				  "changed",
+				  "focus_out_event",
 				  G_CALLBACK( screem_search_find_change ),
 				  search );
 		widget = glade_xml_get_widget( xml, "replace_entry" );
-		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+		screem_gtk_add_history( widget );
+		widget = GTK_BIN( widget )->child;
 		g_signal_connect( G_OBJECT( widget ),
-				  "changed",
+				  "focus_out_event",
 				  G_CALLBACK( screem_search_replace_change ),
 				  search );
 		widget = glade_xml_get_widget( xml, "regex" );
@@ -863,8 +928,8 @@ void screem_search_show_dialog( ScreemSearch *search,
 
 	/* find pattern */
 	widget = glade_xml_get_widget( xml, "find_entry" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
-	if( screem_editor_has_selection( search->priv->editor,
+	widget = GTK_BIN( widget )->child;
+	if( screem_view_has_selection( SCREEM_VIEW( search->priv->editor ),
 				&start, &end ) ) {
 		page = screem_window_get_document( search->priv->window );
 		gtk_text_buffer_get_iter_at_offset( GTK_TEXT_BUFFER( page ), &it, start );
@@ -893,7 +958,7 @@ void screem_search_show_dialog( ScreemSearch *search,
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
 			( pattern != NULL ) || doreplace );
 	widget = glade_xml_get_widget( xml, "replace_entry" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	if( pattern ) {
 		gtk_entry_set_text( GTK_ENTRY( widget ), pattern );
 		g_free( pattern );
@@ -954,7 +1019,7 @@ void screem_search_show_dialog( ScreemSearch *search,
 	gtk_widget_show( search->priv->dialog );
 
 	widget = glade_xml_get_widget( xml, "find_entry" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	gtk_widget_grab_focus( widget );
 }
 
@@ -963,7 +1028,7 @@ void screem_search_do_find( ScreemSearch *search )
 	ScreemSearchPrivate *priv;
 	ScreemWindow *window;
 	ScreemApplication *app;
-	ScreemEditor *editor;
+	ScreemView *editor;
 	ScreemSite *site;
 	const GList *pages;
 	guint pos;
@@ -988,7 +1053,8 @@ void screem_search_do_find( ScreemSearch *search )
 			pages = NULL;
 			break;
 		case SCREEM_SEARCH_SITE:
-			pages = screem_site_get_pages( site );
+			pages = screem_site_get_pages( site,
+					TRUE, TRUE );
 			break;
 		case SCREEM_SEARCH_OPEN:
 			pages = screem_window_get_documents( window );
@@ -1016,7 +1082,7 @@ void screem_search_do_find( ScreemSearch *search )
 			pos = 0;
 			break;
 		case SCREEM_SEARCH_CURRENT:
-			pos = screem_editor_get_pos( editor );
+			pos = screem_view_get_pos( SCREEM_VIEW( editor ) );
 			break;
 		default:
 			pos = 0;
@@ -1036,16 +1102,22 @@ void screem_search_do_find( ScreemSearch *search )
 	}
 
 	screem_application_set_cursor( app, GDK_WATCH );
+	/* we may want submatches here */
+	if( priv->pmatch ) {
+		g_free( priv->pmatch );
+		priv->pmatch = NULL;
+	}
 	priv->found_at = screem_search_static( text, 
 			priv->find_pattern, 
 			priv->regexp, 
-			pos, &priv->length );
+			pos, &priv->length, 
+			&priv->nmatch, &priv->pmatch );
 	priv->found_c = g_utf8_pointer_to_offset( text, 
 			text + priv->found_at );
 	priv->length_c = g_utf8_pointer_to_offset( text,
 			text + priv->found_at + priv->length ) - 
 		priv->found_c;
-	
+
 	screem_application_set_cursor( app, GDK_LAST_CURSOR );
 
 	widget = NULL;
@@ -1077,11 +1149,20 @@ void screem_search_do_find( ScreemSearch *search )
 void screem_search_do_replace( ScreemSearch *search )
 {
 	ScreemSearchPrivate *priv;
-	ScreemEditor *editor;
+	ScreemView *editor;
+	gchar *match_text;
+	GString *replace;
+	const gchar *rstring;
+	gunichar c;
+
+	regmatch_t *pmatch;
+	guint sub;
+	guint offset;
+	guint len;
 	
 	g_return_if_fail( SCREEM_IS_SEARCH( search ) );
 	g_return_if_fail( search->priv->found_at != -1 );
-	g_return_if_fail( SCREEM_IS_EDITOR( search->priv->editor ) );
+	g_return_if_fail( search->priv->editor != NULL );
 	
 	priv = search->priv;
 
@@ -1091,18 +1172,70 @@ void screem_search_do_replace( ScreemSearch *search )
 		screem_search_set_replace_pattern( search, "" );
 	}
 
-	screem_editor_delete_forward( editor, priv->found_c,
-			priv->length_c );
-	screem_editor_insert( editor, priv->found_c,
-			priv->replace_pattern );
-	priv->found_at += strlen( priv->replace_pattern );
+	g_return_if_fail( ( ! priv->regexp ) || ( priv->nmatch > 0 ) );
+
+	pmatch = priv->pmatch;
 	
-	priv->found_c += g_utf8_strlen( priv->replace_pattern, -1 );
+	match_text = screem_view_get_text( SCREEM_VIEW( editor ),
+		priv->found_c, priv->length_c );
+	
+	replace = g_string_new( "" );
+	rstring = priv->replace_pattern;
+	while( *rstring != '\0' ) {
+		c = g_utf8_get_char( rstring );
+
+		if( c != '\\' ) {
+			g_string_append_unichar( replace, c );
+		} else {
+			rstring = g_utf8_next_char( rstring );
+			c = g_utf8_get_char( rstring );
+			if( c == 'n' ) {
+				g_string_append_c( replace, '\n' );
+			} else if( c == 'r' ) {
+				g_string_append_c( replace, '\r' );
+			} else if( c == 't' ) {
+				g_string_append_c( replace, '\t' );
+			} else if( c == '\0' ) {
+				break;
+			} else if( priv->regexp && 
+					c >= '1' && c <= '9' ) {
+				/* insert a submatch if it exists */
+				sub = ( c - '0' );
+				if( sub < priv->nmatch ) {
+					offset = pmatch[ sub ].rm_so -
+						pmatch[ 0 ].rm_so;
+					
+					len = pmatch[ sub ].rm_eo -
+						pmatch[ sub ].rm_so;
+
+					g_string_append_len( replace,
+							match_text + 
+							offset, len );
+				}
+			} else {
+				g_string_append_unichar( replace, c );
+			}
+		}
+		rstring = g_utf8_next_char( rstring );
+	}
+		
+	screem_view_delete_forward( SCREEM_VIEW( editor ), 
+			priv->found_c, priv->length_c );
+
+	screem_view_insert( SCREEM_VIEW( editor ), priv->found_c,
+			replace->str );
+
+	priv->found_at += replace->len;
+
+	priv->found_c += g_utf8_strlen( replace->str, -1 );
 	while( gtk_events_pending() ) {
 		gtk_main_iteration();
 	}
-	
-	screem_editor_set_pos( editor, priv->found_c );
+
+	g_string_free( replace, TRUE );
+	g_free( match_text );
+
+	screem_view_set_pos( SCREEM_VIEW( editor ), priv->found_c );
 	screem_search_do_find( search );
 }
 
