@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -33,23 +33,13 @@
 #include "screem-site-ui.h"
 #include "screem-window.h"
 
-
 #include "fileops.h"
 
 #include <string.h>
-#include <libgnome/gnome-macros.h>
 #include <libgnomeui/gnome-client.h>
 
 #include <libxml/tree.h>
 
-static void screem_session_class_init( ScreemSessionClass *klass );
-static void screem_session_instance_init( ScreemSession *session );
-static void screem_session_finalize( GObject *object );
-static void screem_session_set_prop( GObject *object, guint prop_id,
-				const GValue *value, GParamSpec *spec );
-static void screem_session_get_prop( GObject *object, guint prop_id,
-				GValue *value, GParamSpec *spec );
-
 static gchar *get_session_filename( const gchar *filename );
 static gint
 save_yourself( GnomeClient *client, gint phase, GnomeSaveStyle save_style,
@@ -67,13 +57,22 @@ struct ScreemSessionPrivate {
 
 	GString *output;
 
-	/* hash table of classes and pos / dimensions */
+	/* hash table of classes and pos / dimensions 
+	 * (name / GdkRectangle)*/
 	GHashTable *dialogs;
+
+	/* site file browser expansion status 
+	 * (name / GHashTable (path/1) ) */
+	GHashTable *site_expansion;
 };
 
-GNOME_CLASS_BOILERPLATE( ScreemSession, screem_session,
-			  GObject,
-			  G_TYPE_OBJECT )
+G_DEFINE_TYPE( ScreemSession, screem_session, G_TYPE_OBJECT )
+
+static void screem_session_finalize( GObject *object );
+static void screem_session_set_prop( GObject *object, guint prop_id,
+				const GValue *value, GParamSpec *spec );
+static void screem_session_get_prop( GObject *object, guint prop_id,
+				GValue *value, GParamSpec *spec );
 
 static void screem_session_class_init( ScreemSessionClass *klass )
 {
@@ -93,7 +92,7 @@ static void screem_session_class_init( ScreemSessionClass *klass )
 	g_object_class_install_property( obj_class, PROP_APP, spec );
 }
 	
-static void screem_session_instance_init( ScreemSession *session )
+static void screem_session_init( ScreemSession *session )
 {
 	ScreemSessionPrivate *priv;
 	GnomeClient *client;
@@ -109,13 +108,17 @@ static void screem_session_instance_init( ScreemSession *session )
 	priv->dialogs = g_hash_table_new_full( g_str_hash, g_str_equal,
 			(GDestroyNotify)g_free,
 			(GDestroyNotify)g_free );
+
+	priv->site_expansion = g_hash_table_new_full( g_str_hash, g_str_equal,
+			(GDestroyNotify)g_free,
+			(GDestroyNotify)g_hash_table_destroy );
 }
 
 static void screem_session_finalize( GObject *object )
 {
 	ScreemSession *session;
 	ScreemSessionPrivate *priv;
-	
+			
 	g_return_if_fail( object != NULL );
 	g_return_if_fail( SCREEM_IS_SESSION( object ) );
 
@@ -124,10 +127,11 @@ static void screem_session_finalize( GObject *object )
 	priv = session->priv;
 	
 	g_hash_table_destroy( priv->dialogs );
-	
+	g_hash_table_destroy( priv->site_expansion );
+
 	g_free( priv );
 	
-	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
+	G_OBJECT_CLASS( screem_session_parent_class )->finalize( object );
 }
 
 static void screem_session_set_prop( GObject *object, guint prop_id,
@@ -186,7 +190,7 @@ static gchar *get_session_filename( const gchar *filename )
 		ret = g_build_filename( tmp, "session_gnome-XXXXXX", NULL );
 		g_free( tmp );
 		
-		tmp = screem_create_tmp_file( ret, "xml", FALSE );
+		tmp = screem_create_tmp_file( ret, "xml", "" );
 		g_free( ret );
 		ret = tmp;
 	} else {
@@ -217,6 +221,19 @@ save_yourself( GnomeClient *client, gint phase, GnomeSaveStyle save_style,
         return TRUE;
 }
 
+static gboolean save_expansion( gpointer key, gpointer value,
+		gpointer data )
+{
+	GString *output;
+
+	output = (GString*)data;
+	
+	g_string_append_printf( output, "<uri>%s</uri>",
+			(const gchar*)key );
+	
+	return FALSE;
+}
+
 static void screem_session_save_site( ScreemSite *site, ScreemSession *session )
 {
 	ScreemApplication *app;
@@ -225,6 +242,8 @@ static void screem_session_save_site( ScreemSite *site, ScreemSession *session )
 	GString *output;
 	const gchar *pathname;
 	gchar *etmp;
+
+	GHashTable *expansion;
 	
 	g_return_if_fail( SCREEM_IS_SESSION( session ) );
 	g_return_if_fail( SCREEM_IS_SITE( site ) );
@@ -274,6 +293,16 @@ static void screem_session_save_site( ScreemSite *site, ScreemSession *session )
 		}
 		g_string_append_printf( output, "</pages>" );
 	}
+
+	expansion = g_hash_table_lookup( session->priv->site_expansion,
+			screem_site_get_name( site ) );
+	if( expansion && g_hash_table_size( expansion ) ) {
+		g_string_append( output, "<expanded>" );
+		g_hash_table_foreach( expansion,
+				(GHFunc)save_expansion,
+				output );
+		g_string_append( output, "</expanded>" );
+	}
 	
 	g_string_append( output, "</site>" );
 	g_object_unref( app );
@@ -285,37 +314,49 @@ static void screem_session_load_site( ScreemSession *session,
 {
 	ScreemApplication *app;
 	gboolean fake;
+	xmlNodePtr child;
+	xmlChar *url;
+	const gchar *sname;
 	
 	g_object_get( G_OBJECT( session ), "app", &app, NULL );
 	
 	fake = screem_site_get_fake_flag( site );
+	sname = screem_site_get_name( site );
 	
 	while( node ) {
-		if( ! strcmp( "pages", node->name ) ) {
+		if( ! strcmp( "expanded", (gchar*)node->name ) ) {
+			/* nodes in the file browser to be
+			 * expanded for this site */
+			for( child = node->children; child;
+					child = child->next ) {
+				if( ! strcmp( "uri", (gchar*)child->name ) ) {
+					url = xmlNodeGetContent( child );
+					screem_session_site_set_expanded( session, sname, (const gchar *)url, TRUE );
+					xmlFree( url );
+				} 
+			}
+		} else if( ! strcmp( "pages", (gchar*)node->name ) ) {
 			/* pages to be opened in window name="" */
 			xmlChar *name;
 
-			name = xmlGetProp( node, "name" );
+			name = xmlGetProp( node, (const xmlChar*)"name" );
 			if( name && *name != '\0' ) {
-				xmlNodePtr child;
 				GList *docs;
 				ScreemPage *page;
 				
 				child = node->children;
 				for( docs = NULL; child; child = child->next ) {
-					if( ! strcmp( "page", child->name ) ) {
-						xmlChar *url;
-
-						url = xmlGetProp( child, "url" );
+					if( ! strcmp( "page", (gchar*)child->name ) ) {
+						url = xmlGetProp( child, (const xmlChar*)"url" );
 						if( url ) {
-							page = screem_site_locate_page( site, url );
+							page = screem_site_locate_page( site, (gchar*)url );
 							if( *url == '\0' && ! page ) {
 								page = screem_page_new( G_OBJECT( app ) );
 								screem_site_add_page( site, page );
 								g_object_unref( page );
 							} else if( fake && ! page ) {
 								page = screem_page_new( G_OBJECT( app ) );
-								screem_page_set_pathname( page, url );
+								screem_page_set_pathname( page, (gchar*)url );
 								screem_site_add_page( site, page );
 								g_object_unref( page );
 							}
@@ -330,7 +371,8 @@ static void screem_session_load_site( ScreemSession *session,
 				}
 				docs = g_list_reverse( docs );
 				/* FIXME: name gets leaked */
-				screem_site_set_documents( site, name, docs );
+				screem_site_set_documents( site, 
+						(gchar*)name, docs );
 			} else if( name ) {
 				xmlFree( name );
 			}
@@ -410,14 +452,14 @@ static void screem_session_load_window( ScreemSession *session,
 	dsite = screem_application_get_default_site( app );
 
 	while( node ) {
-		if( ! strcmp( "site", node->name ) ) {
+		if( ! strcmp( "site", (gchar*)node->name ) ) {
 			xmlChar *tmp;
 
-			tmp = xmlGetProp( node, "name" );
+			tmp = xmlGetProp( node, (const xmlChar*)"name" );
 			if( ( ! tmp ) || *tmp == '\0' ) {
 				site = dsite;
 			} else {
-				site = screem_application_get_site_by_name( app, tmp );
+				site = screem_application_get_site_by_name( app, (gchar*)tmp );
 			}
 			if( tmp ) {
 				xmlFree( tmp );
@@ -474,11 +516,11 @@ static void screem_session_load_hint( ScreemSession *session,
 	gint x;
 	gint y;
 				
-	tmp = xmlGetProp( node, "x" );
-	x = atoi( tmp );
+	tmp = xmlGetProp( node, (const xmlChar*)"x" );
+	x = atoi( (gchar*)tmp );
 	xmlFree( tmp );
-	tmp = xmlGetProp( node, "y" );
-	y = atoi( tmp );
+	tmp = xmlGetProp( node, (const xmlChar*)"y" );
+	y = atoi( (gchar*)tmp );
 	xmlFree( tmp );
 
 	g_object_get( G_OBJECT( session ), "app", &app, NULL );
@@ -533,18 +575,18 @@ static void screem_session_load_dialog( ScreemSession *session,
 	
 	priv = session->priv;
 	
-	name = xmlGetProp( node, "name" );
-	tmp = xmlGetProp( node, "x" );
-	x = atoi( tmp );
+	name = (gchar*)xmlGetProp( node, (const xmlChar*)"name" );
+	tmp = xmlGetProp( node, (const xmlChar*)"x" );
+	x = atoi( (gchar*)tmp );
 	xmlFree( tmp );
-	tmp = xmlGetProp( node, "y" );
-	y = atoi( tmp );
+	tmp = xmlGetProp( node, (const xmlChar*)"y" );
+	y = atoi( (gchar*)tmp );
 	xmlFree( tmp );
-	tmp = xmlGetProp( node, "w" );
-	w = atoi( tmp );
+	tmp = xmlGetProp( node, (const xmlChar*)"w" );
+	w = atoi( (gchar*)tmp );
 	xmlFree( tmp );
-	tmp = xmlGetProp( node, "h" );
-	h = atoi( tmp );
+	tmp = xmlGetProp( node, (const xmlChar*)"h" );
+	h = atoi( (gchar*)tmp );
 	xmlFree( tmp );
 
 	rect = g_new0( GdkRectangle, 1 );
@@ -587,23 +629,23 @@ void screem_session_load( ScreemSession *session,
 
 		node = xmlDocGetRootElement( doc );
 		/* make sure we are dealing with a screem session file */
-		if( strcmp( "screemsession", node->name ) ) {
+		if( strcmp( "screemsession", (gchar*)node->name ) ) {
 			node = NULL;
 		} else {
 			node = node->children;
 		}
 		
 		while( node ) {
-			if( ! strcmp( "site", node->name ) ) {
+			if( ! strcmp( "site", (gchar*)node->name ) ) {
 				ScreemSite *site;
 				xmlChar *tmp;
 
-				tmp = xmlGetProp( node, "url" );
+				tmp = xmlGetProp( node, (const xmlChar*)"url" );
 				if( ( ! tmp ) || *tmp == '\0' ) {
 					site = screem_application_get_default_site( app );
 					g_assert( site );
-				} else if( screem_site_open_with_filename( NULL, app, tmp ) ) {
-					site = screem_application_get_site( app, tmp );
+				} else if( screem_site_open_with_filename( NULL, app, (gchar*)tmp ) ) {
+					site = screem_application_get_site( app, (gchar*)tmp );
 				} else {
 					site = NULL;
 				}
@@ -614,7 +656,7 @@ void screem_session_load( ScreemSession *session,
 					screem_session_load_site( session, site,
 								  node->children );
 				}
-			} else if( ! strcmp( "window", node->name ) ) {
+			} else if( ! strcmp( "window", (gchar*)node->name ) ) {
 				ScreemWindow *win;
 				xmlChar *tmp;
 				gint x;
@@ -623,24 +665,24 @@ void screem_session_load( ScreemSession *session,
 				gint h;
 				gint max;
 				
-				tmp = xmlGetProp( node, "x" );
-				x = atoi( tmp );
+				tmp = xmlGetProp( node, (const xmlChar*)"x" );
+				x = atoi( (gchar*)tmp );
 				xmlFree( tmp );
-				tmp = xmlGetProp( node, "y" );
-				y = atoi( tmp );
+				tmp = xmlGetProp( node, (const xmlChar*)"y" );
+				y = atoi( (gchar*)tmp );
 				xmlFree( tmp );
-				tmp = xmlGetProp( node, "w" );
-				w = atoi( tmp );
+				tmp = xmlGetProp( node, (const xmlChar*)"w" );
+				w = atoi( (gchar*)tmp );
 				xmlFree( tmp );
-				tmp = xmlGetProp( node, "h" );
-				h = atoi( tmp );
+				tmp = xmlGetProp( node, (const xmlChar*)"h" );
+				h = atoi( (gchar*)tmp );
 				xmlFree( tmp );
-				tmp = xmlGetProp( node, "max" );
+				tmp = xmlGetProp( node, (const xmlChar*)"max" );
 				max = 0;
 				if( tmp ) {
-					max = atoi( tmp );
+					max = atoi( (gchar*)tmp );
+					xmlFree( tmp );
 				}
-				xmlFree( tmp );
 
 				win = screem_application_create_window( app );
 				win->x = x;
@@ -661,9 +703,9 @@ void screem_session_load( ScreemSession *session,
 				if( max ) {
 					gtk_window_maximize( GTK_WINDOW( win ) );
 				}
-			} else if( ! strcmp( "hints", node->name ) ) {
+			} else if( ! strcmp( "hints", (gchar*)node->name ) ) {
 				screem_session_load_hint( session, node );
-			} else if( ! strcmp( "dialog", node->name ) ) {
+			} else if( ! strcmp( "dialog", (gchar*)node->name ) ) {
 				screem_session_load_dialog( session,
 						node );
 			}
@@ -682,13 +724,14 @@ gboolean screem_session_save( ScreemSession *session, const gchar *filename )
 	gboolean ret;
 	GList *list;
 	gchar *file;	
+	GError *error;
 	
 	g_return_val_if_fail( SCREEM_IS_SESSION( session ), FALSE );
 
 	priv = session->priv;
 
-	/* ensure the the current site has a record of which pages are open
-	 * in each window */
+	/* ensure the the current site has a record of which pages
+	 * are open in each window */
 	list = screem_application_get_window_list( priv->app );
 	while( list ) {
 		ScreemSite *site;
@@ -738,8 +781,13 @@ gboolean screem_session_save( ScreemSession *session, const gchar *filename )
 	ret = save_file( file, priv->output->str, 
 			 GNOME_VFS_PERM_USER_READ |
 			 GNOME_VFS_PERM_USER_WRITE,
-			 FALSE,
-			 NULL );
+			 FALSE, FALSE,
+			 &error );
+	if( error ) {
+		g_printerr( "Failed to save session: %s\n",
+				error->message );
+		g_error_free( error );
+	}
 	g_free( file );
 	
 	g_string_free( priv->output, TRUE );
@@ -843,3 +891,63 @@ void screem_session_store_dialog( ScreemSession *session,
 	}
 }
 
+void screem_session_site_set_expanded( ScreemSession *session,
+		const gchar *name, const gchar *uri,
+		gboolean status )
+{
+	ScreemSessionPrivate *priv;
+	GHashTable *expansion;
+	
+	g_return_if_fail( SCREEM_IS_SESSION( session ) );
+	g_return_if_fail( name != NULL );
+	g_return_if_fail( uri != NULL );
+	
+	priv = session->priv;
+
+	expansion = g_hash_table_lookup( priv->site_expansion, name );
+	if( ! expansion ) {
+		expansion = g_hash_table_new_full( g_str_hash, 
+				g_str_equal,
+				(GDestroyNotify)g_free, NULL );
+		g_hash_table_insert( priv->site_expansion,
+				g_strdup( name ), expansion );
+	}
+
+	g_return_if_fail( expansion != NULL );
+
+	if( g_hash_table_lookup( expansion, uri ) ) {
+		g_return_if_fail( status != TRUE );
+	
+		g_hash_table_remove( expansion, uri );
+		
+	} else {
+		g_return_if_fail( status != FALSE );
+
+		g_hash_table_insert( expansion, g_strdup( uri ), 
+				GINT_TO_POINTER( 1 ) );
+	}
+}
+
+gboolean screem_session_site_get_expanded( ScreemSession *session,
+		const gchar *name, const gchar *uri )
+{
+	ScreemSessionPrivate *priv;
+	gboolean ret;
+	GHashTable *expansion;
+	
+	g_return_val_if_fail( SCREEM_IS_SESSION( session ), FALSE );
+	g_return_val_if_fail( name != NULL, FALSE );
+	g_return_val_if_fail( uri != NULL, FALSE );
+	
+	priv = session->priv;
+	
+	expansion = g_hash_table_lookup( priv->site_expansion, name );
+	
+	ret = FALSE;
+	if( expansion ) {
+		ret = GPOINTER_TO_INT( g_hash_table_lookup( expansion, uri ) );
+	}
+	
+	return ret;
+}
+
