@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -28,17 +28,11 @@
 #include <unistd.h>
 #include <sys/types.h>
 
-#include <gtk/gtkentry.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 
 #include <glib/gi18n.h>
 
-#include <libgnomeui/libgnomeui.h>
+#include <libgnomeui/gnome-druid.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <glade/glade.h>
 
@@ -61,15 +55,14 @@ void screem_site_druid_begin( ScreemApplication *application,
 			      ScreemWindow *window )
 {
 	GladeXML *xml;
-	gchar *glade_path;
 	GtkWidget *druid;
 	ScreemSite *site;
 	ScreemSession *session;
 	
 	session = screem_application_get_session( application );
 
-	glade_path = screem_get_glade_path();	
-	xml = glade_xml_new( glade_path, "new_site_druid", NULL );
+	xml = glade_xml_new( GLADE_PATH"/screem.glade",
+			"new_site_druid", NULL );
 
 	druid = glade_xml_get_widget( xml, "new_site_druid" );
 	site = screem_site_new( G_OBJECT( application ) );
@@ -182,13 +175,21 @@ void screem_site_druid_details_prepare( GnomeDruidPage *druid_page,
 {
 	GladeXML *xml;
 	GtkWidget *widget;
+	gboolean active;
 
 	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
 
 	widget = glade_xml_get_widget( xml, "druid1" );
 	gnome_druid_set_buttons_sensitive( GNOME_DRUID( widget ),
 			TRUE, FALSE, TRUE, TRUE );
+	
+	widget = glade_xml_get_widget( xml, "sitename" );
+	screem_gtk_add_history( widget );
 
+	widget = glade_xml_get_widget( xml, "usesitetemplate" );
+	active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+	widget = glade_xml_get_widget( xml, "sitetemplate" );
+	gtk_widget_set_sensitive( widget, active );
 }
 
 void screem_site_druid_details_validate( GtkWidget *widget )
@@ -196,18 +197,19 @@ void screem_site_druid_details_validate( GtkWidget *widget )
 	GladeXML *xml;
 	const gchar *value;
 	gboolean valid;
+	gchar *uri;
 	
 	xml = glade_get_widget_tree( widget );
 	
 	widget = glade_xml_get_widget( xml, "sitename" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	value = gtk_entry_get_text( GTK_ENTRY( widget ) );
 	valid = ( value && *value != '\0' );
 
 	widget = glade_xml_get_widget( xml, "sitepath" );
-	widget = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
-	value = gtk_entry_get_text( GTK_ENTRY( widget ) );
-	valid = valid && ( value && *value != '\0' );
+	uri = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( widget ) );
+	valid = valid && ( uri && *uri != '\0' );
+	g_free( uri );
 	
 	widget = glade_xml_get_widget( xml, "druid1" );
 	gnome_druid_set_buttons_sensitive( GNOME_DRUID( widget ),
@@ -226,6 +228,27 @@ gboolean screem_site_druid_details_next( GnomeDruidPage *druid_page,
 	return ret;
 }
 
+void screem_site_druid_management_prepare( GnomeDruidPage *druid_page,
+		GtkWidget *druid )
+{
+	GladeXML *xml;
+	GtkWidget *widget;
+	gboolean active;
+
+	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
+
+	widget = glade_xml_get_widget( xml, "usesitecvs" );
+	active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+	widget = glade_xml_get_widget( xml, "sitecvs" );
+	gtk_widget_set_sensitive( widget, active );
+	screem_gtk_add_history( widget );
+
+	widget = glade_xml_get_widget( xml, "usepagetemplate" );
+	active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+	widget = glade_xml_get_widget( xml, "sitepagetemplate" );
+	gtk_widget_set_sensitive( widget, active );
+}
+
 gboolean screem_site_druid_management_next( GnomeDruidPage *druid_page,
 		GtkWidget *druid )
 {
@@ -246,8 +269,15 @@ void screem_site_druid_publishing_prepare( GnomeDruidPage *druid_page,
 
 	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
 
+	widget = glade_xml_get_widget( xml, "remote_location" );
+	screem_gtk_add_history( widget );
+	
+	widget = glade_xml_get_widget( xml, "baseuri" );
+	screem_gtk_add_history( widget );
+	
 	widget = glade_xml_get_widget( xml, "username" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	screem_gtk_add_history( widget );
+	widget = GTK_BIN( widget )->child;
 	gtk_entry_set_text( GTK_ENTRY( widget ), g_get_user_name() );
 
 	widget = glade_xml_get_widget( xml, "remote_method" );
@@ -286,32 +316,35 @@ static void screem_site_druid_create_site( ScreemSite *site, GladeXML *xml )
 	ScreemSitePerms sp;
 	ScreemSiteSymlinks sl;
 	GnomeVFSURI *uri;
+	gchar *path;
+	gchar *tmp;
 	
 	/* process details page */
 	widget = glade_xml_get_widget( xml, "sitename" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	value = gtk_entry_get_text( GTK_ENTRY( widget ) );
 	screem_site_set_name( site, value );
 
 	widget = glade_xml_get_widget( xml, "sitepath" );
-	widget = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
-	value = gtk_entry_get_text( GTK_ENTRY( widget ) );
-	value = gtk_entry_get_text( GTK_ENTRY( widget ) );
-	screem_site_set_pathname( site, value );
+	path = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( widget ) );
+	tmp = g_build_path( G_DIR_SEPARATOR_S, path, value, NULL );
+	screem_site_set_pathname( site, tmp );
+	g_free( tmp );
+	g_free( path );
 
 	widget = glade_xml_get_widget( xml, "usesitetemplate" );
 	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 		widget = glade_xml_get_widget( xml, "sitetemplate" );
-		widget = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(widget));
-		value = gtk_entry_get_text( GTK_ENTRY( widget ) );
-		screem_site_set_site_template( site, value );
+		path = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( widget ) );
+		screem_site_set_site_template( site, path );
+		g_free( path );
 	}
 
 	widget = glade_xml_get_widget( xml, "usesitecvs" );
 	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 		widget = glade_xml_get_widget( xml, "sitecvs" );
-		widget = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(widget));
-		value = gtk_entry_get_text( GTK_ENTRY( widget ) );
+		widget = GTK_BIN( widget )->child;
+		value  = gtk_entry_get_text( GTK_ENTRY( widget ) );
 		screem_site_set_cvs_root( site, value );
 	}
 
@@ -328,9 +361,9 @@ static void screem_site_druid_create_site( ScreemSite *site, GladeXML *xml )
 	widget = glade_xml_get_widget( xml, "usepagetemplate" );
 	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 		widget = glade_xml_get_widget( xml, "sitepagetemplate" );
-		widget = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(widget));
-		value = gtk_entry_get_text( GTK_ENTRY( widget ) );
-		screem_site_set_template_path( site, value );
+		path = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( widget ) );
+		screem_site_set_template_path( site, path );
+		g_free( path );
 	}
 
 	/* process publish settings 1 page */
@@ -339,7 +372,7 @@ static void screem_site_druid_create_site( ScreemSite *site, GladeXML *xml )
 			gtk_combo_box_get_active( GTK_COMBO_BOX( widget ) ) );
 
 	widget = glade_xml_get_widget( xml, "remote_location" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	value = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	uri = gnome_vfs_uri_new( value );
@@ -351,12 +384,12 @@ static void screem_site_druid_create_site( ScreemSite *site, GladeXML *xml )
 		screem_site_set_remote_path( site, value );
 	}	
 	widget = glade_xml_get_widget( xml, "baseuri" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	value = gtk_entry_get_text( GTK_ENTRY( widget ) );
 	screem_site_set_http_url( site, value );
 
 	widget = glade_xml_get_widget( xml, "username" );
-	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
 	value = gtk_entry_get_text( GTK_ENTRY( widget ) );
 	screem_site_set_remote_user( site, value );
 
@@ -365,10 +398,6 @@ static void screem_site_druid_create_site( ScreemSite *site, GladeXML *xml )
 	screem_site_set_remote_pass( site, value );
 
 	/* process publish settings 2 page */
-	widget = glade_xml_get_widget( xml, "passiveftp" );
-	flag = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
-	screem_site_set_passive_ftp( site, flag );
-
 	widget = glade_xml_get_widget( xml, "nodelete" );
 	flag = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
 	screem_site_set_no_delete( site, flag );
@@ -463,7 +492,7 @@ static gboolean screem_site_druid_create_initial_page( ScreemWindow *window )
 		if( error ) {
 			secondary = error->message;
 		}
-		screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+		screem_hig_alert( GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
 				primary, secondary,
 				GTK_WIDGET( window ) );
 		if( error ) {
