@@ -649,13 +649,14 @@ static GtkTreeIter *screem_site_model_add_virtual_root( GtkTreeModel *model,
 
 static void screem_site_model_add_virtual( GtkTreeModel *model,
 		GtkTreeIter *it, GtkTreeIter *parent,
-		const gchar *uri, const gchar *mime_type,
+		const gchar *uri,
 		ScreemSiteViewNodeInfo *info )
 {
 	GtkTreeIter iiter;
 	gchar *base;
 	GdkPixbuf *pixbuf;
 	gboolean visible;
+	const gchar *mime_type;
 		
 	gtk_tree_store_append( GTK_TREE_STORE( model ), &iiter, 
 			parent );
@@ -663,6 +664,7 @@ static void screem_site_model_add_virtual( GtkTreeModel *model,
 	gtk_tree_model_get( model, it,
 			FILE_BROWSER_ICON_COL, &pixbuf, 
 			FILE_BROWSER_SHOW_COL, &visible,
+			FILE_BROWSER_MIME_COL, &mime_type,
 			-1 );
 
 	gtk_tree_store_set( GTK_TREE_STORE( model ), &iiter,
@@ -676,6 +678,9 @@ static void screem_site_model_add_virtual( GtkTreeModel *model,
 	screem_site_model_set_val_at( GTK_TREE_STORE( model),
 			&iiter, info );
 
+	if( pixbuf ) {
+		g_object_unref( pixbuf );
+	}
 }
 
 void screem_site_model_added( ScreemFileBrowser *browser, 
@@ -689,7 +694,6 @@ void screem_site_model_added( ScreemFileBrowser *browser,
 	ScreemSiteViewNodeInfo *info;
 	GtkTreeModel *model;
 	gboolean is_dir;
-	gchar *tmp;
 
 	site = SCREEM_SITE( data );
 
@@ -697,6 +701,8 @@ void screem_site_model_added( ScreemFileBrowser *browser,
 		return;
 	}
 
+	g_assert( mime_type );
+	
 	g_object_get( G_OBJECT( site ), "app", &app, NULL );
 	cache = screem_application_get_icon_cache( app );
 	
@@ -704,25 +710,15 @@ void screem_site_model_added( ScreemFileBrowser *browser,
 
 	model = screem_site_get_model( site, TRUE );
 	
-	is_dir = FALSE;
-	if( mime_type ) {
-		is_dir = ! strcmp( "x-directory/normal", mime_type );
-	} else {
-		/* mime type hack */
-		mime_type = "application/octet-stream";
-	}
+	is_dir = ! strcmp( "x-directory/normal", mime_type );
 	
-	tmp = NULL;
 	if( ! is_dir ) {
 		/* if mime type is application/octet-stream
 		 * do a slow mime type check, we don't want
 		 * to miss any pages
 		 */
 		if( ! strcmp( "application/octet-stream", mime_type ) ){
-			tmp = screem_get_mime_type( uri, FALSE );
-			if( tmp ) {
-				mime_type = tmp;
-			}
+			mime_type = screem_get_mime_type( uri, FALSE );
 		}
 		
 		info = screem_site_model_node_info_new( SCREEM_SITE_VIEW_FILE,
@@ -743,23 +739,20 @@ void screem_site_model_added( ScreemFileBrowser *browser,
 			}
 			g_object_unref( page );
 		}
-
-		if( ! strncmp( "image/", mime_type, strlen( "image/" ) ) ) {
+		if( g_str_has_prefix( mime_type, "image/" ) ) {
 			/* add to images branch as well */
 			if( ! priv->images ) {
 				priv->images = screem_site_model_add_virtual_root( model, _( "Images" ), cache );
 			}
 			screem_site_model_add_virtual( model, it,
-					priv->images,
-					uri, mime_type, info );
+					priv->images, uri, info );
 		} else if( ! strcmp( "text/css", mime_type ) ) {
 			/* add to stylesheets branch as well */
 			if( ! priv->styles ) {
 				priv->styles = screem_site_model_add_virtual_root( model, _( "Stylesheets" ), cache );
 			}
 			screem_site_model_add_virtual( model, it,
-					priv->styles,
-					uri, mime_type, info );
+					priv->styles, uri, info );
 		} else if( ! strcmp( "text/javascript", mime_type ) ||
 			   ! strcmp( "text/x-javascript", mime_type ) ) {
 			/* add to scripts branch as well */
@@ -767,8 +760,7 @@ void screem_site_model_added( ScreemFileBrowser *browser,
 				priv->scripts = screem_site_model_add_virtual_root( model, _( "Scripts" ), cache );
 			}
 			screem_site_model_add_virtual( model, it,
-					priv->scripts,
-					uri, mime_type, info );
+					priv->scripts, uri, info );
 		}  
 	} else {
 		info = screem_site_model_node_info_new( SCREEM_SITE_VIEW_FOLDER,
@@ -783,7 +775,6 @@ void screem_site_model_added( ScreemFileBrowser *browser,
 	}
 	g_object_unref( cache );
 	g_object_unref( app );
-	g_free( tmp );
 }
 
 void screem_site_model_icon_change( ScreemFileBrowser *browser, const gchar *uri,
