@@ -128,7 +128,7 @@ gboolean screem_site_parse_project_file( ScreemSite *site,
 	ctx = g_markup_parse_context_new( &parser_cbs, 0, 
 					  data, NULL );
 	if( ctx ) {
-		GString *file;
+		gchar *file;
 
 		/* the lock will be held as we should only
 		 * be called from screem_site_load(), which
@@ -138,10 +138,9 @@ gboolean screem_site_parse_project_file( ScreemSite *site,
 		file = load_file( filename, NULL, NULL, NULL );
 
 		if( file ) {
-			ret = g_markup_parse_context_parse( ctx, file->str,
-								file->len,
-								NULL );
-			g_string_free( file, TRUE );
+			ret = g_markup_parse_context_parse( ctx, 
+					file, strlen( file ), NULL );
+			g_free( file );
 		}
 		g_markup_parse_context_free( ctx );
 	}
@@ -369,10 +368,8 @@ static void screem_site_parse_text( GMarkupParseContext *context,
 			screem_site_add_ignore( site, text );
 			break;
 		case PROJECT_STATE_ASCII:
-			screem_site_add_ascii( site, text );
 			break;
 		case PROJECT_STATE_FILE:
-			screem_site_add_auto_open( site, text );
 			break;
 		default:
 			/* shouldn't happen */
@@ -433,7 +430,7 @@ static void screem_site_parse_preview_tag( ScreemSite *site,
 		attr = attributes[ i ];
 		val = values[ i ];
 
-		state = ( ! g_strcasecmp( "TRUE", val ) );
+		state = ( ! strcmp( "TRUE", val ) );
 		
 		if( ! strcmp( "static_publish", attr ) ) {
 			screem_site_set_preview_static( site, state );
@@ -456,11 +453,9 @@ static void screem_site_parse_options_tag( ScreemSite *site,
 		attr = attributes[ i ];
 		val = values[ i ];
 
-		state = ( ! g_strcasecmp( "TRUE", val ) );
+		state = ( ! strcmp( "TRUE", val ) );
 		
-		if( ! strcmp( "passive_ftp", attr ) ) {
-			screem_site_set_passive_ftp( site, state );
-		} else if( ! strcmp( "no_delete", attr ) ) {
+		if( ! strcmp( "no_delete", attr ) ) {
 			screem_site_set_no_delete( site, state );
 		} else if( ! strcmp( "check_moved", attr ) ) {
 			screem_site_set_check_moved( site, state );
@@ -501,7 +496,8 @@ static void screem_site_parse_ctags_tag( ScreemSite *site,
 		attr = attributes[ i ];
 		val = values[ i ];
 
-		state = ( ! g_strcasecmp( "TRUE", val ) );
+		state = ( ! strcmp( "TRUE", val ) );
+		state |= ( ! strcmp( "true", val ) );
 		
 		if( ! strcmp( "use", attr ) ) {
 			screem_site_set_use_ctags( site, state );
@@ -522,7 +518,8 @@ static void screem_site_parse_cvs_tag( ScreemSite *site,
 		attr = attributes[ i ];
 		val = values[ i ];
 
-		state = ( ! g_strcasecmp( "TRUE", val ) );
+		state = ( ! strcmp( "TRUE", val ) );
+		state |= ( ! strcmp( "true", val ) );
 		
 		if( ! strcmp( "auto_update", attr ) ) {
 			screem_site_set_auto_update( site, state );
@@ -558,7 +555,8 @@ static void screem_site_parse_task_tag( ScreemSite *site,
 		} else if( ! strcmp( "description", attr ) ) {
 			todo->description = g_strdup( val );
 		} else if( ! strcmp( "completed", attr ) ){
-			todo->complete = ( ! g_strcasecmp( "TRUE", val ) );
+			todo->complete = ( ! strcmp( "TRUE", val ) );
+			todo->complete |= ( ! strcmp( "true", val ) );
 		}
 	}
 	screem_todo_add_todo_item( SCREEM_TODO( screem_site_get_todo( site ) ),
