@@ -679,6 +679,7 @@ static void screem_site_view_site_set( ScreemSiteView *view )
 	model = screem_site_get_model( site, FALSE );
 
 	gtk_tree_view_set_model( GTK_TREE_VIEW( priv->files ), NULL );
+	gtk_tree_view_set_model( GTK_TREE_VIEW( priv->folders ), NULL );
 
 	filter = gtk_tree_model_filter_new( model, NULL );
 	gtk_tree_model_filter_set_visible_func( GTK_TREE_MODEL_FILTER( filter ),
@@ -1859,7 +1860,6 @@ gboolean screem_site_view_properties_rename( GtkWidget *widget )
 		if( page ) {
 			screem_page_set_pathname( page, newpath );
 		}
-		g_print( "PAGE RENAMED\n");
 		if( screem_uri_is_dir( fullname ) ) {
 	       		done = copy_dir( fullname, newpath, FALSE,
 					screem_site_model_file_op,
@@ -1870,7 +1870,6 @@ gboolean screem_site_view_properties_rename( GtkWidget *widget )
 					screem_site_model_file_op,
 					site->priv->model );
 		}
-		g_print( "COPIED: %i\n", done );
 		if( done ) {
 			screem_site_file_change( site,
 						 fullname,
@@ -1886,7 +1885,6 @@ gboolean screem_site_view_properties_rename( GtkWidget *widget )
 			screem_page_set_pathname( page, fullname );
 			g_free( newpath );
 		}
-		g_print( "DONE\n" );
 		g_free( dirname );
 	}
 	g_free( base );
