@@ -156,15 +156,28 @@ void screem_site_set_pathname( ScreemSite *site, const gchar *pathname )
 	GnomeVFSURI *uri;
 	gchar *tmp;
 	gsize len;
-
+	const gchar *home;
+	
 	priv = site->priv;
+
+	if( ! pathname ) {
+		home = g_get_home_dir();
+		if( ! home ) {
+			home = "/";
+		}
+		screem_file_browser_scan_directory( priv->model->browser, home, -1 );
+		return;
+	}
 	
 	/* does the site already have a pathname set? */
 	if( priv->pathname ) {
 		g_free( priv->pathname );
 	}
-
-	uri = gnome_vfs_uri_new( pathname );
+	
+	uri = NULL;
+	if( pathname ) {
+		uri = gnome_vfs_uri_new( pathname );
+	}
 	if( uri ) {
 		tmp = gnome_vfs_uri_to_string( uri, 
 				GNOME_VFS_URI_HIDE_NONE );
@@ -613,7 +626,7 @@ gboolean screem_site_load( ScreemSite *site )
 {
 	gchar *project_file;
 	const gchar *pathname;
-	gchar *mime_type;
+	const gchar *mime_type;
 
 	GList *project_files = NULL;
 	GList *file;
@@ -658,7 +671,6 @@ gboolean screem_site_load( ScreemSite *site )
 						g_strdup( info->name ) );
 		}
 		g_free( name );
-		g_free( mime_type );
 	}
 	gnome_vfs_directory_close( handle );
 	
@@ -1194,7 +1206,7 @@ static void screem_site_foreach_page_real( gpointer key,
 
 	if( ! SCREEM_IS_PAGE( value ) ) {
 		g_print( "INVALID ENTRY IN PAGE TABLE: %p\t%s\n",
-				value, key );
+				value, (const gchar*)key );
 	} else {
 		page = SCREEM_PAGE( value );
 
@@ -1284,9 +1296,9 @@ static void screem_site_file_change_real( ScreemPage *page,
 		 * should we even do this? as we get pages created
 		 * when the file is saved / created in the process
 		 * which led to doing this */
-//		if( strcmp( dest, temp ) ) {
+/*		if( strcmp( dest, temp ) ) {*/
 			screem_page_set_pathname( page, temp );
-//		}
+/*		}*/
 		g_free( temp );
 	}
 	if( screem_page_load( page, NULL ) ) {
@@ -1752,12 +1764,10 @@ static void screem_site_init( ScreemSite *site )
 
 	screem_site_model_initialise( site );
 
-	site->priv->excludes = gtk_list_store_new( 2, 
-						      G_TYPE_STRING,
-						      G_TYPE_BOOLEAN );
-	site->priv->ignores = gtk_list_store_new( 2, 
-						     G_TYPE_STRING,
-						     G_TYPE_BOOLEAN );
+	site->priv->excludes = gtk_list_store_new( 2,
+			G_TYPE_STRING, G_TYPE_BOOLEAN );
+	site->priv->ignores = gtk_list_store_new( 2,
+			G_TYPE_STRING, G_TYPE_BOOLEAN );
 
 	site->priv->pages = g_hash_table_new_full( g_str_hash,
 			g_str_equal,
@@ -1792,7 +1802,6 @@ static void screem_site_set_prop( GObject *object, guint prop_id,
 	ScreemSitePrivate *priv;
 	ScreemIconCache *cache;
 	ScreemSiteModel *model;
-	const gchar *home;
 	
 	site = SCREEM_SITE( object );
 	priv = site->priv;
@@ -1822,11 +1831,6 @@ static void screem_site_set_prop( GObject *object, guint prop_id,
 			screem_file_browser_set_sort_func( model->browser,
 				(GtkTreeIterCompareFunc)screem_site_model_compare_func,
 				site );
-			home = g_get_home_dir();
-			if( ! home ) {
-				home = "/";
-			}
-			screem_file_browser_scan_directory( model->browser, home, -1 );
 			
 			break;
 		case PROP_NAME:
