@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -34,13 +34,13 @@
 
 #include <glade/glade.h>
 
-#include <gtk/gtkliststore.h>
-#include <gtk/gtktextbuffer.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkentry.h>
+#include <gtk/gtk.h>
+
+#include <gconf/gconf-client.h>
+
+#ifdef HAVE_ENCHANT
+#include <enchant/enchant.h>
+#endif
 
 #include "screem-spell.h"
 #include "screem-page.h"
@@ -63,12 +63,23 @@ struct ScreemSpellPrivate {
 	GList *words;
 
 	GtkListStore *store;
-
+#ifdef HAVE_ENCHANT
+	EnchantBroker *broker;
+	EnchantDict *dict;
+	EnchantDict *pwl;
+#else
 	/* pid of ispell process */
 	gint pid;
 
 	gint inpipe[ 2 ];
 	gint outpipe[ 2 ];
+#endif
+
+	GConfClient *client;
+	gboolean ignore_tags;
+	gboolean ignore_uppercase;
+	gboolean ignore_numbers;
+	GSList *notifies;
 };
 
 static const gchar *ispell_command = "ispell";
@@ -115,10 +126,68 @@ gboolean screem_spell_check_word( ScreemSpell *spell, const gchar *word )
 	return screem_spell_check( spell, word, NULL );
 }
 
+
+static void dict_describe_cb( const char * const lang_tag,
+		const char * const provider_name,
+		const char * const provider_desc,
+		const char * const provider_file,
+		void * user_data )
+{
+	GtkListStore *store;
+	GtkTreeIter tit;
+
+	store = GTK_LIST_STORE( user_data );
+
+	gtk_list_store_insert_with_values( store, &tit, G_MAXINT,
+			0, lang_tag, -1 );
+}
+
+static void screem_spell_ignore_tag_toggle( GtkToggleButton *button,
+		ScreemSpell *spell )
+{
+	ScreemSpellPrivate *priv;
+	gboolean flag;
+
+	priv = spell->priv;
+	flag = gtk_toggle_button_get_active( button );
+	
+	gconf_client_set_bool( priv->client,
+			"/apps/screem/spell/ignore_tags", flag,
+			NULL );
+}
+
+static void screem_spell_ignore_uppercase_toggle( GtkToggleButton *button,
+		ScreemSpell *spell )
+{
+	ScreemSpellPrivate *priv;
+	gboolean flag;
+
+	priv = spell->priv;
+	flag = gtk_toggle_button_get_active( button );
+	
+	gconf_client_set_bool( priv->client,
+			"/apps/screem/spell/ignore_uppercase", flag,
+			NULL );
+}
+
+static void screem_spell_ignore_numbers_toggle( GtkToggleButton *button,
+		ScreemSpell *spell )
+{
+	ScreemSpellPrivate *priv;
+	gboolean flag;
+
+	priv = spell->priv;
+	flag = gtk_toggle_button_get_active( button );
+	
+	gconf_client_set_bool( priv->client,
+			"/apps/screem/spell/ignore_numbers", flag,
+			NULL );
+}
+
 void screem_spell_check_interactive( ScreemSpell *spell )
 {
+	ScreemSpellPrivate *priv;
 	GladeXML *xml;
-	gchar *gladepath;
 	GtkWidget *widget;
 	gpointer data;
 	GList *list;
@@ -132,24 +201,29 @@ void screem_spell_check_interactive( ScreemSpell *spell )
 	GtkWidget *dialog;
 	gint offset;
 
+	GtkListStore *store;
+	
+	g_return_if_fail( SCREEM_IS_SPELL( spell ) );
+	
 	data = g_object_get_data( G_OBJECT( spell ), "page" );
 	
 	if( ! data )
 		return;
 	
+	priv = spell->priv;
+	
 	page = SCREEM_PAGE( data );
 	tbuffer = GTK_TEXT_BUFFER( page );
 	text = screem_page_get_data( page );
 	screem_spell_split_text( spell, text );
 	g_free( text );
 
-	gladepath = screem_get_glade_path();
-	xml = glade_xml_new( gladepath, "spellcheck_dialog", NULL );
-	g_free( gladepath );
-
+	xml = glade_xml_new( GLADE_PATH"/screem.glade",
+			"spellcheck_dialog", NULL );
+	
 	widget = glade_xml_get_widget( xml, "spellaltlist" );
 	gtk_tree_view_set_model( GTK_TREE_VIEW( widget ),
-				 GTK_TREE_MODEL( spell->private->store ) );
+				 GTK_TREE_MODEL( spell->priv->store ) );
 	renderer = gtk_cell_renderer_text_new();
 	col = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_title( col, "Alternatives" );
@@ -157,17 +231,48 @@ void screem_spell_check_interactive( ScreemSpell *spell )
 	gtk_tree_view_append_column( GTK_TREE_VIEW( widget ), col );
 	gtk_tree_view_column_set_attributes( col, renderer, "text", 0, NULL );
 
+	/* config tab */
 
+	/* ignore */
+	widget = glade_xml_get_widget( xml, "ignore_markup" );
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
+			priv->ignore_tags );
+	g_signal_connect( G_OBJECT( widget ), "toggled",
+			G_CALLBACK( screem_spell_ignore_tag_toggle ),
+			spell );
+	
+	widget = glade_xml_get_widget( xml, "ignore_upper" );
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
+			priv->ignore_uppercase );
+	g_signal_connect( G_OBJECT( widget ), "toggled",
+			G_CALLBACK( screem_spell_ignore_uppercase_toggle ),
+			spell );
+	
+	widget = glade_xml_get_widget( xml, "ignore_numbers" );
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
+			priv->ignore_numbers );
+	g_signal_connect( G_OBJECT( widget ), "toggled",
+			G_CALLBACK( screem_spell_ignore_numbers_toggle ),
+			spell );
+
+	/* dictionary */
+	widget = glade_xml_get_widget( xml, "default_dict" );
+	gtk_combo_box_set_active( GTK_COMBO_BOX( widget ), 0 );
+	store = GTK_LIST_STORE( gtk_combo_box_get_model( GTK_COMBO_BOX( widget ) ) );
+#ifdef HAVE_ENCHANT
+	enchant_broker_list_dicts( priv->broker,
+			dict_describe_cb, store );
+#endif
 	dialog = glade_xml_get_widget( xml, "spellcheck_dialog" );
-
+	
 	glade_xml_signal_autoconnect( xml );
 
 	offset = 0;
-	list = spell->private->words;
+	list = spell->priv->words;
 	while( dialog ) {
 		gchar *buffer;
-		GList *alts = NULL;
-		GList *alist;
+		GSList *alts = NULL;
+		GSList *alist;
 		gchar c;
 		gchar *curword;
 		gboolean replace = FALSE;
@@ -178,16 +283,32 @@ void screem_spell_check_interactive( ScreemSpell *spell )
 			list = list->next;
 		} else {
 			GtkWidget *notebook;
-			notebook = glade_xml_get_widget( xml, "notebook3" );
+			notebook = glade_xml_get_widget( xml, "check_table" );
 			gtk_widget_set_sensitive( notebook, FALSE );
 			word = NULL;
 			gtk_dialog_run( GTK_DIALOG( dialog ) );
 			gtk_widget_destroy( dialog );
 			dialog = NULL;
 		}
+	
+		if( ! word ) {
+			continue;
+		}
 		
-		if( word && 
-		    ! screem_spell_check( spell, word->word, &buffer ) ) {
+		if( ! screem_spell_check( spell, word->word, &buffer ) ) {
+#ifdef HAVE_ENCHANT
+			char **suggestions;
+			size_t num;
+			
+			suggestions = enchant_dict_suggest( priv->dict,
+					word->word, 
+					strlen( word->word ),
+					&num );
+			while( num > 0 ) {
+				num --;
+				alts = g_slist_prepend( alts, suggestions[ num ] );
+			}
+#else
 			/* buffer format:
 			   <symbol> <word> <number> <word number>: <word>, ...
 			*/
@@ -210,25 +331,30 @@ void screem_spell_check_interactive( ScreemSpell *spell )
 						if( temp )
 							*temp = '\0';
 					}
-					alts = g_list_append( alts, curword );
+					alts = g_slist_append( alts, curword );
 					num --;
 				}
 			}
+#endif
 			/* fill in entries in dialog */
 			widget = glade_xml_get_widget( xml, "spellword" );
 			gtk_entry_set_text( GTK_ENTRY( widget ), word->word );
-			gtk_list_store_clear( spell->private->store );
+			gtk_list_store_clear( spell->priv->store );
 			for( alist = alts; alist; alist = alist->next ) {
 				GtkTreeIter it;
-				gtk_list_store_append( spell->private->store,
+				gtk_list_store_append( spell->priv->store,
 						       &it );
-				gtk_list_store_set( spell->private->store,
+				gtk_list_store_set( spell->priv->store,
 						    &it,
 						    0, alist->data,
 						    1, spell,
 						    -1 );
 			}
-			g_list_free( alts );
+#ifdef HAVE_ENCHANT
+			g_slist_foreach( alts, (GFunc)g_free, NULL );
+			g_free( suggestions );				
+#endif
+			g_slist_free( alts );
 			/* highlight the word */
 			g_signal_emit( G_OBJECT( spell ),
 				       screem_spell_signals[ HIGHLIGHT ], 0,
@@ -263,15 +389,26 @@ void screem_spell_check_interactive( ScreemSpell *spell )
 				break;
 			curword = (gchar*)
 				gtk_entry_get_text( GTK_ENTRY( widget ) );
+
+			/* add to dict */
 			if( c != '\0' ) {
+#ifdef HAVE_ENCHANT
+				if( c == '@' ) {
+					enchant_dict_add_to_session( priv->dict, curword, strlen( curword ) );
+				} else {
+					enchant_dict_add_to_pwl( priv->pwl, curword, strlen( curword ) );
+				}
+#else
 				/* FIXME: remove command characters from
 				   curword */
 				gchar *temp = g_strdup_printf( "%c%s\n",
 							       c, curword );
-				write( spell->private->outpipe[ 1 ],
+				write( spell->priv->outpipe[ 1 ],
 				       temp, strlen( temp ) );
 				g_free( temp );
+#endif
 			}
+			
 			if( replace ) {
 				gint len = strlen( word->word );
 
@@ -386,83 +523,132 @@ static void screem_spell_word_destroy( ScreemSpellWord *word )
    in an <img> tag */
 static void screem_spell_split_text( ScreemSpell *spell, const gchar *text )
 {
+	ScreemSpellPrivate *priv;
 	GList *list;
 	gint len;
 	const gchar *start;
-	gchar c;
-	gint i;
 	ScreemSpellWord *w;
-
-	g_list_foreach( spell->private->words,
+	gchar *word;
+	gint wordlen;
+	gunichar c;
+	const gchar *tstart;
+	gboolean containsnum;
+	gboolean allupper;
+	gboolean newword;
+	
+	g_return_if_fail( SCREEM_IS_SPELL( spell ) );
+	
+	priv = spell->priv;
+	
+	g_list_foreach( spell->priv->words,
 			(GFunc)screem_spell_word_destroy,
 			NULL );
-	g_list_free( spell->private->words );
-	spell->private->words = NULL;
+	g_list_free( spell->priv->words );
+	spell->priv->words = NULL;
 
 	list = NULL;
 	len = strlen( text );
 
-	for( i = 0, start = text; i < len; ++ i ) {
-		c = *( text + i );
-		if( isspace( c ) || ( ispunct( c ) && c != '\'' ) ) {
-			if( start != text + i ) {
-				/* we have come to the end of a word */
-				gchar *word;
-				gint wordlen;
+	c = ' ';
+	tstart = text;
+	containsnum = FALSE;
+	allupper = FALSE;
+	for( start = text; c != '\0'; text = g_utf8_next_char( text ) ) {
+		c = g_utf8_get_char( text );
 
-				wordlen = text + i - start;
+		newword = ( g_unichar_isspace( c ) ||
+			( g_unichar_ispunct( c ) && c != '\'' ) );
+	
+		if( start == text ) {
+			allupper = g_unichar_isupper( c );
+		} else if( ! newword ) {
+			allupper &= g_unichar_isupper( c );
+		}
+		if( g_unichar_isdigit( c ) ) {
+			containsnum = TRUE;
+		}
+
+		if( newword ) {
+			word = NULL;
+			if( start != text ) {
+				wordlen = text - start;
 				word = g_strndup( start, wordlen );
-				w = screem_spell_word_new( word, 
-							   start - text );
+			}
+			if( word && priv->ignore_numbers && containsnum ) {
 				g_free( word );
-				list = g_list_append( list, w );
+				word = NULL;
 			}
-			start = text + i + 1;
+			if( word && priv->ignore_uppercase && allupper ) {
+				g_free( word );
+				word = NULL;
+			}
+			if( word ) {
+				w = screem_spell_word_new( word, 
+						start - tstart );
+				list = g_list_prepend( list, w );
+				g_free( word );
+			}
+			start = g_utf8_next_char( text );
+			containsnum = FALSE;
 		}
-
-		/* stopped at the start of a tag, advance forward until we
-		   hit the end of the tag */
-		if( c == '<' ) {
-			while( ( c != '>' ) && ( i < len ) ) {
-				i ++;
-				c = *( text + i );
+		if( priv->ignore_tags ) {
+			if( c == '<' ) {
+				while( c != '>' && c != '\0' ) {
+					text = g_utf8_next_char( text );
+					c = g_utf8_get_char( text );
+				}
+				start = g_utf8_next_char( text );
+				containsnum = FALSE;
+			} else if( c == '&' ) {
+				/* we stopped at a char encoding element */
+				while( c != '>' && c != '\0' ) {
+					text = g_utf8_next_char( text );
+					c = g_utf8_get_char( text );
+				}
+				start = g_utf8_next_char( text );
+				containsnum = FALSE;
 			}
-			start = text + i + 1;
-		} else if( c == '&' ) {
-			/* we stopped at a char encoding element */
-			while( ( c != ';' ) && ( i < len ) ) {
-				i ++;
-				c = *( text + i );
-			}
-			start = text + i + 1;
 		}
-
 	}
-	spell->private->words = list;
+	spell->priv->words = g_list_reverse( list );
 }
 
 static gboolean screem_spell_check( ScreemSpell *spell,
 				    const gchar *word,
 				    gchar **buffer )
 {
+	ScreemSpellPrivate *priv;
+	gboolean success = FALSE;
+#ifndef HAVE_ENCHANT
 	gchar *temp;
 	gint size;
 	gchar buf[ BUFSIZ ];
-	gboolean success = FALSE;
+#endif
 
+	g_return_val_if_fail( SCREEM_IS_SPELL( spell ), TRUE );
+	
+	priv = spell->priv;
+#ifdef HAVE_ENCHANT
+
+	success = ! enchant_dict_check( priv->dict, word, strlen( word ) );
+	if( ! success ) {
+		success = ! enchant_dict_check( priv->pwl, word,
+				strlen( word ) );
+	}
+#else
 	if( buffer )
 		*buffer = NULL;
 
 	temp = g_strconcat( "^", word, "\n", NULL );
 
-	if( write( spell->private->outpipe[ 1 ], temp, strlen( temp ) ) < 0 ) {
+	if( write( spell->priv->outpipe[ 1 ], temp, strlen( temp ) ) < 0 ) {
 		g_free( temp );
 		return success;
 	}
 
 	g_free( temp );
 
-	if( ( size = read( spell->private->inpipe[ 0 ], buf, BUFSIZ ) ) < 0 ) {
+	if( ( size = read( spell->priv->inpipe[ 0 ], buf, BUFSIZ ) ) < 0 ) {
 		return success;
 	}
 
@@ -500,10 +686,61 @@ static gboolean screem_spell_check( ScreemSpell *spell,
 		success = TRUE;
 		break;
 	}
+#endif
 
 	return success;
 }
 
+static void screem_spell_ignore_tags_notify( GConfClient *client,
+		guint cnxn_id, GConfEntry *entry, gpointer data )
+{
+	ScreemSpell *spell;
+	ScreemSpellPrivate *priv;
+	gboolean value;
+	
+	spell = SCREEM_SPELL( data );
+	priv = spell->priv;
+
+	if( entry->value && entry->value->type == GCONF_VALUE_BOOL ) {
+		value = gconf_value_get_bool( entry->value );
+		priv->ignore_tags = value;
+	}
+}
+
+static void screem_spell_ignore_uppercase_notify( GConfClient *client,
+		guint cnxn_id, GConfEntry *entry, gpointer data )
+{
+	ScreemSpell *spell;
+	ScreemSpellPrivate *priv;
+	gboolean value;
+	
+	spell = SCREEM_SPELL( data );
+	priv = spell->priv;
+
+	if( entry->value && entry->value->type == GCONF_VALUE_BOOL ) {
+		value = gconf_value_get_bool( entry->value );
+		priv->ignore_uppercase = value;
+	}
+}
+
+static void screem_spell_ignore_numbers_notify( GConfClient *client,
+		guint cnxn_id, GConfEntry *entry, gpointer data )
+{
+	ScreemSpell *spell;
+	ScreemSpellPrivate *priv;
+	gboolean value;
+	
+	spell = SCREEM_SPELL( data );
+	priv = spell->priv;
+
+	if( entry->value && entry->value->type == GCONF_VALUE_BOOL ) {
+		value = gconf_value_get_bool( entry->value );
+		priv->ignore_numbers = value;
+	}
+}
+
+
+
 /* G Object stuff */
 
 #define PARENT_TYPE G_TYPE_OBJECT
@@ -534,70 +771,155 @@ static void screem_spell_class_init( ScreemSpellClass *klass )
 
 static void screem_spell_init( ScreemSpell *spell )
 {
-	spell->private = g_new0( ScreemSpellPrivate, 1 );
-
-	spell->private->store = gtk_list_store_new( 2, 
-						    G_TYPE_STRING, 
-						    G_TYPE_POINTER, 
-						    NULL );
+	ScreemSpellPrivate *priv;
+	const gchar *lang;
+	const gchar *const *langs;
+	GConfClient *client;
+	gchar *glang;
+	guint handle;
+	
+	priv = spell->priv = g_new0( ScreemSpellPrivate, 1 );
 
+	priv->store = gtk_list_store_new( 2,
+			G_TYPE_STRING, G_TYPE_POINTER, NULL );
+	priv->client = client = gconf_client_get_default();
+	
+#ifdef HAVE_ENCHANT
+	priv->broker = enchant_broker_init();
+	
+	langs = g_get_language_names();
+	if( ! langs ) {
+		lang = "en";
+	} else {
+		for( lang = *langs; lang; lang = *(langs++) ) {
+			if( enchant_broker_dict_exists( priv->broker,
+						lang ) ) {
+				break;
+			}
+		}
+		if( ! lang ) {
+			lang = "en";
+		}
+	}
+	glang = gconf_client_get_string( client,
+			"/apps/screem/spell/dictionary",
+			NULL );
+	if( glang && *glang != '\0' &&
+			enchant_broker_dict_exists( priv->broker,
+						glang ) ) {
+		lang = glang;
+	}
+	
+	priv->dict = enchant_broker_request_dict( priv->broker, lang );
+	priv->pwl = enchant_broker_request_pwl_dict( priv->broker,
+			lang );
+	g_free( glang );
+#else
 	/* spawn an ispell process */
-	pipe( spell->private->inpipe );
-	pipe( spell->private->outpipe );
+	pipe( spell->priv->inpipe );
+	pipe( spell->priv->outpipe );
 
-	spell->private->pid = fork();
+	spell->priv->pid = fork();
 
-	switch( spell->private->pid ) {
+	switch( spell->priv->pid ) {
 	case -1:
 		/* error */
 		break;
 	case 0:
 		/* child process */
 		close( 0 );
-		dup( spell->private->outpipe[ 0 ] );
-		close( spell->private->outpipe[ 1 ] );
+		dup( spell->priv->outpipe[ 0 ] );
+		close( spell->priv->outpipe[ 1 ] );
 		close( 1 );
-		dup( spell->private->inpipe[ 1 ] );
-		close( spell->private->inpipe[ 0 ] );
+		dup( spell->priv->inpipe[ 1 ] );
+		close( spell->priv->inpipe[ 0 ] );
 		execvp( ispell_command, ispell_command_args );
 		_exit( 1 );
 		break;
 	default:
 		/* parent process */
-		close( spell->private->outpipe[ 0 ] );
-		close( spell->private->inpipe[ 1 ] );
+		close( spell->priv->outpipe[ 0 ] );
+		close( spell->priv->inpipe[ 1 ] );
 
 		/* read in the ispell banner */
 		{
 			gchar buffer[ BUFSIZ + 1 ];
-			read( spell->private->inpipe[ 0 ], buffer, BUFSIZ );
+			read( spell->priv->inpipe[ 0 ], buffer, BUFSIZ );
 		}
 		break;
 	}
+#endif
+	priv->ignore_tags = gconf_client_get_bool( client,
+			"/apps/screem/spell/ignore_tags",
+			NULL );	
+	handle = gconf_client_notify_add( client,
+			"/apps/screem/spell/ignore_tags",
+			screem_spell_ignore_tags_notify,
+			spell, NULL, NULL );
+	priv->notifies = g_slist_prepend( priv->notifies,
+			GUINT_TO_POINTER( handle ) );
+
+	priv->ignore_uppercase = gconf_client_get_bool( client,
+			"/apps/screem/spell/ignore_uppercase",
+			NULL );
+	handle = gconf_client_notify_add( client,
+			"/apps/screem/spell/ignore_uppercase",
+			screem_spell_ignore_uppercase_notify,
+			spell, NULL, NULL );
+	priv->notifies = g_slist_prepend( priv->notifies,
+			GUINT_TO_POINTER( handle ) );
+			
+	priv->ignore_numbers = gconf_client_get_bool( client,
+			"/apps/screem/spell/ignore_numbers",
+			NULL );
+	handle = gconf_client_notify_add( client,
+			"/apps/screem/spell/ignore_numbers",
+			screem_spell_ignore_numbers_notify,
+			spell, NULL, NULL );
+	priv->notifies = g_slist_prepend( priv->notifies,
+			GUINT_TO_POINTER( handle ) );
 }
 
 static void screem_spell_finalize( GObject *object )
 {
 	ScreemSpell *spell;
+	ScreemSpellPrivate *priv;
 	gint status;
-
+	GSList *tmp;
+	
 	spell = SCREEM_SPELL( object );
+	priv = spell->priv;
+	
+#ifdef HAVE_ENCHANT
+	enchant_broker_free_dict( priv->broker, priv->dict );
+	enchant_broker_free( priv->broker );
+#else
+	kill( priv->pid, SIGTERM );
+	waitpid( priv->pid, &status, 0 );
+	close( priv->inpipe[ 0 ] );
+	close( priv->outpipe[ 1 ] );
+#endif
 
-	kill( spell->private->pid, SIGTERM );
-	waitpid( spell->private->pid, &status, 0 );
-	close( spell->private->inpipe[ 0 ] );
-	close( spell->private->outpipe[ 1 ] );
-
-	if( spell->private->words ) {
-		g_list_foreach( spell->private->words,
+	for( tmp = priv->notifies; tmp; tmp = tmp->next ) {
+		gconf_client_notify_remove( priv->client,
+				GPOINTER_TO_UINT( tmp->data ) );
+	}
+	g_slist_free( priv->notifies );
+	
+	if( priv->client ) {
+		g_object_unref( G_OBJECT( priv->client ) );
+	}
+	
+	if( spell->priv->words ) {
+		g_list_foreach( spell->priv->words,
 				(GFunc)screem_spell_word_destroy,
 				NULL );
-		g_list_free( spell->private->words );
+		g_list_free( spell->priv->words );
 	}
 
-	g_object_unref( spell->private->store );
+	g_object_unref( spell->priv->store );
 
-	g_free( spell->private );
+	g_free( spell->priv );
 
 	G_OBJECT_CLASS( parent_class )->finalize( object );
 }
