@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -34,6 +34,7 @@
 
 #include "fileops.h"
 #include "pageUI.h"
+#include "support.h"
 #include "screem-tag-inspector.h"
 
 #include "screem-markup.h"
@@ -54,6 +55,14 @@ enum {
 	LAST_SIGNAL
 };
 
+enum {
+	ATTR_MODEL_NAME,
+	ATTR_MODEL_DISPLAY_NAME,
+	ATTR_MODEL_VALUES,
+	ATTR_MODEL_VALUE,
+	ATTR_MODEL_ROWS
+};
+
 enum {
 	STYLE_MODEL_NAME,
 	STYLE_MODEL_FILE,
@@ -65,9 +74,10 @@ static guint screem_tag_inspector_signals[ LAST_SIGNAL ] = { 0 };
 static void dashboard_send_cluepacket( ScreemTagInspector *inspector );
 static void build_attribute_entries( ScreemTagInspector *inspector, 
 			ScreemDTD *dtd, const gchar *name,
-			GtkWidget *box, GSList *attr );
-static void replace_attr( GtkWidget *entry, gchar *attr_name );
-
+			GSList *attr );
+static void attr_edited( GtkCellRendererText *rend,
+		const gchar *path_str, const gchar *txt,
+		ScreemTagInspector *inspector );
 
 static void combo_change( GtkComboBox *combo, gpointer data );
 static void switch_page( GtkNotebook *notebook,
@@ -108,7 +118,8 @@ struct ScreemTagInspectorPrivate {
 
 	gchar *tag;
 	gchar *name;
-	
+
+	GtkTreeModel *attr_model;
 	GtkTreeModel *style_model;
 };
 
@@ -126,9 +137,6 @@ gboolean screem_tag_inspector_inspect( ScreemTagInspector *inspector,
 			ScreemPage *page, guint pos, guint ppos )
 {
 	ScreemTagInspectorPrivate *priv;
-	GtkWidget *box;
-	GtkWidget *sw;
-	GtkBoxChild *bchild;
 	GSList *set_attr;
 	gchar *name;
 
@@ -164,22 +172,23 @@ gboolean screem_tag_inspector_inspect( ScreemTagInspector *inspector,
 	}
 	
 	/* clear old attribute contents */
-	if( GTK_BOX( priv->attrs )->children ) {
-		bchild = GTK_BOX( priv->attrs )->children->data;
-		gtk_container_remove( GTK_CONTAINER( priv->attrs ),
-					bchild->widget );
-	}
+	gtk_list_store_clear( GTK_LIST_STORE( priv->attr_model ) );
 		
 	if( istag ) {
 		/* get current tag */
 		tag = screem_markup_next_tag( text, pos, NULL, 
 						&pos, &name );
-		if( tag ) {
+		if( tag && name ) {
 			priv->tag = tag;
 			priv->name = name;
 			dashboard_send_cluepacket( inspector );
-			set_attr = screem_markup_build_attributes_list( tag, NULL );
-			set_attr = g_slist_reverse( set_attr );
+			set_attr = NULL;
+			if( ( *name != '!' && *name != '?' ) || 
+				! strncmp( "?xml", name,
+					strlen( "?xml" ) ) ) {
+				set_attr = screem_markup_build_attributes_list( tag, NULL );
+				set_attr = g_slist_reverse( set_attr );
+			}
 		} else {
 			name = NULL;
 			set_attr = NULL;
@@ -204,27 +213,13 @@ gboolean screem_tag_inspector_inspect( ScreemTagInspector *inspector,
 			dtd = screem_page_get_dtd( page );
 
 			/* attribute page */
-			sw = gtk_scrolled_window_new( NULL, NULL );
-			gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( sw ),
-	   					GTK_POLICY_AUTOMATIC, 
-	       					GTK_POLICY_AUTOMATIC );
-			gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW( sw ),
-						GTK_SHADOW_IN );
-			box = gtk_vbox_new( FALSE, 0 );
-			gtk_box_set_spacing( GTK_BOX( box ), 6 );
-			gtk_container_set_border_width( GTK_CONTAINER( box ), 6 );
 			build_attribute_entries( inspector, dtd, name,
-						box, set_attr );
+						set_attr );
 			
-			gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW( sw ),
-					box );
 			/* destroy all of set_attr */
 			g_slist_foreach( set_attr, (GFunc)g_free, NULL );
 			g_slist_free( set_attr );
 
-			gtk_box_pack_start_defaults( GTK_BOX( priv->attrs ), sw );
-			gtk_widget_show_all( sw );
-
 			g_free( name );
 		}
 	} else {
@@ -288,30 +283,31 @@ static void dashboard_send_cluepacket( ScreemTagInspector *inspector )
 
 static void build_attribute_entries( ScreemTagInspector *inspector, 
 				ScreemDTD *dtd, const gchar *name,
-				GtkWidget *box, GSList *attr )
+				GSList *attr )
 {
-	GtkWidget *line;
-	GtkWidget *label;
-	GtkWidget *combo;
-	GtkWidget *entry;
-
+	ScreemTagInspectorPrivate *priv;
+	GtkTreeIter it;
 	GSList *list;
-	GtkSizeGroup *group;
 
 	const ScreemDTDElement *element;
 	const GSList *attrs;
+	GtkWidget *label;
 	const gchar *description;
 	GCompareFunc compare;
+	gint i;
+	gchar *tmp;
 
 	if( ! dtd ) {
 		return;
 	}
-
+	
 	element = screem_dtd_valid_element( dtd, name );
 	if( ! element ) {
 		return;
 	}
 
+	priv = inspector->priv;
+	
 	g_object_get( G_OBJECT( dtd ), "compare", &compare, NULL );
 
 	label = inspector->priv->desc_label;
@@ -320,20 +316,25 @@ static void build_attribute_entries( ScreemTagInspector *inspector,
 		description = "";
 	}
 	if( label ) {
-		gtk_label_set_text( GTK_LABEL( label ), description );
+		tmp = screem_strip_extra_space( description );
+		if( tmp ) {
+			gtk_label_set_text( GTK_LABEL( label ), tmp );
+			g_free( tmp );
+		} else {
+			g_print( "failed to strip space from desc\n" );
+		}
 	}
 	
 	attrs = screem_dtd_element_get_attrs( element );
-
-	group = gtk_size_group_new( GTK_SIZE_GROUP_BOTH );
 	
-	for( ;attrs; attrs = attrs->next ) {
+	for( i = 0; attrs; attrs = attrs->next, ++ i ) {
 		ScreemDTDAttribute *att;
 		const gchar *attr_name;
 		const GSList *values;
 		gchar *tmp;
 		GtkTooltips *tip;
-		GtkWidget *ebox;
+		GtkListStore *store;
+		gint j;
 		
 		att = (ScreemDTDAttribute*)attrs->data;
 		attr_name = screem_dtd_attribute_get_name( att );
@@ -341,42 +342,29 @@ static void build_attribute_entries( ScreemTagInspector *inspector,
 		
 		values = screem_dtd_attribute_get_values( att );
 
-		/* add entrys */
-		line = gtk_hbox_new( FALSE, 0 );
-		gtk_box_set_spacing( GTK_BOX( line ), 6 );
+		/* add entrys, FIXME: escape attr_name */
 		if( screem_dtd_attribute_get_required( att ) ) {
 			tmp = g_strconcat( "<b>", attr_name, "</b>", 
 					NULL );
 		} else {
 			tmp = g_strdup( attr_name );
 		}
-		ebox = gtk_event_box_new();
-		gtk_box_pack_start_defaults( GTK_BOX( line ), ebox );
+		store = gtk_list_store_new( 1, G_TYPE_STRING );
+		for( j = 0; values; values = values->next, ++ j ) {
+			gtk_list_store_insert_with_values( store, &it, j,
+					0, values->data, -1 );
+		}
+		gtk_list_store_insert_with_values( GTK_LIST_STORE( priv->attr_model ),
+				&it, i,
+				ATTR_MODEL_NAME, attr_name,
+				ATTR_MODEL_DISPLAY_NAME, tmp,
+				ATTR_MODEL_VALUES, store,
+				-1 );
+		g_free( tmp );
+		g_object_unref( store );
 		
-		label = gtk_label_new( tmp );
-		gtk_label_set_use_markup( GTK_LABEL( label ), TRUE );
-		gtk_label_set_justify( GTK_LABEL( label ), 
-				GTK_JUSTIFY_LEFT );
-		gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.5 );
-		gtk_container_add( GTK_CONTAINER( ebox ), label );
-		gtk_size_group_add_widget( group, label );
-		
-		combo = gtk_combo_box_entry_new_text();
-		gtk_widget_set_size_request( combo, 128, -1 );
-		entry = GTK_BIN( combo )->child;
-		gtk_box_pack_start_defaults( GTK_BOX( line ), combo );
-
-		gtk_box_pack_start( GTK_BOX( box ), line, 
-					FALSE, FALSE, 0 );
 		if( description ) {
 			tip = gtk_tooltips_new();
-			gtk_tooltips_set_tip( tip, ebox,
-					description, "" );
-		}
-		
-		/* add combo entries */
-		for( ; values; values = values->next ) {
-			gtk_combo_box_append_text( GTK_COMBO_BOX( combo ), values->data );
 		}
 		
 		/* 
@@ -391,55 +379,71 @@ static void build_attribute_entries( ScreemTagInspector *inspector,
 				break;
 			list = list->next;
 		}
-		
 		if( ! list ) {
-			gtk_entry_set_text( GTK_ENTRY( entry ), "" );
+			tmp = "";
 		} else if( ! list->next ) {
-			gtk_entry_set_text( GTK_ENTRY( entry ), "1" );
+			tmp = (gchar*)attr_name;
 		} else {
-			gtk_entry_set_text( GTK_ENTRY( entry ),
-					    list->next->data );
+			tmp = list->next->data;
 		}
-
-		g_object_set_data( G_OBJECT( entry ), "inspector", 
-				inspector );
-		g_signal_connect( G_OBJECT( entry ), "activate",
-				  G_CALLBACK( replace_attr ), 
-				  (gpointer)attr_name );
-
-		gtk_widget_show_all( line );
+		gtk_list_store_set( GTK_LIST_STORE( priv->attr_model ),
+				&it,
+				ATTR_MODEL_VALUE, tmp,
+				-1 );
 	}
 }
 
-static void replace_attr( GtkWidget *entry, gchar *attr_name )
+static void attr_edited( GtkCellRendererText *rend,
+		const gchar *path_str, const gchar *txt,
+		ScreemTagInspector *inspector )
 {
-	ScreemTagInspector *inspector;
-	const gchar *text;
+	ScreemTagInspectorPrivate *priv;
+	GtkTreePath *path;
+	GtkTreeIter it;
+	gchar *attr_name;
 	gchar *attr;
+	
+	priv = inspector->priv;
 
-	inspector = g_object_get_data( G_OBJECT( entry ), "inspector" );
+	path = gtk_tree_path_new_from_string( path_str );
+	gtk_tree_model_get_iter( GTK_TREE_MODEL( priv->attr_model ),
+			&it, path );
+	gtk_tree_model_get( GTK_TREE_MODEL( priv->attr_model ),
+			&it, ATTR_MODEL_NAME, &attr_name, -1 );
 
-	text = gtk_entry_get_text( GTK_ENTRY( entry ) );
-	attr = g_strconcat( attr_name, "=\"", text, "\"", NULL );
+	/* let this get updated normally rather than setting here,
+	 * as the insertion will cause the tag inspector to rescan
+	 * the whole tag
+	 
+	 gtk_list_store_set( GTK_LIST_STORE( priv->attr_model ),
+			&it,
+			ATTR_MODEL_VALUE, txt,
+			-1 );
+	*/	
+	gtk_tree_path_free( path );
+
+	attr = g_strconcat( attr_name, "=\"", txt, "\"", NULL );
 
 	g_signal_emit( G_OBJECT( inspector ),
 		screem_tag_inspector_signals[ REPLACE_ATTRIBUTE_SIGNAL],
 		0, attr );
 
 	g_free( attr );
-}									
+	g_free( attr_name );
+}
 
 static void ruleset_to_ui( CRStatement *s, GtkTreeModel *model,
 		const gchar *file )
 {
 	GtkTreeIter it;
-	gchar *txt;
+	guchar *txt;
 
 	txt = cr_selector_to_string( s->kind.ruleset->sel_list );
 	
-	gtk_list_store_append( GTK_LIST_STORE( model ), &it );
-	gtk_list_store_set( GTK_LIST_STORE( model ), &it,
-			STYLE_MODEL_NAME, txt,
+	/* use large pos to ensure append */
+	gtk_list_store_insert_with_values( GTK_LIST_STORE( model ), &it,
+			G_MAXINT32,
+			STYLE_MODEL_NAME, (gchar*)txt,
 			STYLE_MODEL_FILE, file,
 			-1 );
 	g_free( txt );
@@ -473,7 +477,6 @@ static void calc_styles( ScreemTagInspector *inspector,
 	const gchar *pathname;
 	gchar *url;
 	gchar *temp;
-	gchar *local;
 	
 	enum CRStatus status;
 	CROMParser *parser;
@@ -505,30 +508,36 @@ static void calc_styles( ScreemTagInspector *inspector,
 	stylesheet = NULL;
 	
 	if( isdata ) {
-		status = cr_om_parser_parse_buf( parser, sheet,
-				g_utf8_strlen( sheet, -1 ),
+		/* use strlen, we want byte length, not char */
+		status = cr_om_parser_parse_buf( parser, 
+				(const guchar*)sheet,
+				strlen( sheet ),
 				CR_UTF_8, &stylesheet );
+		sheet = "";
 	} else if( ! cpage ) {
-		local = gnome_vfs_get_local_path_from_uri( url );
-		g_free( url );
-		url = local;
+		data = load_file( url, NULL, NULL, NULL );
+
+		status = cr_om_parser_parse_buf( parser, 
+				(const guchar*)data,
+				strlen( data ),
+				CR_UTF_8, &stylesheet );
 		
-		status = cr_om_parser_parse_file( parser, url,
-					CR_ASCII, &stylesheet );
+		g_free( data );
 	} else if( screem_page_load( cpage, NULL ) ){
 		data = screem_page_get_data( cpage );
 		
-		status = cr_om_parser_parse_buf( parser, data,
-				gtk_text_buffer_get_char_count( GTK_TEXT_BUFFER( cpage ) ),
+		/* use strlen, we want byte length, not char */
+		status = cr_om_parser_parse_buf( parser, 
+				(const guchar*)data,
+				strlen( data ),
 				CR_UTF_8, &stylesheet );
+		
+		g_free( data );
 	} else {
 		status = CR_ERROR;
 		/* FIXME: report error loading stylesheet */
 
 	}
-	if( isdata ) {
-		sheet = "";
-	}
 	
 	sel_eng = NULL;
 	num_statements = 0;
@@ -578,6 +587,8 @@ static void style_entries_build( ScreemPage *page,
 	gchar *rel;
 	gchar *sheet;
 	gchar *temp;
+	ScreemDTD *dtd;
+	GCompareFunc compare;
 
 	xmlNodePtr enode;	
 	priv = inspector->priv;
@@ -587,6 +598,15 @@ static void style_entries_build( ScreemPage *page,
 				     0, 0, NULL, 
 				     NULL,
 				     inspector );
+	dtd = screem_page_get_dtd( page );
+	if( dtd ) {
+		g_object_get( G_OBJECT( dtd ), 
+				"compare", &compare, NULL );
+	} else {
+		g_warning( "ScreemPage without ScreemDTD\n" );
+		compare = (GCompareFunc)strcmp;
+	}
+		
 	doc = NULL;
 	if( priv->page == page ) {
 
@@ -595,6 +615,7 @@ static void style_entries_build( ScreemPage *page,
 	}
 	sheet = NULL;
 	gotit = FALSE;
+	model = NULL; /* shutup gcc */
 	if( doc ) {
 		temp = screem_page_query_context( page, priv->pos, 
 				FALSE, FALSE, NULL, NULL, NULL, 
@@ -606,33 +627,31 @@ static void style_entries_build( ScreemPage *page,
 		gtk_tree_path_next( path );
 		gtk_tree_path_next( path );
 		gtk_tree_path_down( path );
-		if( gtk_tree_model_get_iter( model, &it, path ) ) {
-			/* got a link */
-			gotit = TRUE;
-		}
+		gotit = gtk_tree_model_get_iter( model, &it, path );
 		gtk_tree_path_free( path );
 	}
 	/* check all <link> */
-	if( doc && gotit ) {
+	if( gotit ) {
 		gotit = FALSE;
 		do {
 			gtk_tree_model_get( model, &it,
 					SCREEM_PAGE_MODEL_NODE,
 					&node, -1 );
-			if( node && 
-				! g_strcasecmp( "link", node->name ) ) {
+			if( node && ! compare( "link", node->name ) ) {
 				gotit = TRUE;
-				rel = xmlGetProp( node, "rel" );
+				rel = (gchar*)xmlGetProp( node, (const xmlChar*)"rel" );
 				if( ! rel ) {
-					rel = xmlGetProp( node, "REL" );
+					rel = (gchar*)xmlGetProp( node,
+							(const xmlChar*)"REL" );
 				}
 				if( rel &&
 				 ( ! g_strcasecmp( "stylesheet", rel ) ||
 				   ! g_strcasecmp( "alternate stylesheet",
 					   	   rel ) ) ) {
-					sheet = xmlGetProp( node, "href" );
+					sheet = (gchar*)xmlGetProp( node, 
+							(const xmlChar*)"href" );
 					if( ! sheet ) {
-						sheet = xmlGetProp( node, "HREF" );
+						sheet = (gchar*)xmlGetProp( node, (const xmlChar*)"HREF" );
 					}
 					if( sheet && enode ) {
 						calc_styles( inspector,
@@ -656,10 +675,7 @@ static void style_entries_build( ScreemPage *page,
 		gtk_tree_path_next( path );
 		gtk_tree_path_next( path );
 		gtk_tree_path_down( path );
-		if( gtk_tree_model_get_iter( model, &it, path ) ) {
-			/* got a style */
-			gotit = TRUE;
-		}
+		gotit = gtk_tree_model_get_iter( model, &it, path );
 		gtk_tree_path_free( path );
 	}
 	if( gotit ) {
@@ -668,7 +684,7 @@ static void style_entries_build( ScreemPage *page,
 					SCREEM_PAGE_MODEL_NODE,
 					&node, -1 );
 			if( node && node->children ) {
-				sheet = xmlNodeGetContent( node->children );
+				sheet = (gchar*)xmlNodeGetContent( node->children );
 				if( sheet ) {
 					calc_styles( inspector,
 						enode, page,
@@ -696,8 +712,8 @@ static void build_style_entries( ScreemTagInspector *inspector )
 				G_CALLBACK( style_entries_build ),
 				inspector );
 		gdk_threads_leave();
-		screem_page_build_model( page );	
-		gdk_threads_leave();
+		screem_page_build_model( page );
+		gdk_threads_enter();
 		screem_page_ensure_model_built( page );
 	}
 }
@@ -780,9 +796,8 @@ static void style_row_activated( GtkTreeView *view,
 
 
 /* G Object stuff */
-#define PARENT_TYPE GTK_TYPE_BIN
 
-static gpointer parent_class;
+G_DEFINE_TYPE( ScreemTagInspector, screem_tag_inspector, GTK_TYPE_BIN )
 
 static void screem_tag_inspector_class_init( ScreemTagInspectorClass *klass )
 {
@@ -792,7 +807,6 @@ static void screem_tag_inspector_class_init( ScreemTagInspectorClass *klass )
 
 	object_class = G_OBJECT_CLASS( klass );
 	widget_class = (GtkWidgetClass *)klass;
-	parent_class = g_type_class_peek_parent( klass );
 
 	object_class->finalize = screem_tag_inspector_finalize;
 	object_class->get_property = screem_tag_inspector_get_prop;
@@ -868,8 +882,57 @@ static void screem_tag_inspector_init( ScreemTagInspector *tag_inspector )
 	gtk_combo_box_append_text( GTK_COMBO_BOX( combo ), _("Attributes" ) );
 	priv->attrs = child = gtk_vbox_new( FALSE, 0 );
 	gtk_box_set_spacing( GTK_BOX( child ), 6 );
+
+	priv->attr_model = GTK_TREE_MODEL( gtk_list_store_new( ATTR_MODEL_ROWS,
+			G_TYPE_STRING,
+			G_TYPE_STRING,
+			GTK_TYPE_LIST_STORE,
+			G_TYPE_STRING ) );
+	
+	view = gtk_tree_view_new();
+	sw = gtk_scrolled_window_new( NULL, NULL );
+	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( sw ),
+	   				GTK_POLICY_NEVER, 
+	       				GTK_POLICY_AUTOMATIC );
+	gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW( sw ),
+					GTK_SHADOW_IN );
+	gtk_container_add( GTK_CONTAINER( sw ), view );
+	gtk_box_pack_start( GTK_BOX( child ), sw, TRUE, TRUE, 0 );
+
+	rend = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title( col, _( "Name" ) );
+	gtk_tree_view_column_pack_start( col, rend, TRUE );
+	gtk_tree_view_column_set_attributes( col, rend,
+					     "markup", 
+					     ATTR_MODEL_DISPLAY_NAME,
+					     NULL );
+	gtk_tree_view_append_column( GTK_TREE_VIEW( view ), col );
+
+	rend = gtk_cell_renderer_combo_new();
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title( col, _( "Value" ) );
+	gtk_tree_view_column_pack_start( col, rend, TRUE );
+	g_object_set( G_OBJECT( rend ), 
+			"text_column", 0, 
+			"editable", TRUE, 
+			"ellipsize", PANGO_ELLIPSIZE_END,
+			"mode", GTK_CELL_RENDERER_MODE_EDITABLE,
+			NULL );
+	gtk_tree_view_column_set_attributes( col, rend,
+			"text", ATTR_MODEL_VALUE,
+			"model", ATTR_MODEL_VALUES,
+			NULL );
+	g_signal_connect( G_OBJECT( rend ), "edited",
+			G_CALLBACK( attr_edited ), tag_inspector );
+
+	gtk_tree_view_append_column( GTK_TREE_VIEW( view ), col );
+	
+	gtk_tree_view_set_model( GTK_TREE_VIEW( view ),
+			priv->attr_model );
 	gtk_notebook_append_page( GTK_NOTEBOOK( notebook ),
 				child, label );
+
 	/* styles page */
 	label = gtk_label_new( _( "Applied Styles" ) );
 	gtk_combo_box_append_text( GTK_COMBO_BOX( combo ), _( "Applied Styles" ) );
@@ -926,8 +989,9 @@ static void screem_tag_inspector_init( ScreemTagInspector *tag_inspector )
 	gtk_notebook_append_page( GTK_NOTEBOOK( notebook ),
 				child, label );
 	priv->desc_label = label = gtk_label_new( "" );
+	gtk_label_set_line_wrap( GTK_LABEL( label ), TRUE );
 	gtk_misc_set_alignment( GTK_MISC( label ), 0.0, 0.0 );
-	gtk_box_pack_start( GTK_BOX( child ), label, FALSE, FALSE, 0 );
+	gtk_box_pack_start( GTK_BOX( child ), label, TRUE, TRUE, 0 );
 
 	gtk_container_add( GTK_CONTAINER( tag_inspector ), box );
 	gtk_widget_show_all( box );
@@ -946,7 +1010,7 @@ static void screem_tag_inspector_finalize( GObject *screem_tag_inspector )
 
 	g_free( tag_inspector->priv );
 
-	G_OBJECT_CLASS( parent_class )->finalize( screem_tag_inspector );
+	G_OBJECT_CLASS( screem_tag_inspector_parent_class )->finalize( screem_tag_inspector );
 }
 
 static void screem_tag_inspector_set_prop( GObject *object, 
@@ -1029,27 +1093,3 @@ static void screem_tag_inspector_size_allocate( GtkWidget *widget,
 	}
 }
 
-GType screem_tag_inspector_get_type()
-{
-	static guint type = 0;
-
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemTagInspectorClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_tag_inspector_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemTagInspector ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_tag_inspector_init
-		};
-		
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemTagInspector",
-					       &info, 0 );
-	}
-
-	return type;
-}
