@@ -248,6 +248,8 @@ static void screem_tag_file_finalize( GObject *object )
 	
 	g_object_unref( priv->cache );
 
+	g_object_unref( priv->model );
+	
 	g_free( priv );
 	
 	G_OBJECT_CLASS( screem_tag_file_parent_class )->finalize( object );
@@ -1220,7 +1222,7 @@ void screem_tag_file_load( ScreemTagFile *tagfile,
 		ScreemTagFileFormat format )
 {
 	ScreemTagFilePrivate *priv;
-	gchar *type;
+	const gchar *type;
 	
 	g_return_if_fail( SCREEM_IS_TAGFILE( tagfile ) );
 
@@ -1241,10 +1243,8 @@ void screem_tag_file_load( ScreemTagFile *tagfile,
 			 * we support those files */
 			format = SCREEM_TAG_TREE_FORMAT;
 		} else {
-			g_free( type );
 			return;
 		}
-		g_free( type );
 	}
 	priv->format = format;
 
