@@ -696,7 +696,7 @@ static gboolean screem_tag_tree_set_model( ScreemTagTree *tree )
 static gint tagtree_filter( const GnomeVFSFileInfo *info )
 {
 	const gchar *mime_type;
-	gchar *type;
+	const gchar *type;
 	gint ret;
 	
 	mime_type = info->mime_type;
@@ -707,7 +707,6 @@ static gint tagtree_filter( const GnomeVFSFileInfo *info )
 	if( type ) {
 		ret = strcmp( "application/x-screem-tag-tree", 
 				type );
-		g_free( type );
 	}
 
 	return ret;
@@ -716,7 +715,7 @@ static gint tagtree_filter( const GnomeVFSFileInfo *info )
 static gint weml_filter( const GnomeVFSFileInfo *info )
 {
 	const gchar *mime_type;
-	gchar *type;
+	const gchar *type;
 	gint ret;
 	
 	mime_type = info->mime_type;
@@ -727,7 +726,6 @@ static gint weml_filter( const GnomeVFSFileInfo *info )
 	if( type ) {
 		ret = strcmp( "application/x-weml+xml", 
 				type );
-		g_free( type );
 	}
 
 	return ret;
