@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -176,7 +176,6 @@ void screem_todo_view( ScreemTodo *todo )
 {
 	ScreemTodoPrivate *private;
 	ScreemSite *site;
-	gchar *gladepath;
 
 	private = todo->private;
 
@@ -192,9 +191,8 @@ void screem_todo_view( ScreemTodo *todo )
 			"Description", NULL
 		};
 
-		gladepath = screem_get_glade_path();
-		private->xml = glade_xml_new( gladepath, "todo_dialog", NULL );
-		g_free( gladepath );
+		private->xml = glade_xml_new( GLADE_PATH"/screem.glade",
+				"todo_dialog", NULL );
 		
 		private->dialog = glade_xml_get_widget( private->xml,
 							"todo_dialog" );
@@ -311,6 +309,8 @@ void screem_todo_add_todo_item( ScreemTodo *todo, ScreemTodoItem *item )
 			    6, (! item->complete),
 			    7, item->complete,
 			    -1 );
+
+	screem_site_save( todo->private->site );
 }
 
 void screem_todo_add_todo_item_with_values( ScreemTodo *todo,
@@ -392,6 +392,8 @@ static void screem_todo_edited( GtkCellRendererText *cell,
 				    col, new_text, -1 );
 	}
 	gtk_tree_path_free( tpath );
+	
+	screem_site_save( todo->private->site );
 }
 
 /* G Object stuff */
