@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #include <config.h>
@@ -53,15 +53,6 @@ enum {
 };
 
 static guint screem_tree_view_signals[ LAST_SIGNAL ] = { 0 };
-static void screem_tree_view_class_init( ScreemTreeViewClass *klass );
-static void screem_tree_view_init( ScreemTreeView *tree_view );
-static void screem_tree_view_finalize( GObject *tree_view );
-static void screem_tree_view_window_set( ScreemView *view );
-static void screem_tree_view_size_request( GtkWidget *widget, 
-					   GtkRequisition *req );
-static void screem_tree_view_realize( GtkWidget *widget );
-static void screem_tree_view_show( GtkWidget *widget );
-static void screem_tree_view_hide( GtkWidget *widget );
 
 static void screem_tree_view_display( ScreemView *view );
 static void screem_tree_view_page( ScreemTreeView *view, ScreemPage *page );
@@ -123,32 +114,32 @@ static void screem_tree_view_display( ScreemView *view )
 
 static void screem_tree_view_page( ScreemTreeView *view, ScreemPage *page )
 {
-	if( view->private->current == page ) {
+	if( view->priv->current == page ) {
 		return;
 	}
 
-	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( view->private->sw ),
+	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( view->priv->sw ),
 					GTK_POLICY_AUTOMATIC,
                         	        GTK_POLICY_AUTOMATIC );
 
-	if( view->private->timeout ) {
-		g_source_remove( view->private->timeout );
-		view->private->timeout = 0;
+	if( view->priv->timeout ) {
+		g_source_remove( view->priv->timeout );
+		view->priv->timeout = 0;
 	}
 	
-	if( view->private->current ) {
-		g_signal_handlers_disconnect_matched( G_OBJECT( view->private->current ),
+	if( view->priv->current ) {
+		g_signal_handlers_disconnect_matched( G_OBJECT( view->priv->current ),
 						      G_SIGNAL_MATCH_DATA,
 						      0, 0, NULL, NULL, view );
 	}
 	
-	view->private->current = page;
-	if( view->private->built ) {
-		g_source_remove( view->private->built );
-		view->private->built = 0;
+	view->priv->current = page;
+	if( view->priv->built ) {
+		g_source_remove( view->priv->built );
+		view->priv->built = 0;
 	}
 	if( page ) {
-		view->private->model = NULL;
+		view->priv->model = NULL;
 		page_change( page, NULL, view );
 		g_signal_connect( G_OBJECT( page ), "notify::changed",
 				  G_CALLBACK( page_change ), view );
@@ -173,10 +164,10 @@ static void page_change( ScreemPage *page, GParamSpec *spec,
 	status = screem_page_get_changed( page );
 	status |= ( spec == NULL );
 	
-	if( view->private->timeout ) {
-		g_source_remove( view->private->timeout );
+	if( view->priv->timeout ) {
+		g_source_remove( view->priv->timeout );
 	}
-	view->private->timeout = g_timeout_add( 1000, 
+	view->priv->timeout = g_timeout_add( 1000, 
 						(GSourceFunc)build_tree,
 						view );
 }
@@ -187,12 +178,12 @@ static void tree_building( ScreemPage *page, gpointer data )
 	ScreemTreeViewPrivate *priv;
 
 	view = SCREEM_TREE_VIEW( data );
-	priv = view->private;
+	priv = view->priv;
 
 	/* terminate timeout */
-	if( view->private->timeout ) {
-		g_source_remove( view->private->timeout );
-		view->private->timeout = 0;
+	if( view->priv->timeout ) {
+		g_source_remove( view->priv->timeout );
+		view->priv->timeout = 0;
 	}
 
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( priv->sw ),
@@ -217,7 +208,7 @@ static gboolean tree_built_idle( gpointer data )
 	gdk_threads_enter();
 	
 	view = SCREEM_TREE_VIEW( data );
-	priv = view->private;
+	priv = view->priv;
 	page = priv->current;
 	
 	model = screem_page_get_model( page );
@@ -271,7 +262,7 @@ static void tree_built( ScreemPage *page, gpointer data )
 	ScreemTreeViewPrivate *priv;
 
 	view = SCREEM_TREE_VIEW( data );
-	priv = view->private;
+	priv = view->priv;
 
 	if( priv->built ) {
 		g_source_remove( priv->built );
@@ -287,7 +278,7 @@ static gboolean build_tree( ScreemTreeView *view )
 	ScreemTreeViewPrivate *priv;
 	ScreemPage *page;
 	
-	priv = view->private;
+	priv = view->priv;
 
 	/* called via timeout, do not claim the gdk
 	 * lock though, as this will be handled
@@ -296,7 +287,7 @@ static gboolean build_tree( ScreemTreeView *view )
 	page = priv->current;
 	if( page ) {
 		screem_page_build_model( page );
-		view->private->timeout = 0;
+		view->priv->timeout = 0;
 	}
 	
 	return FALSE;
@@ -304,7 +295,7 @@ static gboolean build_tree( ScreemTreeView *view )
 
 static void screem_tree_view_create_mini_view( ScreemTreeView *view )
 {
-	ScreemTreeViewPrivate *private;
+	ScreemTreeViewPrivate *priv;
 	GtkWidget *box;
 	GtkWidget *tree;
 	GtkWidget *sw;
@@ -313,17 +304,17 @@ static void screem_tree_view_create_mini_view( ScreemTreeView *view )
 	GtkTreeViewColumn *col;
 	ScreemWindow *window;
 	
-	private = view->private;
+	priv = view->priv;
 
 	box = gtk_vbox_new( FALSE, 0 );
 	gtk_widget_show( box );
 
-	tree = gtk_tree_view_new_with_model( GTK_TREE_MODEL(private->model) );
+	tree = gtk_tree_view_new_with_model( GTK_TREE_MODEL(priv->model) );
 
-	private->mini = GTK_TREE_VIEW( tree );
+	priv->mini = GTK_TREE_VIEW( tree );
 	gtk_tree_view_set_rules_hint( GTK_TREE_VIEW( tree ), TRUE );
 
-	private->sw = sw = gtk_scrolled_window_new( NULL, NULL );
+	priv->sw = sw = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( sw ),
 					GTK_POLICY_AUTOMATIC,
                                         GTK_POLICY_AUTOMATIC );
@@ -409,7 +400,7 @@ static void screem_tree_view_sel_changed( GtkTreeSelection *sel,
 	guint start;
 	guint end;
 	
-	priv = view->private;
+	priv = view->priv;
 
 	if( gtk_tree_selection_get_selected( sel, &model, &it ) ) {
 		gtk_tree_model_get( model, &it, 
@@ -423,9 +414,15 @@ static void screem_tree_view_sel_changed( GtkTreeSelection *sel,
 }
 
 /* G Object stuff */
-#define PARENT_TYPE SCREEM_TYPE_VIEW
 
-static gpointer parent_class;
+static void screem_view_iface_init( ScreemView *iface );
+
+G_DEFINE_TYPE( ScreemTreeView, screem_tree_view, SCREEM_TYPE_VIEW )
+
+static void screem_tree_view_finalize( GObject *tree_view );
+static void screem_tree_view_show( GtkWidget *widget );
+static void screem_tree_view_hide( GtkWidget *widget );
+static void screem_tree_view_window_set( ScreemView *view );
 
 static void screem_tree_view_class_init( ScreemTreeViewClass *klass )
 {
@@ -434,12 +431,9 @@ static void screem_tree_view_class_init( ScreemTreeViewClass *klass )
 
 	object_class = G_OBJECT_CLASS( klass );
 	widget_class = (GtkWidgetClass *)klass;
-	parent_class = g_type_class_peek_parent( klass );
 
 	object_class->finalize = screem_tree_view_finalize;
 
-	widget_class->realize = screem_tree_view_realize;
-	widget_class->size_request = screem_tree_view_size_request;
 	widget_class->show = screem_tree_view_show;
 	widget_class->hide = screem_tree_view_hide;
 
@@ -466,9 +460,17 @@ static void screem_tree_view_class_init( ScreemTreeViewClass *klass )
 			      G_TYPE_UINT );
 }
 
+static void screem_view_iface_init( ScreemView *iface )
+{
+	/* we only support display */
+
+	iface->display = screem_tree_view_display;
+}
+
+
 static void screem_tree_view_init( ScreemTreeView *tree_view )
 {
-	ScreemTreeViewPrivate *private;
+	ScreemTreeViewPrivate *priv;
 	GtkWidget *sw;
 	GtkCellRenderer *rend;
 	GtkCellRenderer *prend;
@@ -476,11 +478,14 @@ static void screem_tree_view_init( ScreemTreeView *tree_view )
 	GtkTreeViewColumn *col;
 
 	GtkTreeSelection *sel;
-	
-	SCREEM_VIEW( tree_view )->window_set = screem_tree_view_window_set;
-	SCREEM_VIEW( tree_view )->display = screem_tree_view_display;
 
-	private = tree_view->private = g_new0( ScreemTreeViewPrivate, 1 );
+	screem_view_iface_init( SCREEM_VIEW( tree_view ) );
+	
+	g_signal_connect( G_OBJECT( tree_view ), "notify::window",
+			G_CALLBACK( screem_tree_view_window_set ),
+			NULL );
+
+	priv = tree_view->priv = g_new0( ScreemTreeViewPrivate, 1 );
 
 	sw = gtk_scrolled_window_new( NULL, NULL );
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( sw ),
@@ -489,11 +494,11 @@ static void screem_tree_view_init( ScreemTreeView *tree_view )
 	gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW( sw ),
 						GTK_SHADOW_IN );
 
-	private->model = NULL;
+	priv->model = NULL;
 	
-	private->view = GTK_TREE_VIEW(
-	gtk_tree_view_new_with_model( GTK_TREE_MODEL(private->model) ) );
-	gtk_tree_view_set_rules_hint( GTK_TREE_VIEW( private->view ), TRUE );
+	priv->view = GTK_TREE_VIEW(
+	gtk_tree_view_new_with_model( GTK_TREE_MODEL(priv->model) ) );
+	gtk_tree_view_set_rules_hint( GTK_TREE_VIEW( priv->view ), TRUE );
 
 	prend = gtk_cell_renderer_pixbuf_new();
 	rend = gtk_cell_renderer_text_new ();
@@ -510,7 +515,7 @@ static void screem_tree_view_init( ScreemTreeView *tree_view )
 					     SCREEM_PAGE_MODEL_ICON,
 					     NULL );
 	gtk_tree_view_column_set_resizable( col, TRUE );
-	gtk_tree_view_append_column( GTK_TREE_VIEW( private->view ), col );
+	gtk_tree_view_append_column( GTK_TREE_VIEW( priv->view ), col );
 
 	rend = gtk_cell_renderer_text_new ();
 	col = gtk_tree_view_column_new_with_attributes( _( "Text" ),
@@ -519,7 +524,7 @@ static void screem_tree_view_init( ScreemTreeView *tree_view )
 							SCREEM_PAGE_MODEL_TEXT,
 							NULL );
 	gtk_tree_view_column_set_resizable( col, TRUE );
-	gtk_tree_view_append_column( GTK_TREE_VIEW( private->view ), col );
+	gtk_tree_view_append_column( GTK_TREE_VIEW( priv->view ), col );
 
 	rend = gtk_cell_renderer_toggle_new();
 	col = gtk_tree_view_column_new_with_attributes( _( "Valid" ),
@@ -527,17 +532,17 @@ static void screem_tree_view_init( ScreemTreeView *tree_view )
 							"active",
 							SCREEM_PAGE_MODEL_VALID,
 							NULL );
-	gtk_tree_view_append_column( GTK_TREE_VIEW( private->view ), col );
+	gtk_tree_view_append_column( GTK_TREE_VIEW( priv->view ), col );
 	
-	gtk_container_add( GTK_CONTAINER( sw ), GTK_WIDGET( private->view ) );
+	gtk_container_add( GTK_CONTAINER( sw ), GTK_WIDGET( priv->view ) );
 	gtk_container_add( GTK_CONTAINER( tree_view ), sw );
 
-	gtk_widget_show( GTK_WIDGET( private->view ) );
+	gtk_widget_show( GTK_WIDGET( priv->view ) );
 	gtk_widget_show( sw );
 
-	private->timeout = 0;
+	priv->timeout = 0;
 
-	sel = gtk_tree_view_get_selection( GTK_TREE_VIEW( private->view ) );
+	sel = gtk_tree_view_get_selection( GTK_TREE_VIEW( priv->view ) );
 	g_signal_connect( G_OBJECT( sel ), "changed",
 			G_CALLBACK( screem_tree_view_sel_changed ),
 			tree_view );
@@ -546,16 +551,16 @@ static void screem_tree_view_init( ScreemTreeView *tree_view )
 static void screem_tree_view_finalize( GObject *tree_view )
 {
 	ScreemTreeView *view;
-
+	
 	view = SCREEM_TREE_VIEW( tree_view );
 
-	if( view->private->timeout ) {
-		g_source_remove( view->private->timeout );
+	if( view->priv->timeout ) {
+		g_source_remove( view->priv->timeout );
 	}
 
-	g_free( view->private );
+	g_free( view->priv );
 
-	G_OBJECT_CLASS( parent_class )->finalize( tree_view );
+	G_OBJECT_CLASS( screem_tree_view_parent_class )->finalize( tree_view );
 }
 
 static void screem_tree_view_window_set( ScreemView *view )
@@ -564,44 +569,33 @@ static void screem_tree_view_window_set( ScreemView *view )
 	screem_tree_view_create_mini_view( SCREEM_TREE_VIEW( view ) );
 }
 
-static void screem_tree_view_size_request( GtkWidget *widget, 
-					   GtkRequisition *req )
-{
-	GTK_WIDGET_CLASS( parent_class )->size_request( widget, req );
-}
-
-static void screem_tree_view_realize( GtkWidget *widget )
-{
-	GTK_WIDGET_CLASS( parent_class )->realize( widget );
-}
-
 static void screem_tree_view_show( GtkWidget *widget )
 {
 	ScreemTreeView *view;
 	GtkTreePath *path;
 	GtkTreeIter it;
-
+	
 	view = SCREEM_TREE_VIEW( widget );
 
-	GTK_WIDGET_CLASS( parent_class )->show( widget );
+	GTK_WIDGET_CLASS( screem_tree_view_parent_class )->show( widget );
 	
-	gtk_tree_view_set_model( view->private->view, 
-				 view->private->model );
+	gtk_tree_view_set_model( view->priv->view, 
+				 view->priv->model );
 
-	gtk_widget_set_sensitive( GTK_WIDGET( view->private->mini ), FALSE );
+	gtk_widget_set_sensitive( GTK_WIDGET( view->priv->mini ), FALSE );
 	
-	gtk_widget_show( GTK_WIDGET( view->private->view ) );
-	gtk_tree_view_expand_all( GTK_TREE_VIEW( view->private->view ) );
+	gtk_widget_show( GTK_WIDGET( view->priv->view ) );
+	gtk_tree_view_expand_all( GTK_TREE_VIEW( view->priv->view ) );
 	path = gtk_tree_path_new_first();
 	gtk_tree_path_next( path );
-	if( view->private->model &&
-		screem_page_is_markup( view->private->current ) &&  
-		gtk_tree_model_get_iter( view->private->model, 
+	if( view->priv->model &&
+		screem_page_is_markup( view->priv->current ) &&  
+		gtk_tree_model_get_iter( view->priv->model, 
 					&it, path ) ) {
-		gtk_tree_view_collapse_row( GTK_TREE_VIEW( view->private->view ),
+		gtk_tree_view_collapse_row( GTK_TREE_VIEW( view->priv->view ),
 						path );
 		gtk_tree_path_next( path );
-		gtk_tree_view_collapse_row( GTK_TREE_VIEW( view->private->view ),
+		gtk_tree_view_collapse_row( GTK_TREE_VIEW( view->priv->view ),
 						path );
 	}
 	gtk_tree_path_free( path );
@@ -610,39 +604,14 @@ static void screem_tree_view_show( GtkWidget *widget )
 static void screem_tree_view_hide( GtkWidget *widget )
 {
 	ScreemTreeView *view;
-
+	
 	view = SCREEM_TREE_VIEW( widget );
 
-	GTK_WIDGET_CLASS( parent_class )->hide( widget );
+	GTK_WIDGET_CLASS( screem_tree_view_parent_class )->hide( widget );
 
-	gtk_widget_set_sensitive( GTK_WIDGET( view->private->mini ), TRUE );
+	gtk_widget_set_sensitive( GTK_WIDGET( view->priv->mini ), TRUE );
 	
-	gtk_widget_hide( GTK_WIDGET( view->private->view ) );
-	gtk_tree_view_set_model( view->private->view, NULL );
-}
-
-GType screem_tree_view_get_type()
-{
-	static guint type = 0;
-
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemTreeViewClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_tree_view_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemTreeView ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_tree_view_init
-		};
-		
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemTreeView",
-					       &info, 0 );
-	}
-
-	return type;
+	gtk_widget_hide( GTK_WIDGET( view->priv->view ) );
+	gtk_tree_view_set_model( view->priv->view, NULL );
 }
 
