@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  *  For contact information with the author of this source code please see
  *  the AUTHORS file.  If there is no AUTHORS file present then check the
@@ -42,19 +42,6 @@ enum {
 	PROP_APP
 };
 
-static void screem_view_class_init( ScreemViewClass *klass );
-static void screem_view_init( ScreemView *view );
-static void screem_view_finalize( GObject *view );
-static void screem_view_set_prop( GObject *object, guint property_id,
-				  const GValue *value, 
-				  GParamSpec *pspec );
-static void screem_view_get_prop( GObject *object, guint property_id,
-				  GValue *value, GParamSpec *pspec );
-static void screem_view_size_request( GtkWidget *widget,
-                           		GtkRequisition *requisition );
-static void screem_view_size_allocate( GtkWidget *widget,
-                              		GtkAllocation *allocation );
-
 struct ScreemViewPrivate {
 	ScreemWindow *window;
 
@@ -63,66 +50,247 @@ struct ScreemViewPrivate {
 	gboolean online;
 };
 
+
+/* slight GCCism here with args */
+
+#define SCREEM_CALL_IFACE( view, func, args... ) \
+	if( view->func != NULL ) \
+		view->func( view, ## args ) 
+
+#define SET_ACTION( name, func ) \
+	action = gtk_action_group_get_action( group, name );\
+	if( action ) { \
+		gboolean active;\
+		active = ( page != NULL ) && ( view->func != NULL );\
+		gtk_action_set_sensitive( action, active );\
+	}
+
+/* base implementation */
 void screem_view_display( ScreemView *view )
 {
-	if( view->display ) {
-		view->display( view );
+	GtkActionGroup *group;
+	GtkAction *action;
+	ScreemPage *page;
+	
+	g_return_if_fail( SCREEM_IS_VIEW( view ) );
+	
+	SCREEM_CALL_IFACE( view, display );
+
+	/* if this view is visible we need to toggle
+	 * action sensitivity based on what the
+	 * view can do */
+	g_object_get( G_OBJECT( view ), 
+			"actions", &group, "page", &page, NULL );
+	
+	if( group ) {
+		if( GTK_WIDGET_VISIBLE( view ) ) {
+			SET_ACTION( "Print", print );
+			SET_ACTION( "PrintPreview", print );
+
+			SET_ACTION( "Undo", undo );
+			SET_ACTION( "Redo", redo );
+		
+			SET_ACTION( "Cut", cut );
+			SET_ACTION( "Copy", copy );
+			SET_ACTION( "Paste", paste );
+			SET_ACTION( "PasteEncoded", paste_encoded );
+			SET_ACTION( "PasteText", paste_unformatted );
+			SET_ACTION( "Clear", clear_selection );
+			SET_ACTION( "SelectAll", select_region );
+			SET_ACTION( "SelectContext", select_region );
+			SET_ACTION( "SelectContent", select_region );
+		}
+
+		g_object_unref( group );
+		
+		if( page ) {
+			g_object_unref( page );
+		}
 	}
 }
 
 void screem_view_print( ScreemView *view, gboolean preview )
 {
-	if( view->print ) {
-		view->print( view, preview );
-	}
+	SCREEM_CALL_IFACE( view, print, preview );
 }
 
 void screem_view_undo( ScreemView *view )
 {
-	if( view->undo ) {
-		view->undo( view );
-	}
+	SCREEM_CALL_IFACE( view, undo );
 }
 
 void screem_view_redo( ScreemView *view )
 {
-	if( view->redo ) {
-		view->redo( view );
-	}
+	SCREEM_CALL_IFACE( view, redo );
 }
 
-void screem_view_show_message( ScreemView *view, const gchar *msg )
+void screem_view_show_message( ScreemView *view, 
+		const gchar *msg )
 {
-	if( view->private->window ) {
-		screem_window_show_message( view->private->window,
-				msg, FALSE );
+	ScreemView *dview;
+	ScreemViewPrivate *priv;
+	
+	dview = SCREEM_VIEW( view );
+	priv = dview->priv;
+
+	if( priv->window ) {
+		screem_window_show_message( priv->window, msg, FALSE );
 	}
 }
 
 void screem_view_show_error( ScreemView *view, const gchar *msg )
 {
-	if( view->private->window ) {
-		screem_window_show_error( view->private->window,
-				msg, FALSE );
+	ScreemView *dview;
+	ScreemViewPrivate *priv;
+	
+	dview = SCREEM_VIEW( view );
+	priv = dview->priv;
+
+	if( priv->window ) {
+		screem_window_show_error( priv->window, msg, FALSE );
 	}
 }
+	
+/* clipboard */
+void screem_view_cut( ScreemView *view )
+{
+	SCREEM_CALL_IFACE( view, cut );
+}
+
+void screem_view_copy( ScreemView *view )
+{
+	SCREEM_CALL_IFACE( view, copy );
+}
+
+void screem_view_paste( ScreemView *view )
+{
+	SCREEM_CALL_IFACE( view, paste );
+}
+
+void screem_view_paste_unformatted( ScreemView *view )
+{
+	SCREEM_CALL_IFACE( view, paste_unformatted );
+}
+
+void screem_view_paste_encoded( ScreemView *view )
+{
+	SCREEM_CALL_IFACE( view, paste_encoded );
+}
+
+void screem_view_clear_selection( ScreemView *view )
+{
+	SCREEM_CALL_IFACE( view, clear_selection );
+}
+
+gboolean screem_view_has_selection( ScreemView *view, 
+		guint *start, guint *end )
+{
+	gboolean ret;
+	
+	ret = FALSE;
+	if( view->has_selection != NULL ) {
+		ret = view->has_selection( view, start, end );
+	}	
+	
+	return ret;
+}
+
+void screem_view_select_region( ScreemView *view, 
+		guint start, guint end )
+{
+	SCREEM_CALL_IFACE( view, select_region, start, end );
+}
+
+/* text insertion / deletion */
+void screem_view_insert( ScreemView *view, gint pos,
+		const gchar *text )
+{
+	SCREEM_CALL_IFACE( view, insert, pos, text );
+}
+
+void screem_view_insert_markup( ScreemView *view,
+		const gchar *open_element,
+		const gchar *close_element )
+{
+	SCREEM_CALL_IFACE( view, insert_markup, 
+			open_element, close_element );
+}
+
+void screem_view_insert_file( ScreemView *view,
+		const gchar *filename )
+{
+	SCREEM_CALL_IFACE( view, insert_file, filename );
+}
+
+void screem_view_insert_attribute( ScreemView *view,
+		const gchar *attribute )
+{
+	SCREEM_CALL_IFACE( view, insert_attribute, attribute );
+}
+
+void screem_view_delete_forward( ScreemView *view,
+		guint pos, guint len )
+{
+	SCREEM_CALL_IFACE( view, delete_forward, pos, len );
+}
+
+/* position / text functions */
+gchar* screem_view_get_text( ScreemView *view, 
+		guint from, guint len )
+{
+	gchar *ret;
+	
+	ret = NULL;
+	if( view->get_text != NULL ) {
+		ret = view->get_text( view, from, len );
+	}	
+	
+	return ret;
+}
+
+void screem_view_set_pos( ScreemView *view, guint pos )
+{
+	SCREEM_CALL_IFACE( view, set_pos, pos );
+}
+
+guint screem_view_get_pos( ScreemView *view )
+{
+	guint ret;
+	
+	ret = 0;
+	
+	if( view->get_pos != NULL ) {
+		ret = view->get_pos( view ); 
+	}	
+	
+	return ret;
+}
+
+
 
 /* G Object stuff */
-#define PARENT_TYPE GTK_TYPE_BIN
 
-static gpointer parent_class;
+G_DEFINE_TYPE( ScreemView, screem_view, GTK_TYPE_BIN )
 
+static void screem_view_finalize( GObject *view );
+static void screem_view_set_prop( GObject *object, 
+		guint property_id, const GValue *value, 
+		GParamSpec *pspec );
+static void screem_view_get_prop( GObject *object, 
+		guint property_id, GValue *value, GParamSpec *pspec );
+static void screem_view_size_request( GtkWidget *widget,
+		GtkRequisition *requisition );
+static void screem_view_size_allocate( GtkWidget *widget,
+		GtkAllocation *allocation );
+	
 static void screem_view_class_init( ScreemViewClass *klass )
 {
 	GObjectClass *object_class;
 	GtkWidgetClass *widget_class;
-
 	GParamSpec *pspec;
 
 	object_class = G_OBJECT_CLASS( klass );
 	widget_class = (GtkWidgetClass *)klass;
-	parent_class = g_type_class_peek_parent( klass );
-
 
 	object_class->finalize = screem_view_finalize;
 	object_class->get_property = screem_view_get_prop;
@@ -130,78 +298,67 @@ static void screem_view_class_init( ScreemViewClass *klass )
 
 	widget_class->size_request = screem_view_size_request;
 	widget_class->size_allocate = screem_view_size_allocate;
-	
+
+
 	pspec = g_param_spec_pointer( "window", "window",
-				      "window",
-				      G_PARAM_READABLE |
-				      G_PARAM_WRITABLE );
+			"window", G_PARAM_READABLE | G_PARAM_WRITABLE );
 	g_object_class_install_property( G_OBJECT_CLASS( object_class ),
-					 PROP_WINDOW,
-					 pspec );
+			PROP_WINDOW, pspec );
+
 	pspec = g_param_spec_object( "site", "site",
-				     "site",
-				     SCREEM_TYPE_SITE,
-				     G_PARAM_READABLE );
+			"site", SCREEM_TYPE_SITE, G_PARAM_READABLE );
 	g_object_class_install_property( G_OBJECT_CLASS( object_class ),
-					 PROP_SITE,
-					 pspec );
+			PROP_SITE, pspec );
+	
 	pspec = g_param_spec_object( "page", "page",
-				     "page",
-				     SCREEM_TYPE_PAGE,
-				     G_PARAM_READABLE |
-				     G_PARAM_WRITABLE );
+			"page", SCREEM_TYPE_PAGE, 
+			G_PARAM_READABLE | G_PARAM_WRITABLE );
 	g_object_class_install_property( G_OBJECT_CLASS( object_class ),
-					 PROP_PAGE,
-					 pspec );
+			PROP_PAGE, pspec );
+
 	pspec = g_param_spec_boolean( "online", "online",
-				      "online",
-				      FALSE,
-				      G_PARAM_READABLE |
-				      G_PARAM_WRITABLE );
+			"online", FALSE,
+			G_PARAM_READABLE | G_PARAM_WRITABLE );
 	g_object_class_install_property( G_OBJECT_CLASS( object_class ),
-					 PROP_ONLINE,
-					 pspec );
+			PROP_ONLINE, pspec );
+	
 	pspec = g_param_spec_object( "uimanager", "uimanager",
-				     "uimanager",
-				     GTK_TYPE_UI_MANAGER,
-				     G_PARAM_READABLE );
+			"uimanager", GTK_TYPE_UI_MANAGER,
+			G_PARAM_READABLE );
 	g_object_class_install_property( G_OBJECT_CLASS( object_class ),
-					 PROP_UIMANAGER,
-					 pspec );
+			PROP_UIMANAGER, pspec );
+	
 	pspec = g_param_spec_object( "actions", "actions",
-				     "actions",
-				     GTK_TYPE_ACTION_GROUP,
-				     G_PARAM_READABLE );
+			"actions", GTK_TYPE_ACTION_GROUP,
+			G_PARAM_READABLE );
 	g_object_class_install_property( G_OBJECT_CLASS( object_class ),
-					 PROP_ACTIONS,
-					 pspec );
-	pspec = g_param_spec_object( "app", "app",
-				     "app",
-				     SCREEM_TYPE_APPLICATION,
-				     G_PARAM_READABLE );
+			PROP_ACTIONS, pspec );
+	
+	pspec = g_param_spec_object( "app", "app", "app",
+			SCREEM_TYPE_APPLICATION, G_PARAM_READABLE );
 	g_object_class_install_property( G_OBJECT_CLASS( object_class ),
-					 PROP_APP,
-					 pspec );
+			PROP_APP, pspec );
 }
 
 static void screem_view_init( ScreemView *view )
 {
-	view->private = g_new0( ScreemViewPrivate, 1 );
+	view->priv = g_new0( ScreemViewPrivate, 1 );
 }
 
-static void screem_view_finalize( GObject *screem_view )
+static void screem_view_finalize( GObject *sview )
 {
 	ScreemView *view;
+	
+	view = SCREEM_VIEW( sview );
 
-	view = SCREEM_VIEW( screem_view );
+	g_free( view->priv );
 
-	g_free( view->private );
-
-	G_OBJECT_CLASS( parent_class )->finalize( screem_view );
+	G_OBJECT_CLASS( screem_view_parent_class )->finalize( sview );
 }
 
-static void screem_view_set_prop( GObject *object, guint property_id,
-				  const GValue *value, GParamSpec *pspec )
+static void screem_view_set_prop( GObject *object, 
+		guint property_id, const GValue *value, 
+		GParamSpec *pspec )
 {
 	ScreemView *view;
 	
@@ -209,23 +366,20 @@ static void screem_view_set_prop( GObject *object, guint property_id,
 	
 	switch( property_id ) {
 	case PROP_WINDOW:
-		view->private->window = 
+		view->priv->window = 
 			SCREEM_WINDOW( g_value_get_pointer( value ) );
-		if( view->window_set ) {
-			view->window_set( view );
-		}
 		break;
 	case PROP_PAGE:
-		view->private->page = SCREEM_PAGE( g_value_get_object(value) );
+		view->priv->page = SCREEM_PAGE( g_value_get_object(value) );
 		break;
 	case PROP_ONLINE:
-		view->private->online = g_value_get_boolean( value );
+		view->priv->online = g_value_get_boolean( value );
 		break;
 	}
 }
 
-static void screem_view_get_prop( GObject *object, guint property_id,
-				  GValue *value, GParamSpec *pspec)
+static void screem_view_get_prop( GObject *object, 
+		guint property_id, GValue *value, GParamSpec *pspec)
 {
 	ScreemView *view;
 	ScreemWindow *window;
@@ -237,7 +391,7 @@ static void screem_view_get_prop( GObject *object, guint property_id,
 
 	view = SCREEM_VIEW( object );
 
-	window = view->private->window;
+	window = view->priv->window;
 	if( window ) {
 		site = screem_window_get_current( window );
 		page = screem_window_get_document( window );
@@ -246,7 +400,7 @@ static void screem_view_get_prop( GObject *object, guint property_id,
 		app = window->application;
 	} else {
 		site = NULL;
-		page = view->private->page;
+		page = view->priv->page;
 		ui = NULL;
 		group = NULL;
 		app = NULL;
@@ -254,7 +408,7 @@ static void screem_view_get_prop( GObject *object, guint property_id,
 
 	switch( property_id ) {
 	case PROP_WINDOW:
-		g_value_set_pointer( value, view->private->window );
+		g_value_set_pointer( value, view->priv->window );
 		break;
 	case PROP_SITE:
 		g_value_set_object( value, site );
@@ -263,7 +417,7 @@ static void screem_view_get_prop( GObject *object, guint property_id,
 		g_value_set_object( value, page );
 		break;
 	case PROP_ONLINE:
-		g_value_set_boolean( value, view->private->online );
+		g_value_set_boolean( value, view->priv->online );
 		break;
 	case PROP_UIMANAGER:
 		g_value_set_object( value, ui );
@@ -320,27 +474,3 @@ static void screem_view_size_allocate( GtkWidget *widget,
 	}
 }
 
-GType screem_view_get_type()
-{
-	static guint type = 0;
-
-	if( ! type ) {
-		static const GTypeInfo info = {
-			sizeof( ScreemViewClass ),
-			NULL, /* base init */
-			NULL, /* base finalise */
-			(GClassInitFunc)screem_view_class_init,
-			NULL, /* class finalise */
-			NULL, /* class data */
-			sizeof( ScreemView ),
-			0, /* n_preallocs */
-			(GInstanceInitFunc)screem_view_init
-		};
-		
-		type = g_type_register_static( PARENT_TYPE,
-					       "ScreemView",
-					       &info, 0 );
-	}
-
-	return type;
-}
