@@ -96,6 +96,30 @@ extern void screem_marshal_VOID__UINT_UINT_STRING_STRING_STRING_STRING (GClosure
 /* VOID:UINT (./screemmarshal.list:39) */
 #define screem_marshal_VOID__UINT	g_cclosure_marshal_VOID__UINT
 
+/* VOID:STRING,BOOLEAN (./screemmarshal.list:40) */
+extern void screem_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
+                                                 GValue       *return_value,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint,
+                                                 gpointer      marshal_data);
+
+/* VOID:STRING,UINT64,UINT64 (./screemmarshal.list:41) */
+extern void screem_marshal_VOID__STRING_UINT64_UINT64 (GClosure     *closure,
+                                                       GValue       *return_value,
+                                                       guint         n_param_values,
+                                                       const GValue *param_values,
+                                                       gpointer      invocation_hint,
+                                                       gpointer      marshal_data);
+
+/* VOID:STRING,STRING (./screemmarshal.list:42) */
+extern void screem_marshal_VOID__STRING_STRING (GClosure     *closure,
+                                                GValue       *return_value,
+                                                guint         n_param_values,
+                                                const GValue *param_values,
+                                                gpointer      invocation_hint,
+                                                gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __screem_marshal_MARSHAL_H__ */
