@@ -334,6 +334,47 @@ gboolean screem_gtk_list_store_find_mime_type( GtkListStore *store,
 	return ret;
 }
 
+static gint screem_pointer_compare_types( GValue *a, GValue *b )
+{
+	gint ret;
+	
+	g_assert( G_VALUE_TYPE( a ) == G_TYPE_POINTER );
+	g_assert( G_VALUE_TYPE( b ) == G_TYPE_POINTER );
+
+	ret = 0;
+	if( a && b ) {
+		if( a < b ) {
+			ret = -1;
+		} else if( a > b ) {
+			ret = 1;
+		}
+	} else if( a ) {
+		ret = 1;
+	} else {
+		ret = -1;
+	}
+
+	return ret;
+}
+
+gboolean screem_gtk_list_store_find_pointer( GtkListStore *store,
+		GtkTreeIter *iter, guint col, const gpointer value )
+
+{
+	GValue val = { 0 };
+	gboolean ret;
+
+	g_value_init( &val, G_TYPE_POINTER );
+	g_value_set_pointer( &val, value );
+
+	ret = screem_gtk_list_store_find( store, iter, col,
+			&val, 
+			(GCompareFunc)screem_pointer_compare_types );
+	g_value_unset( &val );
+	
+	return ret;
+}
+
 
 gchar *screem_support_charset_convert( const gchar *data, guint len,
 		gchar **data_charset )
@@ -465,12 +506,15 @@ gchar *screem_support_charset_convert_to( const gchar *data,
  * determining that anything starting <?xml  is text/xml
  * no matter what any *.mime files say about the extension,
  * likewise php files can be identified as html */
-gchar *screem_get_mime_type( const gchar *filename, gboolean fast )
+const gchar *screem_get_mime_type( const gchar *filename, 
+		gboolean fast )
 {
 	GnomeVFSFileInfo *info;
 	GnomeVFSFileInfoOptions options;
 	const gchar *type;
-	gchar *ret;
+	const gchar *ret;
+	
+	g_return_val_if_fail( filename != NULL, NULL );
 	
 	options = GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
 		GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
@@ -486,47 +530,69 @@ gchar *screem_get_mime_type( const gchar *filename, gboolean fast )
 	if( gnome_vfs_get_file_info( filename, info,
 				options ) == GNOME_VFS_OK ) {
 		type = info->mime_type;
+	} else {
+		/* failed to get file info, fall back to a safe
+		 * default mime type */
+		type = "application/octet-stream";
 	}
+
 	ret = screem_get_mime_type_override( filename, type );
 	gnome_vfs_file_info_unref( info );
 	
 	return ret;
 }
 
-gchar *screem_get_mime_type_override( const gchar *filename,
-					const gchar *type ) 
+const gchar *screem_get_mime_type_override( const gchar *filename,
+		const gchar *type ) 
 {
+	static GStringChunk *cache = NULL;
 	const gchar *ext;
 	gchar *ret;
 	
+	if( ( ! filename ) && ( ! type ) && cache ) {
+		g_string_chunk_free( cache );
+		return NULL;
+	}
+	if( ! cache ) {
+		cache = g_string_chunk_new( 128 );
+	}
+	
 	ret = NULL;
 	if( type ) {
 		ext = strrchr( filename, '.' );
 		if( ! ext ) {
-			ret = g_strdup( type );
+			ret = g_string_chunk_insert_const( cache, 
+					type );
 		} else if( ! strcmp( ".screem", ext ) ) {
-			ret = g_strdup("application/x-screem");
+			ret = g_string_chunk_insert_const( cache, 
+					"application/x-screem" );
 		} else if( ! strcmp( ".tagtree", ext ) ) { 
-			ret = g_strdup( "application/x-screem-tag-tree" );
+			ret = g_string_chunk_insert_const( cache, 
+					"application/x-screem-tag-tree" );
 		} else if( ! strcmp( ".wml", ext ) ) {
-			ret = g_strdup( "text/vnd.wap.wml" );
+			ret = g_string_chunk_insert_const( cache, 
+					"text/vnd.wap.wml" );
 		} else if( ! strcmp( ".wmls", ext ) ) {
-			ret = g_strdup( "text/vnd.wap.wmlscript" );
+			ret = g_string_chunk_insert_const( cache, 
+					"text/vnd.wap.wmlscript" );
 		} else if( ! strcmp( ".wmlb", ext ) ) {
-			ret = g_strdup( "text/vnd.wap.wmlb" );
+			ret = g_string_chunk_insert_const( cache, 
+					"text/vnd.wap.wmlb" );
 		} else if( ! strcmp( ".wmlc", ext ) ) {
-			ret = g_strdup( "text/vnd.wap.wmlc" );
+			ret = g_string_chunk_insert_const( cache, 
+					"text/vnd.wap.wmlc" );
 		} else if( ! strcmp( ".wmlsc", ext ) ) {
-			ret = g_strdup( "text/vnd.wap.wmlscriptc" );
+			ret = g_string_chunk_insert_const( cache, 
+					"text/vnd.wap.wmlscriptc" );
 		} else if( ! strcmp( ".php", ext ) ) {
-			ret = g_strdup( "application/x-php" );
+			ret = g_string_chunk_insert_const( cache, 
+					"application/x-php" );
 		} else if( ! strcmp( ".phps", ext ) ) {
-			ret = g_strdup( "application/x-php-souce" );
+			ret = g_string_chunk_insert_const( cache, 
+					"application/x-php-souce" );
 		} else {
-			ret = g_strdup( type );
+			ret = g_string_chunk_insert_const( cache, type );
 		}
-	} else if( type ) {
-		ret = g_strdup( type );
 	}
 
 	return ret;
